@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/calma/CalmaRead.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/calma/CalmaRead.c,v 1.2 2009/05/13 15:03:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -324,6 +324,60 @@ calmaParseUnits()
     return (TRUE);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * calmaReadError --
+ *
+ * This procedure is called to print out error messages during
+ * Calma file reading.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      An error message is printed.
+ *
+ * Note:
+ *      You can add more arguments if 10 turns out not to be enough.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+    /*VARARGS1*/
+calmaReadError(format, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10)
+    char *format;
+    char *a1, *a2, *a3, *a4, *a5, *a6, *a7, *a8, *a9, *a10;
+{
+    calmaTotalErrors++;
+    if (CIFWarningLevel == CIF_WARN_NONE) return;
+
+    if ((calmaTotalErrors < 100) || (CIFWarningLevel != CIF_WARN_LIMIT))
+    {   
+
+        if (CIFWarningLevel == CIF_WARN_REDIRECT)
+        {
+            if (calmaErrorFile != NULL)
+            {
+                fprintf(calmaErrorFile, "Error while reading cell \"%s\": ",
+                                cifReadCellDef->cd_name);
+                fprintf(calmaErrorFile, format, a1, a2, a3, a4, a5, a6, a7, 
+                                a8, a9, a10);
+            }
+        }
+        else
+        {
+            TxError("Error while reading cell \"%s\": ", cifReadCellDef->cd_name);
+            TxError(format, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10);
+        }
+    }
+    else if ((calmaTotalErrors == 100) && (CIFWarningLevel == CIF_WARN_LIMIT))
+    {
+        TxError("Error limit set:  Remaining errors will not be reported.\n");
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -359,11 +413,11 @@ calmaUnexpected(wanted, got)
 		fprintf(calmaErrorFile, "but got %s.\n", calmaRecordName(got));
 	    }
 	}
-	else
-	{
-	    TxError("    Expected %s record ", calmaRecordName(wanted));
-	    TxError("but got %s.\n", calmaRecordName(got));
-	}
+	    else
+	    {
+	        TxError("    Expected %s record ", calmaRecordName(wanted));
+	        TxError("but got %s.\n", calmaRecordName(got));
+	    }
     }
 }
 
@@ -416,59 +470,6 @@ calmaRecordName(rtype)
     return (calmaRecordNames[rtype]);
 }
 
-/*
- * ----------------------------------------------------------------------------
- *
- * calmaReadError --
- *
- * This procedure is called to print out error messages during
- * Calma file reading.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	An error message is printed.
- *
- * Note:
- *	You can add more arguments if 10 turns out not to be enough.
- *
- * ----------------------------------------------------------------------------
- */
-
-    /*VARARGS1*/
-calmaReadError(format, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10)
-    char *format;
-    char *a1, *a2, *a3, *a4, *a5, *a6, *a7, *a8, *a9, *a10;
-{
-    calmaTotalErrors++;
-    if (CIFWarningLevel == CIF_WARN_NONE) return;
-
-    if ((calmaTotalErrors < 100) || (CIFWarningLevel != CIF_WARN_LIMIT))
-    {
-
-	if (CIFWarningLevel == CIF_WARN_REDIRECT)
-	{
-	    if (calmaErrorFile != NULL)
-	    {
-		fprintf(calmaErrorFile, "Error while reading cell \"%s\": ",
-				cifReadCellDef->cd_name);
-		fprintf(calmaErrorFile, format, a1, a2, a3, a4, a5, a6, a7,
-				a8, a9, a10);
-	    }
-	}
-	else
-	{
-	    TxError("Error while reading cell \"%s\": ", cifReadCellDef->cd_name);
-	    TxError(format, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10);
-	}
-    }
-    else if ((calmaTotalErrors == 100) && (CIFWarningLevel == CIF_WARN_LIMIT))
-    {
-	TxError("Error limit set:  Remaining errors will not be reported.\n");
-    }
-}
-
 /*
  * ----------------------------------------------------------------------------
  *
