@@ -22,7 +22,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFhier.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFhier.c,v 1.2 2008/12/04 16:21:43 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -240,9 +240,16 @@ cifHierCopyFunc(tile, cxp)
     int dinfo = 0;
 #endif
 
-    /* Ignore tiles in vendor GDS */
+    /* Ignore tiles in vendor GDS, unless this is specifically	*/
+    /* overridden by the "see-vendor" option.			*/
 
-    if (cxp->tc_scx->scx_use->cu_def->cd_flags & CDVENDORGDS) return 0;
+    if (cxp->tc_scx->scx_use->cu_def->cd_flags & CDVENDORGDS)
+    {
+	if (!CIFCurStyle)
+	    return 0;
+	else if (!(CIFCurStyle->cs_flags & CWF_SEE_VENDOR))
+	    return 0;
+    }
 
     /* Ignore space tiles, since they won't do anything anyway. */
 
