@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFrdcl.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFrdcl.c,v 1.2 2009/05/03 15:01:04 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -502,7 +502,7 @@ CIFParseStart()
 void
 CIFPaintCurrent()
 {
-    Plane *plane;
+    Plane *plane, *swapplane;
     int i;
 
     for (i = 0; i < cifCurReadStyle->crs_nLayers; i += 1)
@@ -516,11 +516,21 @@ CIFPaintCurrent()
 	/* Generate a paint/erase table, then paint from the CIF
 	 * plane into the current Magic cell.
 	 */
-
 	type = cifCurReadStyle->crs_layers[i]->crl_magicType;
 
-	(void) DBSrPaintArea((Tile *) NULL, plane, &TiPlaneRect,
-		&CIFSolidBits, cifPaintCurrentFunc, (ClientData) type);
+	if (cifCurReadStyle->crs_layers[i]->crl_flags & CIFR_TEMPLAYER)
+	{
+	    /* Swap planes */
+	    swapplane = cifCurReadPlanes[type];
+	    cifCurReadPlanes[type] = plane;
+	    plane = swapplane;
+	}
+	else
+	{
+	    DBSrPaintArea((Tile *) NULL, plane, &TiPlaneRect,
+			&CIFSolidBits, cifPaintCurrentFunc,
+			(ClientData)type);
+	}
 	
 	/* Recycle the plane, which was dynamically allocated. */
 
