@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFrdtech.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFrdtech.c,v 1.2 2009/05/03 15:01:04 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -680,6 +680,51 @@ CIFReadTechLine(sectionName, argc, argv)
 	return TRUE;
     }
 
+    /* Process templayer lines next.  (templayers in cifinput added 5/3/09) */
+
+    if (strcmp(argv[0], "templayer") == 0)
+    {
+	TileType type;
+
+	cifCurReadLayer = NULL;
+	cifCurReadOp = NULL;
+	if (cifCurReadStyle->crs_nLayers == MAXCIFRLAYERS)
+	{
+	    TechError("Can't handle more than %d layers per style.\n",
+		    MAXCIFRLAYERS);
+	    TechError("Your local Magic wizard can increase the table size.\n");
+	    goto errorReturn;
+	}
+        if ((argc != 2) && (argc != 3)) goto wrongNumArgs;
+	type = CIFReadNameToType(argv[1], TRUE);
+	if (type < 0) goto errorReturn;
+
+	cifCurReadLayer = (CIFReadLayer *) mallocMagic(sizeof(CIFReadLayer));
+	cifCurReadStyle->crs_layers[cifCurReadStyle->crs_nLayers]
+		= cifCurReadLayer;
+	cifCurReadStyle->crs_nLayers += 1;
+	cifCurReadLayer->crl_magicType = type;
+	cifCurReadLayer->crl_ops = NULL;
+	cifCurReadLayer->crl_flags = CIFR_TEMPLAYER | CIFR_SIMPLE;
+
+	/* Handle a special case of a list of layer names on the
+	 * layer line.  Turn them into an OR operation.
+	 */
+	
+	if (argc == 3)
+	{
+	    cifCurReadOp = (CIFOp *) mallocMagic(sizeof(CIFOp));
+	    cifCurReadOp->co_opcode = CIFOP_OR;
+	    cifCurReadOp->co_client = (ClientData)NULL;
+	    CIFParseReadLayers(argv[2], &cifCurReadOp->co_cifMask);
+	    TTMaskZero(&cifCurReadOp->co_paintMask);
+	    cifCurReadOp->co_next = NULL;
+	    cifCurReadOp->co_distance = 0;
+	    cifCurReadLayer->crl_ops = cifCurReadOp;
+	}
+	return TRUE;
+    }
+
     /* Process mapping between CIF layers and calma layers/types */
     if ((strcmp(argv[0], "calma") == 0) || (strncmp(argv[0], "gds", 3) == 0))
     {
