@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFrdutils.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFrdutils.c,v 1.2 2008/06/01 18:37:38 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -834,7 +834,7 @@ path_intersect(pathHead, start, respt)
     bool does_cross = FALSE, diagonal = FALSE;
     int tdist, newdist;
 
-    tdist = INFINITY;
+    tdist = newdist = INFINITY;
     for (path = pathHead; path->cifp_next; path = path->cifp_next)
     {
 	/* don't compare with self */
