@@ -18,7 +18,7 @@
  *     *********************************************************************
  *
  *
- * rcsid "$Header: /usr/cvsroot/magic-8.0/cif/CIFread.h,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $
+ * rcsid "$Header: /usr/cvsroot/magic-8.0/cif/CIFread.h,v 1.2 2009/05/03 15:01:04 tim Exp $
  */
 
 #ifndef _CIFREAD_H
@@ -50,9 +50,12 @@ typedef struct
  *
  * CIFR_SIMPLE: Means this layer is a simple one, coming from only
  *		a single OR operation, so it can be handled specially.
+ * CIFR_TEMPLAYER: Means this layer is a temporary CIF layer, and that
+ *		the "crl_magicType" should be interpreted as a CIF layer.
  */
 
 #define CIFR_SIMPLE 1
+#define CIFR_TEMPLAYER 2
 
 /* The following structure defines a complete CIF read-in style.
  * The constant MAXCIFRLAYERS must be less than TT_MAXTYPES, and
