@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdCD.c,v 1.3 2008/12/03 14:12:08 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdCD.c,v 1.5 2009/09/11 16:09:49 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -3030,7 +3030,7 @@ CmdDrc(w, cmd)
 	"*flatcheck             check box area by flattening",
 	"*halo [d]		limit error checking to areas of d lambda",
 	"*showint radius        show interaction area under box",
-	"*stepsize [d]		change drc step size to d lambda",
+	"*stepsize [d]		change DRC step size to d lambda",
 	"catchup                run checker and wait for it to complete",
 	"check                  recheck area under box in all cells",
 	"count                  count error tiles in each cell under box",
@@ -3165,6 +3165,8 @@ CmdDrc(w, cmd)
 		freeMagic((char *)dcl);
 		dcl = dcl->dcl_next;
 	    }
+	    if ((DRCBackGround != DRC_SET_OFF) && (count_total == -1))
+		count_total = 0;
 	    if (count_total >= 0)
 #ifdef MAGIC_WRAPPER
 	        Tcl_SetObjResult(magicinterp, Tcl_NewIntObj(count_total));
@@ -3281,7 +3283,7 @@ CmdDrc(w, cmd)
 	    break;
 	
 	case DRC_OFF:
-	    DRCBackGround = FALSE;
+	    DRCBackGround = DRC_SET_OFF;
 #ifdef MAGIC_WRAPPER
 	    if (TxInputRedirect != TX_INPUT_REDIRECTED)
 #endif
@@ -3289,14 +3291,18 @@ CmdDrc(w, cmd)
 	    break;
 	
 	case DRC_ON:
-	    DRCBackGround = TRUE;
+	    /* Don't allow overriding of DRC_NOT_SET */
+	    if (DRCBackGround == DRC_SET_OFF) DRCBackGround = DRC_SET_ON;
 	    break;
 
 	case DRC_STATUS:
 #ifdef MAGIC_WRAPPER
-	    Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(DRCBackGround));
+	    Tcl_SetObjResult(magicinterp,
+			Tcl_NewBooleanObj((DRCBackGround == DRC_SET_OFF) ?
+			FALSE : TRUE));
 #else
-	    TxPrintf("DRC checker is %s.\n", (DRCBackGround) ? "on" : "off");
+	    TxPrintf("DRC checker is %s.\n", (DRCBackGround == DRC_SET_OFF) ?
+			"off" : "on");
 #endif
 	    break;
 	
