@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdRS.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdRS.c,v 1.8 2008/12/11 14:11:45 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -33,6 +33,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
+#include "database/fonts.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
 #include "utils/main.h"
@@ -672,16 +673,17 @@ CmdSelect(w, cmd)
 #define SEL_VISIBLE	 1
 #define SEL_CELL	 2
 #define SEL_CLEAR	 3
-#define SEL_HELP	 4
-#define SEL_KEEP	 5
-#define SEL_MOVE	 6
-#define SEL_PICK	 7
-#define SEL_SAVE	 8
-#define SEL_BOX		 9
-#define SEL_CHUNK	10
-#define SEL_REGION	11
-#define SEL_NET		12
-#define SEL_DEFAULT	13
+#define SEL_FLAT	 4
+#define SEL_HELP	 5
+#define SEL_KEEP	 6
+#define SEL_MOVE	 7
+#define SEL_PICK	 8
+#define SEL_SAVE	 9
+#define SEL_BOX		10
+#define SEL_CHUNK	11
+#define SEL_REGION	12
+#define SEL_NET		13
+#define SEL_DEFAULT	14
 
     static char *cmdSelectOption[] =
     {
@@ -689,6 +691,7 @@ CmdSelect(w, cmd)
 	"visible",
 	"cell",
 	"clear",
+	"flat",
 	"help",
 	"keep",
 	"move",
@@ -710,6 +713,7 @@ CmdSelect(w, cmd)
 	"[more | less] visible [layers]  [de]select all visible info under box in layers",
 	"[more | less | top] cell [name] [de]select cell under cursor, or \"name\"",
 	"clear                           clear selection",
+	"flat				 flatten the contents of the selection",
 	"help                            print this message",
 	"keep                            copy the selection to the layout",
 	"move                            move selection to a location in the layout",
@@ -951,6 +955,17 @@ CmdSelect(w, cmd)
 	    DBWHLRedraw(SelectRootDef, &selarea, FALSE);
 	    return;
 
+	/*--------------------------------------------------------------------
+	 * Flatten the contents of the selection cell, but keep the result
+	 * within the selection cell.
+	 *--------------------------------------------------------------------
+	 */
+
+	case SEL_FLAT:
+	    if ((more) || (less) || (cmd->tx_argc > 2)) goto usageError;
+	    SelectFlat();
+	    return;
+
 	/*--------------------------------------------------------------------
 	 * Save the selection as a new Magic cell on disk.
 	 *--------------------------------------------------------------------
@@ -1124,6 +1139,17 @@ Okay:
 		{
 		    samePlace = TRUE;
 		    level = option;
+
+		    if (level != SEL_CHUNK)
+		    {
+			int count = 0;
+			for (type++; count <= DBNumUserLayers; type++, count++)
+			{
+			    if (type >= DBNumUserLayers)
+				type = TT_SELECTBASE;
+			    if (TTMaskHasType(&mask, type)) break;
+			}
+		    }
 		}
 		else
 		{
@@ -1192,6 +1218,7 @@ Okay:
 		}
 		if (level == SEL_NET)
 		    SelectNet(&scx, type, crec->dbw_bitmask, (Rect *) NULL, less);
+
 		return;
 	    }
 
@@ -1318,7 +1345,446 @@ Okay:
 	    return;
     }
 }
-
+
+/*
+ *-----------------------------------------------------------------------
+ * Callback functions used with CmdSetLabel to change individual labels
+ *-----------------------------------------------------------------------
+ */
+
+int
+cmdLabelTextFunc(label, cellUse, transform, text)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    char *text;
+{
+    Label *newlab;
+    CellDef *cellDef = cellUse->cu_def;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    if (text == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewStringObj(label->lab_text, -1));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	TxPrintf("%s\n", label->lab_text);
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	newlab = DBPutFontLabel(cellDef, &label->lab_rect, label->lab_font,
+		label->lab_size, label->lab_rotate, &label->lab_offset,
+		label->lab_just, text, label->lab_type, label->lab_flags);
+	DBEraseLabelsByContent(cellDef, &label->lab_rect, -1, label->lab_text);
+	DBWLabelChanged(cellDef, newlab, DBW_ALLWINDOWS);
+    }
+    return 0;
+}
+
+int
+cmdLabelRotateFunc(label, cellUse, transform, value)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    int *value;
+{
+    CellDef *cellDef = cellUse->cu_def;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    if (value == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewIntObj(label->lab_rotate));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	TxPrintf("%d\n", label->lab_rotate);
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	DBUndoEraseLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+	label->lab_rotate = *value;
+	DBFontLabelSetBBox(label);
+	DBUndoPutLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+    }
+    return 0;
+}
+
+int
+cmdLabelSizeFunc(label, cellUse, transform, value)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    int *value;
+{
+    CellDef *cellDef = cellUse->cu_def;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    if (value == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewDoubleObj((double)label->lab_size / 8.0));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	TxPrintf("%g\n", (double)label->lab_size / 8.0);
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	DBUndoEraseLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+	label->lab_size = *value;
+	DBFontLabelSetBBox(label);
+	DBUndoPutLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+    }
+    return 0;
+}
+
+int
+cmdLabelJustFunc(label, cellUse, transform, value)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    int *value;
+{
+    CellDef *cellDef = cellUse->cu_def;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    if (value == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewStringObj(GeoPosToName(label->lab_just), -1));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	TxPrintf("%s\n", GeoPosToName(label->lab_just));
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	DBUndoEraseLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+	label->lab_just = *value;
+	DBFontLabelSetBBox(label);
+	DBUndoPutLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+    }
+    return 0;
+}
+
+int
+cmdLabelOffsetFunc(label, cellUse, transform, point)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    Point *point;
+{
+    CellDef *cellDef = cellUse->cu_def;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj, *pobj;
+#endif
+
+    if (point == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	pobj = Tcl_NewListObj(0, NULL);
+	Tcl_ListObjAppendElement(magicinterp, lobj, pobj);
+	Tcl_ListObjAppendElement(magicinterp, pobj,
+			Tcl_NewDoubleObj((double)label->lab_offset.p_x / 8.0));
+	Tcl_ListObjAppendElement(magicinterp, pobj,
+			Tcl_NewDoubleObj((double)label->lab_offset.p_y / 8.0));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	TxPrintf("%g %g\n", (double)(label->lab_offset.p_x) / 8.0,
+		(double)(label->lab_offset.p_y) / 8.0);
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	DBUndoEraseLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+	label->lab_offset = *point;
+	DBFontLabelSetBBox(label);
+	DBUndoPutLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+    }
+    return 0;
+}
+
+int
+cmdLabelFontFunc(label, cellUse, transform, font)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    int *font;
+{
+    CellDef *cellDef = cellUse->cu_def;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    if (font == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	if (label->lab_font == -1)
+	    Tcl_ListObjAppendElement(magicinterp, lobj, Tcl_NewStringObj("default", 7));
+	else
+	    Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewStringObj(DBFontList[label->lab_font]->mf_name, -1));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	if (label->lab_font == -1)
+	    TxPrintf("default\n");
+	else
+	    TxPrintf("%s\n", DBFontList[label->lab_font]->mf_name);
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	DBUndoEraseLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+	label->lab_font = *font;
+	if ((*font > -1) && (label->lab_size == 0)) label->lab_size = DBLambda[1];
+	DBFontLabelSetBBox(label);
+	DBUndoPutLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+    }
+    return 0;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CmdSetLabel --
+ *
+ * Implement the "setlabel" command.
+ * Query or change properties of a (selected) label in the edit cell
+ *
+ * Usage:
+ *	setlabel option [name]
+ *
+ *
+ * Option may be one of:
+ *	text
+ *	font
+ *	justify
+ *	size
+ *	offset
+ *	rotate
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Modified label entry.  The "setlabel font" command may load font
+ *	information if the requested font is not already in the font list.
+ *	"setlabel font <name>" can be used without any select to load fonts
+ *	from a startup script.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+#define SETLABEL_TEXT		0
+#define SETLABEL_FONT		1
+#define SETLABEL_FONTLIST	2
+#define SETLABEL_JUSTIFY	3
+#define SETLABEL_SIZE		4
+#define SETLABEL_OFFSET		5
+#define SETLABEL_ROTATE		6
+
+void
+CmdSetLabel(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    int pos = -1, font = -1, size = 0, rotate = 0;
+    Point offset;
+    int option;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    static char *cmdLabelSetOption[] =
+    {
+	"text <text>		change/get label text",
+	"font <name>		change/get label font",
+	"fontlist		list available fonts",
+	"justify <position>	change/get label justification",
+	"size <value>		change/get label size",
+	"offset <x> <y>		change/get label offset",
+	"rotate <degrees>	change/get label rotation",
+	NULL
+    };
+
+    if (cmd->tx_argc < 2 || cmd->tx_argc > 4)
+    {
+	TxError("%s <option> [<value>]\n", cmd->tx_argv[0]);
+	return;
+    }
+
+    option = Lookup(cmd->tx_argv[1], cmdLabelSetOption);
+
+    switch (option)
+    {
+	case SETLABEL_FONTLIST:
+#ifdef MAGIC_WRAPPER
+	    lobj = Tcl_NewListObj(0, NULL);
+	    for (font = 0; font < DBNumFonts; font++)
+		Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewStringObj(DBFontList[font]->mf_name, -1));
+	    Tcl_SetObjResult(magicinterp, lobj);
+#else
+	    for (font = 0; font < DBNumFonts; font++)
+		TxPrintf("%s ", DBFontList[font]->mf_name);
+	    TxPrintf("\n");
+#endif
+	    break;
+	
+	case SETLABEL_TEXT:
+	    if (EditCellUse)
+	    {
+		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+			cmdLabelTextFunc, (cmd->tx_argc == 3) ?
+			(ClientData)cmd->tx_argv[2] : NULL);
+		SelectTransform(&GeoIdentityTransform);
+	    }
+	    break;
+
+	case SETLABEL_FONT:
+	    if (cmd->tx_argc >= 2 && cmd->tx_argc <= 4)
+	    {
+		if ((cmd->tx_argc == 3) && StrIsInt(cmd->tx_argv[2]))
+		{
+		    int font = atoi(cmd->tx_argv[2]);
+		    if (font < -1 || font >= DBNumFonts)
+		    {
+			if (DBNumFonts == 0)
+			    TxError("No vector outline fonts are loaded.\n");
+			else
+			    TxError("Font index out of range (0 to %d)\n",
+					DBNumFonts - 1);
+		    }
+		    else if (font == -1)
+			TxPrintf("default\n");
+		    else
+			TxPrintf("%s\n", DBFontList[font]->mf_name);
+		}
+		else if ((cmd->tx_argc == 3 || cmd->tx_argc == 4) &&
+				!StrIsInt(cmd->tx_argv[2]))
+		{
+		    font = DBNameToFont(cmd->tx_argv[2]);
+		    if (font < -1)
+		    {
+			float scale = 1.0;
+			if ((cmd->tx_argc == 4) && StrIsNumeric(cmd->tx_argv[3]))
+			    scale = (float)atof(cmd->tx_argv[3]);
+			if (DBLoadFont(cmd->tx_argv[2], scale) == 0)
+			    TxPrintf("%s\n", DBFontList[DBNumFonts - 1]->mf_name);
+			font = DBNameToFont(cmd->tx_argv[2]);
+			if (font < -1) break;
+		    }
+		}
+	    
+		if (EditCellUse)
+		{
+		    SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+				cmdLabelFontFunc, (cmd->tx_argc == 3) ?
+				(ClientData)&font : NULL);
+		    SelectTransform(&GeoIdentityTransform);
+		}
+	    }
+	    break;
+
+	case SETLABEL_JUSTIFY:
+	    if (cmd->tx_argc == 3)
+	    {
+		pos = GeoNameToPos(cmd->tx_argv[2], FALSE, TRUE);
+		if (pos < 0) break;
+	    }
+	    if (EditCellUse)
+	    {
+		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+			cmdLabelJustFunc, (cmd->tx_argc == 3) ?
+			(ClientData)&pos : NULL);
+		SelectTransform(&GeoIdentityTransform);
+	    }
+	    break;
+
+	case SETLABEL_SIZE:
+	    if (cmd->tx_argc == 3)
+	    {
+		if (StrIsNumeric(cmd->tx_argv[2]))
+		    size = cmdScaleCoord(w, cmd->tx_argv[2], TRUE, TRUE, 8);
+		if (size <= 0) break;
+	    }
+	    if (EditCellUse)
+	    {
+		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+			cmdLabelSizeFunc, (cmd->tx_argc == 3) ?
+			(ClientData)&size : NULL);
+		SelectTransform(&GeoIdentityTransform);
+	    }
+	    break;
+
+	case SETLABEL_OFFSET:
+	    if (cmd->tx_argc == 3)
+	    {
+		TxError("Usage:  setlabel offset <x> <y>\n");
+		return;
+	    }
+	    else if (cmd->tx_argc == 4)
+	    {
+		offset.p_x = cmdScaleCoord(w, cmd->tx_argv[2], TRUE, TRUE, 8);
+		offset.p_y = cmdScaleCoord(w, cmd->tx_argv[3], TRUE, FALSE, 8);
+	    }	
+	    if (EditCellUse)
+	    {
+		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+			cmdLabelOffsetFunc, (cmd->tx_argc == 4) ?
+			(ClientData)&offset : NULL);
+		SelectTransform(&GeoIdentityTransform);
+	    }
+	    break;
+
+	case SETLABEL_ROTATE:
+	    if (cmd->tx_argc == 3)
+	    {
+		if (StrIsInt(cmd->tx_argv[2]))
+		    rotate = atoi(cmd->tx_argv[2]);
+	    }
+	    if (EditCellUse)
+	    {
+		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+			cmdLabelRotateFunc, (cmd->tx_argc == 3) ?
+			(ClientData)&rotate : NULL);
+		SelectTransform(&GeoIdentityTransform);
+	    }
+	    break;
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1884,32 +2350,24 @@ CmdStretch(w, cmd)
 
 	if (!ToolGetEditBox((Rect *)NULL)) return;
 
-	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, TRUE);
-	if (indx < 0)
-	    return;
-	if (cmd->tx_argc == 3)
+	indx = GeoNameToPos(cmd->tx_argv[1], TRUE, TRUE);
+	if (indx < 0) return;
+
+	if (cmd->tx_argc >= 3)
 	{
-	    if (DBWSnapToGrid != DBW_SNAP_USER)
+	    switch (indx)
 	    {
-		amountx = amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
-	    }
-	    else
-	    {
-		switch (indx)
-		{
-		    case GEO_EAST: case GEO_WEST:
-			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
-			amounty = amountx;
-			break;
-		    case GEO_NORTH: case GEO_SOUTH:
-			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
-			amountx = amounty;
-			break;
-		    default:
-			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
-			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
-			break;
-		}
+		case GEO_EAST: case GEO_WEST:
+		    amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
+		    amounty = 0;
+		    break;
+		case GEO_NORTH: case GEO_SOUTH:
+		    amountx = 0;
+		    amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
+		    break;
+		default:
+		    amountx = amounty = 0;	/* Should not happen */
+		    break;
 	    }
 	}
 	else
@@ -1936,22 +2394,6 @@ CmdStretch(w, cmd)
 		xdelta = -amountx;
 		ydelta = 0;
 		break;
-	    case GEO_NORTHEAST:
-		xdelta = amountx;
-		ydelta = amounty;
-		break;
-	    case GEO_NORTHWEST:
-		xdelta = -amountx;
-		ydelta = amounty;
-		break;
-	    case GEO_SOUTHEAST:
-		xdelta = amountx;
-		ydelta = -amounty;
-		break;
-	    case GEO_SOUTHWEST:
-		xdelta = -amountx;
-		ydelta = -amounty;
-		break;
 	    default:
 		ASSERT(FALSE, "Bad direction in CmdStretch");
 		return;
