@@ -17,14 +17,93 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBbound.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBbound.c,v 1.2 2008/12/11 04:20:04 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
+#include "database/database.h"
 
+
+typedef struct dbcellboundstruct
+{
+    Rect *area;
+    bool extended;
+    bool found;
+} DBCellBoundStruct;
+
+/*
+ * --------------------------------------------------------------------
+ * --------------------------------------------------------------------
+ */
+
+int
+DBBoundCellPlane(def, extended, rect)
+    CellDef *def;
+    bool extended;
+    Rect *rect;
+{
+    TreeFilter filter;
+    DBCellBoundStruct cbs;
+    int dbCellBoundFunc();
+    Plane *plane;
+    
+    filter.tf_func = NULL;
+    filter.tf_arg = (ClientData)&cbs;
+
+    cbs.area = rect;
+    cbs.extended = extended;
+    cbs.found = FALSE;
+
+    *rect = GeoNullRect;
+    if (TiSrArea((Tile *)NULL, def->cd_planes[PL_CELL],
+		&TiPlaneRect, dbCellBoundFunc, (ClientData) &filter) == 0)
+	return cbs.found;
+    else
+	return -1;
+}
+
+int
+dbCellBoundFunc(tile, fp)
+    Tile *tile;
+    TreeFilter *fp;
+{
+    CellUse *use;
+    CellTileBody *body;
+    Rect *bbox;
+    DBCellBoundStruct *cbs;
+
+    cbs = (DBCellBoundStruct *)fp->tf_arg;
+
+    for (body = (CellTileBody *) TiGetBody(tile); body != NULL;
+		body = body->ctb_next)
+    {
+	use = body->ctb_use;
+	bbox = &use->cu_bbox;
+	if ((BOTTOM(tile) <= bbox->r_ybot) && (RIGHT(tile) >= bbox->r_xtop))
+ 	{
+	    if (cbs->found)
+	    {
+		if (cbs->extended)
+		    GeoInclude(&use->cu_extended, cbs->area);
+		else
+		    GeoInclude(&use->cu_bbox, cbs->area);
+	    }
+	    else
+	    {
+		if (cbs->extended)
+		    *cbs->area = use->cu_extended;
+		else
+		    *cbs->area = use->cu_bbox;
+		cbs->found = TRUE;
+	    }
+	}
+    }
+    return 0;
+}
+
 /*
  * --------------------------------------------------------------------
  *
