@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellcopy.c,v 1.6 2008/12/11 04:20:04 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellcopy.c,v 1.10 2009/05/03 15:01:04 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -46,7 +46,7 @@ static PaintResultType (*dbCurPaintTbl)[NT][NT] = DBPaintResultTbl;
  * The following variable points to the version of DBPaintPlane used
  * for painting during yanks.  This is occasionally switched by clients
  * such as the design-rule checker that need to use, for example,
- * DBPaintPlaneMergeOnce instead of the standard version.
+ * DBPaintPlaneMark instead of the standard version.
  */
 static void (*dbCurPaintPlane)() = DBPaintPlaneWrapper;
 
@@ -89,7 +89,7 @@ struct copyLabelArg
  *
  * DBPaintPlaneWrapper --
  *
- *    Simple wrapper to DBPaintPlane.  Argument "type" is ignored.
+ *    Simple wrapper to DBPaintPlane.
  *    Note that this function is passed as a pointer on occasion, so
  *    it cannot be replaced with a macro!
  *
@@ -120,6 +120,42 @@ DBPaintPlaneWrapper(def, pNum, type, area, undo)
 #endif
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBPaintPlaneMark --
+ *
+ *    Another wrapper function to DBPaintPlane.  This one is used for
+ *    hierarchical DRC, and ensures that tiles are never painted twice
+ *    on the same pass, so as not to cause false overlap errors.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBPaintPlaneMark(def, pNum, type, area, undo)
+    CellDef *def;
+    int pNum;
+    TileType type;
+    Rect *area;
+    PaintUndoInfo *undo;
+{
+#ifdef NONMANHATTAN
+    TileType loctype = type & TT_LEFTMASK;
+#else
+    TileType loctype = type;
+#endif
+
+    undo->pu_pNum = pNum;
+#ifdef NONMANHATTAN
+    DBNMPaintPlane0(def->cd_planes[pNum], type, area,
+		dbCurPaintTbl[pNum][loctype], undo, TRUE);
+#else
+    DBPaintPlane0(def->cd_planes[pNum], area,
+		dbCurPaintTbl[pNum][loctype], undo, TRUE);
+#endif
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -251,7 +287,7 @@ DBCellCopyAllPaint(scx, mask, xMask, targetUse)
     locMask = *mask;
     DBMaskAddStacking(&locMask);
 
-    (void) DBTreeSrTiles(scx, &locMask, xMask, dbCopyAllPaint, (ClientData) &arg);
+    DBTreeSrTiles(scx, &locMask, xMask, dbCopyAllPaint, (ClientData) &arg);
 }
 
 /*
