@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBconnect.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBconnect.c,v 1.4 2008/12/11 04:20:04 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -561,16 +561,18 @@ dbcConnectLabelFunc(scx, lab, tpath, csa2)
 {
     CellDef *def = csa2->csa2_use->cu_def;
     Rect r;
-    int pos;
+    Point offset;
+    int pos, rotate;
     int dbcConnectFunc();		/* Forward declaration */
 
     GeoTransRect(&scx->scx_trans, &lab->lab_rect, &r);
     pos = GeoTransPos(&scx->scx_trans, lab->lab_just);
+    GeoTransPointDelta(&scx->scx_trans, &lab->lab_offset, &offset);
+    rotate = GeoTransAngle(&scx->scx_trans, lab->lab_rotate);
 
     DBEraseLabelsByContent(def, &r, -1, lab->lab_text);
-    DBPutFontLabel(def, &r, lab->lab_font, lab->lab_size, lab->lab_rotate,
-		&lab->lab_offset, pos, lab->lab_text, lab->lab_type,
-		lab->lab_flags);
+    DBPutFontLabel(def, &r, lab->lab_font, lab->lab_size, rotate, &offset,
+		pos, lab->lab_text, lab->lab_type, lab->lab_flags);
 
     if (lab->lab_flags & PORT_DIR_MASK)
     {
@@ -693,7 +695,7 @@ dbcConnectFunc(tile, cx)
     if (DBIsContact(loctype))
     {
 	TileType ctype;
-	TileTypeBitMask *rMask = DBResidueMask(loctype);
+	TileTypeBitMask *cMask, *rMask = DBResidueMask(loctype);
 
 	TTMaskSetOnlyType(&notConnectMask, loctype);
 
@@ -703,13 +705,19 @@ dbcConnectFunc(tile, cx)
 	/* (12/1/05)						*/
 
 	for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
-	    if (TTMaskIntersect(rMask, DBResidueMask(ctype)))
+	{
+	    cMask = DBResidueMask(ctype);
+	    if (TTMaskIntersect(rMask, cMask))
 		TTMaskSetType(&notConnectMask, ctype);
+	}
 
 	/* The mask of contact types must include all stacked contacts */
 	for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)
-	    if (TTMaskHasType(DBResidueMask(ctype), loctype))
+	{
+	    cMask = DBResidueMask(ctype);
+	    if (TTMaskHasType(cMask, loctype))
 		TTMaskSetType(&notConnectMask, ctype);
+	}
 
 	TTMaskCom(&notConnectMask);
     }
@@ -756,7 +764,8 @@ dbcConnectFunc(tile, cx)
     /* tile area and plane, and add them to the destination.	*/
 
     DBTreeSrLabels(&scx2, connectMask, csa2->csa2_xMask, NULL,
-			dbcConnectLabelFunc, (ClientData) csa2);
+			TF_LABEL_ATTACH, dbcConnectLabelFunc,
+			(ClientData) csa2);
 
     /* Since the whole area of this tile hasn't been recorded,
      * we must process its area to find any other tiles that
