@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBio.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBio.c,v 1.4 2008/12/17 03:37:37 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1074,6 +1074,41 @@ dbReadOpen(cellDef, name, setFileName)
     cellDef->cd_flags |= CDAVAILABLE;
     return (f);
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBTestOpen --
+ *
+ * Check whether or not a database file can be found on disk.
+ *
+ * Results:
+ *	Returns TRUE if available, FALSE if not.
+ *
+ * Side effects:
+ *	Full pathname is returned in fullPath (if non-NULL)
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+DBTestOpen(name, fullPath)
+    char *name;
+    char **fullPath;
+{
+    FILE *f;
+
+    f = PaLockOpen(name, "r", DBSuffix, Path, CellLibPath,
+		fullPath, (bool *)NULL);
+
+    if (f != NULL)
+    {
+	fclose(f);
+	return TRUE;
+    }
+    return FALSE;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -1254,6 +1289,7 @@ badTransform:
 		    r.r_xbot, r.r_ybot, r.r_xtop, r.r_ytop);
 	}
 	subCellDef->cd_bbox = r;
+	subCellDef->cd_extended = r;
     }
     else if (DBIsAncestor(subCellDef, cellDef))
     {
@@ -1641,6 +1677,7 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 		goto nextlabel;
 	    }
 	    font = DBNameToFont(fontname);
+	    if (font < -1) font = -1;	/* Force default font if font is unknown */
 	}
 	else if (line[0] == 'p')
 	{
@@ -2035,7 +2072,7 @@ DBCellWriteFile(cellDef, f)
     struct writeArg arg;
     int pNum;
     TileType type, stype;
-    TileTypeBitMask typeMask;
+    TileTypeBitMask typeMask, *sMask;
     int reducer;
     char *estring;
 
@@ -2101,8 +2138,11 @@ DBCellWriteFile(cellDef, f)
 	/* have this type as a residue.				*/
 
 	for (stype = DBNumUserLayers; stype < DBNumTypes; stype++)
-	    if (TTMaskHasType(DBResidueMask(stype), type))
+	{
+	    sMask = DBResidueMask(stype);
+	    if (TTMaskHasType(sMask, type))
 		TTMaskSetType(&typeMask, stype);
+	}
 
 	if (DBSrPaintArea((Tile *) NULL, cellDef->cd_planes[pNum],
 		&TiPlaneRect, &typeMask, dbWritePaintFunc, (ClientData) &arg))
@@ -2591,22 +2631,26 @@ dbWritePaintFunc(tile, cdarg)
     char pstring[256];
     struct writeArg *arg = (struct writeArg *) cdarg;
     TileType type = TiGetType(tile);
+    TileTypeBitMask *lMask, *rMask;
 
 #ifdef NONMANHATTAN
     int dir;
 
     if (IsSplit(tile))      
     { 
+	lMask = DBResidueMask(SplitLeftType(tile));
+	rMask = DBResidueMask(SplitRightType(tile));
+
 	if ((SplitLeftType(tile) == arg->wa_type) ||
 		((SplitLeftType(tile) >= DBNumUserLayers) &&
-		TTMaskHasType(DBResidueMask(SplitLeftType(tile)), arg->wa_type)))
+		TTMaskHasType(lMask, arg->wa_type)))
 	{
 	    type = arg->wa_type;
 	    dir = 0x0;
 	}
 	else if ((SplitRightType(tile) == arg->wa_type) ||
 		((SplitRightType(tile) >= DBNumUserLayers) &&
-		TTMaskHasType(DBResidueMask(SplitRightType(tile)), arg->wa_type)))
+		TTMaskHasType(rMask, arg->wa_type)))
 	{
 	    type = arg->wa_type;
 	    dir = 0x2;
@@ -2618,8 +2662,9 @@ dbWritePaintFunc(tile, cdarg)
 #endif   
     if (type != arg->wa_type)
     {
+	rMask = DBResidueMask(type);
 	if ((type < DBNumUserLayers) ||
-	    	(!TTMaskHasType(DBResidueMask(type), arg->wa_type)))
+	    	(!TTMaskHasType(rMask, arg->wa_type)))
 	    return 0;
 
 	type = arg->wa_type;
