@@ -17,13 +17,14 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBlabel.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBlabel.c,v 1.7 2008/12/11 18:36:43 tim Exp $";
 #endif  /* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <math.h>	/* For sin(), cos(), and round() functions */
 
 #include "utils/magic.h"
 #include "utils/malloc.h"
@@ -131,7 +132,7 @@ DBPutFontLabel(cellDef, rect, font, size, rot, offset, align, text, type, flags)
     CellDef *cellDef;	/* Cell in which label is placed */
     Rect *rect;		/* Location of label; see above for description */
     int font;		/* A vector outline font to use, or -1 for X11 font */
-    int size;		/* Scale of vector font relative to the database */
+    int size;		/* Scale of vector font relative to the database (x8) */
     int rot;		/* Rotate of the vector font in degrees */
     Point *offset;	/* Offset of the font from the point of origin */
     int align;		/* Orientation/alignment of text.  If this is < 0,
@@ -218,6 +219,7 @@ DBPutFontLabel(cellDef, rect, font, size, rot, offset, align, text, type, flags)
     }
     cellDef->cd_lastLabel = lab;
 
+    DBFontLabelSetBBox(lab);
     DBUndoPutLabel(cellDef, lab);
     cellDef->cd_flags |= CDMODIFIED|CDGETNEWSTAMP;
     return lab;
@@ -245,13 +247,16 @@ DBPutFontLabel(cellDef, rect, font, size, rot, offset, align, text, type, flags)
  *	Deleting up to the edge of a label won't cause the label
  *	to go away.  There's one final exception:  if the mask includes
  *	L_LABEL, then labels are deleted from all layers even if there's
- *	still enough material to keep them around.
+ *	still enough material to keep them around.  The rect pointed to
+ *	by areaReturn is filled with the area affected by removing the
+ *	label, for purposes of redrawing the necessary portions of the
+*	screen.
  *
  * ----------------------------------------------------------------------------
  */
 
 bool
-DBEraseLabel(cellDef, area, mask)
+DBEraseLabel(cellDef, area, mask, areaReturn)
     CellDef *cellDef;		/* Cell being modified */
     Rect *area;			/* Area from which labels are to be erased.
 				 * This may be a point; any labels touching
@@ -260,6 +265,7 @@ DBEraseLabel(cellDef, area, mask)
     TileTypeBitMask *mask;	/* Mask of types from which labels are to
 				 * be erased.
 				 */
+    Rect *areaReturn;		/* Expand this with label bounding box */
 {
     Label *lab, *labPrev;
     bool erasedAny = FALSE;
@@ -284,13 +290,16 @@ DBEraseLabel(cellDef, area, mask)
 	    }
 	}
 
+	DBWLabelChanged(cellDef, lab, DBW_ALLWINDOWS);
 	if (labPrev == NULL)
 	    cellDef->cd_labels = lab->lab_next;
 	else labPrev->lab_next = lab->lab_next;
 	if (cellDef->cd_lastLabel == lab)
 	    cellDef->cd_lastLabel = labPrev;
 	DBUndoEraseLabel(cellDef, lab);
-	DBWLabelChanged(cellDef, lab, DBW_ALLWINDOWS);
+	if ((lab->lab_font >= 0) && areaReturn)
+	    GeoInclude(&lab->lab_bbox, areaReturn);
+
 	freeMagic((char *) lab);
 	lab = lab->lab_next;
 	erasedAny = TRUE;
@@ -867,7 +876,7 @@ dbPickFunc2(tile, mask)
     TileTypeBitMask *mask;	/* Mask to be modified. */
 {
     TileType type;
-    TileTypeBitMask tmp;
+    TileTypeBitMask tmp, *rMask;
 
 #ifdef NONMANHATTAN 
     if (IsSplit(tile)) 
@@ -888,7 +897,8 @@ dbPickFunc2(tile, mask)
     tmp = *dbAdjustPlaneTypes;
     TTMaskClearType(&tmp, type);
     TTMaskClearMask(&mask[0], &tmp);
-    TTMaskClearMask(&tmp, DBResidueMask(type));
+    rMask = DBResidueMask(type);
+    TTMaskClearMask(&tmp, rMask);
     TTMaskClearMask(&mask[1], &tmp);
     TTMaskSetType(&mask[2], type);
     return 0;
@@ -1042,6 +1052,177 @@ dbGetToken(ff)
     return rptr;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBFontLabelSetBBox --
+ *
+ *	Given a font label with corresponding size, rotation, offset, etc.,
+ *	(1) compute the four corners of the label, and (2) compute the
+ *	Manhattan bounding box of the label.  The corners are computed in
+ *	units of (database / 8) for a finer resolution, and measured
+ *	relative to the label rectangle's origin so we don't run out of
+ *	bits in computing the corner positions.
+ *	
+ *	This routine needs to be run whenever a font label changes
+ *	properties (including when the font label is created).
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The label's lab_bbox record is modified to contain the bounding
+ *	box, in database coordinates, of the given label.  The label's
+ *	lab_corners[] points are modified to contain the corner positions
+ *	of the label, in units as described above.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBFontLabelSetBBox(label)
+    Label *label;
+{
+    char *tptr;
+    Point *coffset, rcenter;
+    Rect *cbbox, locbbox, *frect;
+    Transform labTransform = GeoIdentityTransform;
+    int i, ysave, size;
+    double rrad, cr, sr, tmpx, tmpy;
+    bool expand;
+
+    if (label->lab_font < 0) return;	/* No action for default font */
+
+    /* Label size is computed such that size = 1 means that the font	*/
+    /* height extent is equal to 1 database unit.  The font height	*/
+    /* extent is measured from the baseline up.				*/
+
+    frect = &DBFontList[label->lab_font]->mf_extents;
+
+    /* Get label size from outline fonts */
+
+    locbbox = GeoNullRect;
+    for (tptr = label->lab_text; *tptr != '\0'; tptr++)
+    {
+	DBFontChar(label->lab_font, *tptr, NULL, &coffset, &cbbox);
+	if (*(tptr + 1) == '\0')
+	    locbbox.r_xtop += cbbox->r_xtop;
+	else
+	    locbbox.r_xtop += coffset->p_x;
+	locbbox.r_ytop = MAX(locbbox.r_ytop, cbbox->r_ytop);
+	locbbox.r_ybot = MIN(locbbox.r_ybot, cbbox->r_ybot);
+    }
+
+    /* In case the font did not declare its extents, use the actual text height */
+    locbbox.r_ytop = MAX(locbbox.r_ytop, frect->r_ytop);
+
+    /* Scale to font size, in database units / 8 (upshift 3 bits)  */
+
+    ysave = locbbox.r_ytop;
+
+    locbbox.r_xbot *= label->lab_size;
+    locbbox.r_xtop *= label->lab_size;
+    locbbox.r_ybot *= label->lab_size;
+    locbbox.r_ytop *= label->lab_size;
+
+    locbbox.r_xbot /= ysave;
+    locbbox.r_xtop /= ysave;
+    locbbox.r_ybot /= ysave;
+    locbbox.r_ytop /= ysave;
+
+    /* Apply justification */
+
+    switch (label->lab_just)
+    {
+	case GEO_SOUTH:
+	case GEO_SOUTHWEST:
+	case GEO_SOUTHEAST:
+	    locbbox.r_ybot -= locbbox.r_ytop;
+	    locbbox.r_ytop = 0;
+	    break;
+	case GEO_WEST:
+	case GEO_EAST:
+	case GEO_CENTER:
+	    locbbox.r_ytop >>= 1;
+	    locbbox.r_ybot -= locbbox.r_ytop;
+	    break;
+    }
+    switch (label->lab_just)
+    {
+	case GEO_WEST:
+	case GEO_NORTHWEST:
+	case GEO_SOUTHWEST:
+	    locbbox.r_xbot -= locbbox.r_xtop;
+	    locbbox.r_xtop = 0;
+	    break;
+	case GEO_NORTH:
+	case GEO_SOUTH:
+	case GEO_CENTER:
+	    locbbox.r_xtop >>= 1;
+	    locbbox.r_xbot -= locbbox.r_xtop;
+	    break;
+    }
+
+    /* Apply offset (already in units / 8) */
+
+    locbbox.r_xbot += label->lab_offset.p_x;
+    locbbox.r_xtop += label->lab_offset.p_x;
+    locbbox.r_ybot += label->lab_offset.p_y;
+    locbbox.r_ytop += label->lab_offset.p_y;
+
+    /* Apply rotation */
+
+    if (label->lab_rotate < 0) label->lab_rotate += 360;
+    if (label->lab_rotate >= 360) label->lab_rotate -= 360;
+
+    label->lab_corners[0] = locbbox.r_ll;
+    label->lab_corners[1].p_x = locbbox.r_ur.p_x;
+    label->lab_corners[1].p_y = locbbox.r_ll.p_y;
+    label->lab_corners[2] = locbbox.r_ur;
+    label->lab_corners[3].p_x = locbbox.r_ll.p_x;
+    label->lab_corners[3].p_y = locbbox.r_ur.p_y;
+
+    rrad = (double)label->lab_rotate * 0.0174532925;
+    cr = cos(rrad);
+    sr = sin(rrad);
+
+    for (i = 0; i < 4; i++)
+    {
+	tmpx = label->lab_corners[i].p_x * cr - label->lab_corners[i].p_y * sr;
+	tmpy = label->lab_corners[i].p_x * sr + label->lab_corners[i].p_y * cr;
+	label->lab_corners[i].p_x = (int)round(tmpx);
+	label->lab_corners[i].p_y = (int)round(tmpy);
+
+	/* Initialize bounding box */
+	if (i == 0)
+	    label->lab_bbox.r_ll = label->lab_bbox.r_ur = label->lab_corners[0];
+	else
+	    GeoIncludePoint(&label->lab_corners[i], &label->lab_bbox);
+    }
+
+    /* Compute the bounding box, in (rounded) database coordinates */
+
+    rcenter.p_x = (label->lab_rect.r_xtop + label->lab_rect.r_xbot) << 2;
+    rcenter.p_y = (label->lab_rect.r_ytop + label->lab_rect.r_ybot) << 2;
+    label->lab_bbox.r_xbot += rcenter.p_x;
+    label->lab_bbox.r_xtop += rcenter.p_x;
+    label->lab_bbox.r_ybot += rcenter.p_y;
+    label->lab_bbox.r_ytop += rcenter.p_y;
+
+    expand = (label->lab_bbox.r_xbot & 0x7f) ? TRUE : FALSE;
+    label->lab_bbox.r_xbot >>= 3;
+    if (expand) label->lab_bbox.r_xbot--;
+    expand = (label->lab_bbox.r_xtop & 0x7f) ? TRUE : FALSE;
+    label->lab_bbox.r_xtop >>= 3;
+    if (expand) label->lab_bbox.r_xtop++;
+    expand = (label->lab_bbox.r_ybot & 0x7f) ? TRUE : FALSE;
+    label->lab_bbox.r_ybot >>= 3;
+    if (expand) label->lab_bbox.r_ybot--;
+    expand = (label->lab_bbox.r_ytop & 0x7f) ? TRUE : FALSE;
+    label->lab_bbox.r_ytop >>= 3;
+    if (expand) label->lab_bbox.r_ytop++;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1086,8 +1267,8 @@ DBFontChar(font, ccode, clist, coffset, cbbox)
  *	Given a font name, return the index into DBFontList.
  *
  * Results:
- *	-1 if the name is not found, otherwise return the index into the
- *	DBFontList.
+ *	-1 if the name is "default", and -2 if the name is not found.
+ *	Otherwise, return the index into the DBFontList.
  *
  * Side effects:
  *	None.
@@ -1103,7 +1284,9 @@ DBNameToFont(name)
     for (i = 0; i < DBNumFonts; i++)
 	if (!strcasecmp(name, DBFontList[i]->mf_name))
 	    return i;
-    return -1;
+    if (!strcasecmp(name, "default"))
+	return -1;
+    return -2;
 }
 
 /*
@@ -1131,8 +1314,9 @@ DBNameToFont(name)
 #define NUMBUF 16
 
 int
-DBLoadFont(fontfile)
+DBLoadFont(fontfile, scale)
     char *fontfile;
+    float scale;
 {
     FILE *ff;
     char *ascii_names[] = {
@@ -1155,7 +1339,7 @@ DBLoadFont(fontfile)
     MagicFont *newFont = NULL, **newDBFontList;
     FontChar *newChar;
     FontPath *newPath, *pathStart = NULL, *curPath, *lastPath;
-    Rect extents;
+    Rect extents = GeoNullRect;
 
     ff = PaOpen(fontfile, "r", (char *)NULL, ".", SysLibPath, (char **)NULL);
     if (ff == NULL) return -1;
@@ -1200,7 +1384,19 @@ DBLoadFont(fontfile)
 		    DBNumFonts++;
 		    for (asciiidx = 0; asciiidx < 96; asciiidx++)
 			newFont->mf_vectors[asciiidx] = NULL;
+
 		    newFont->mf_extents = extents;
+
+		    /* This is a hack, but all these fonts declare	*/
+		    /* their "height" to be the Y-value of the tallest	*/
+		    /* character, from which one cannot get any useful	*/
+		    /* measure of the height of a standard capital	*/
+		    /* letter.  I am using an ad-hoc scale passed as an	*/
+		    /* argument to the load command (usually ~0.6).	*/
+
+		    newFont->mf_extents.r_ytop =
+				(int)((float)newFont->mf_extents.r_ytop
+				* scale);
 		}
 	    }
 	    else if (!strcmp(token, "definefont"))
