@@ -22,12 +22,13 @@
 /* #define	PAINTDEBUG /* For debugging */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBpaint.c,v 1.4 2001/04/13 20:02:58 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBpaint.c,v 1.5 2009/05/03 15:01:04 tim Exp $";
 #endif  /* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/malloc.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -37,9 +38,9 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBpaint.c,v 1.4 2
 #include "windows/windows.h"
 #include "graphics/graphics.h"
 #include "dbwind/dbwind.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "textio/textio.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 
 /* ---------------------- Imports from DBundo.c ----------------------- */
 
@@ -79,11 +80,13 @@ int dbPaintDebug = 0;
 				    &&	TOP(t1) == TOP(t2) )
 
 #define	SELMERGE_Y(t1, t2, msk)	(       LEFT(t1) == LEFT(t2) \
+				    &&  TiGetTypeExact(t1) == TiGetTypeExact(t2) \
 				    &&  TiGetTypeExact(t1) == TiGetTypeExact(t2) \
 				    &&	RIGHT(t1) == RIGHT(t2) \
 				    &&  ! TTMaskHasType(msk, TiGetType(t1)) )
 
 #define	SELMERGE_X(t1, t2, msk)	(	BOTTOM(t1) == BOTTOM(t2) \
+				    &&  TiGetTypeExact(t1) == TiGetTypeExact(t2) \
 				    &&  TiGetTypeExact(t1) == TiGetTypeExact(t2) \
 				    &&	TOP(t1) == TOP(t2) \
 				    &&  ! TTMaskHasType(msk, TiGetType(t1)) )
@@ -94,8 +97,8 @@ int dbPaintDebug = 0;
 	Tile *xtile = otile, *xxnew, *xp; \
 	int x = xcoord; \
  \
-	MALLOC(Tile *, xxnew, sizeof (Tile)); \
-	xxnew->ti_client = (ClientData) MINFINITY; \
+	xxnew = (Tile *) TiAlloc(); \
+	xxnew->ti_client = (ClientData) CLIENTDEFAULT; \
  \
 	LEFT(xxnew) = x, BOTTOM(xxnew) = BOTTOM(xtile); \
 	BL(xxnew) = xtile, TR(xxnew) = TR(xtile), RT(xxnew) = RT(xtile); \
@@ -114,6 +117,10 @@ int dbPaintDebug = 0;
 	res = xxnew; \
     }
 
+/* Use this for debugging purposes when necessary */
+// #undef TISPLITX
+// #define TISPLITX(a, b, c) a = TiSplitX(b, c)
+
 /* Record undo information */
 
 #define	DBPAINTUNDO(tile, newType, undo) \
@@ -140,7 +147,7 @@ int dbPaintDebug = 0;
 /*
  * ----------------------------------------------------------------------------
  *
- * DBPaintPlane --
+ * DBPaintPlane0 --
  *
  * Paint a rectangular area ('area') on a single tile plane ('plane').
  *
@@ -164,11 +171,20 @@ int dbPaintDebug = 0;
  *	Callers should remember to set the CDMODIFIED and CDGETNEWSTAMP
  *	bits in the cell definition containing the plane being painted.
  *
+ * NOTE:
+ *	This routine was modified from DBPaintPlane().  To prevent making
+ *	a nested call (since this routine is very high frequency), DBPaintPlane()
+ *	is defined in database.h as DBPaintPlane0(..., FALSE).  The use of
+ *	DBPaintPlane(..., TRUE) is a replacement for the original
+ *	DBPaintPlaneMergeOnce(), the purpose of which is to avoid painting
+ *	any tile twice in the same pass, since the DRC overlap rule depends
+ *	on it.
+ *
  * ----------------------------------------------------------------------------
  */
 
 void
-DBPaintPlane(plane, area, resultTbl, undo)
+DBPaintPlane0(plane, area, resultTbl, undo, mark)
     Plane *plane;		/* Plane whose paint is to be modified */
     Rect *area;	/* Area to be changed */
     PaintResultType *resultTbl;	/* Table, indexed by the type of tile already
@@ -180,6 +196,9 @@ DBPaintPlane(plane, area, resultTbl, undo)
 				 * save undo entries for this operation.
 				 * If NULL, the undo package is not used.
 				 */
+    bool mark;			/* If TRUE the routine marks tiles as it
+				 * goes to avoid processing tiles twice.
+				 */
 {
     Point start;
     int clipTop, mergeFlags;
@@ -217,6 +236,11 @@ enumerate:
 	if (SigInterruptPending)
 	    break;
 
+	/* Skip processed tiles, if the "mark" option was chosen */
+	if (mark)
+	    if (tile->ti_client != (ClientData) CLIENTDEFAULT)
+		goto paintdone;
+
 	clipTop = TOP(tile);
 	if (clipTop > area->r_ytop) clipTop = area->r_ytop;
 	oldType = TiGetTypeExact(tile);
@@ -239,13 +263,6 @@ enumerate:
 	if (RIGHT(tile) >= area->r_xtop) mergeFlags |= MRG_RIGHT;
 	if (BOTTOM(tile) <= area->r_ybot) mergeFlags |= MRG_BOTTOM;
 
-	/*
-	 * The following is a kludge for plowing that should go away
-	 * once the plowing code gets stable.  Make sure that the intermediate
-	 * coordinate of this tile is reset to "uninitialized".
-	 */
-	tile->ti_client = (ClientData) MINFINITY;
-
 	/*
 	 * Determine new type of this tile.
 	 * Change the type if necessary.
@@ -263,17 +280,8 @@ enumerate:
 #endif
 	newType = resultTbl[oldType];
 
-//	/* Sounded good, but violates maximum horiz. strips!	*/
-//	/* Beware violating maximum horiz. strips property	*/
-//	/* unless you want to change the code in a lot of	*/
-//	/* places. . .						*/
-//
-//	if (oldType == newType)
-//	     goto paintdone; /* nothing to do */
-
 	if (oldType != newType)
 	{
-
 	    /*
 	     * Clip the tile against the clipping rectangle.
 	     * Merging is only necessary if we clip to the left or to
@@ -293,6 +301,20 @@ enumerate:
 			oldType = TiGetTypeExact(tile);
 			newType = resultTbl[oldType];
 		    }
+
+		    /* Ensure maximum horizontal strips for non-split-tile areas */
+		    tp = TR(newtile);
+		    if (!IsSplit(tp))
+		        /* Merge the outside tile to its right if possible */
+			if (TiGetTypeExact(TR(tp)) == TiGetTypeExact(tp))
+			    dbPaintMerge(tp, TiGetTypeExact(tp), plane,
+					MRG_RIGHT, undo, mark);
+		    tp = newtile;
+		    if (!IsSplit(tp))
+		        /* Merge the outside tile to its right if possible */
+			if (TiGetTypeExact(BL(tp)) == TiGetTypeExact(tp))
+			    newtile = dbPaintMerge(tp, TiGetTypeExact(tp), plane,
+					MRG_LEFT, undo, mark);
 		}
 		else
 		{
@@ -324,6 +346,20 @@ enumerate:
 			oldType = TiGetTypeExact(tile);
 			newType = resultTbl[oldType];
 		    }
+
+		    /* Ensure maximum horizontal strips for non-split-tile areas */
+		    tp = TR(newtile);
+		    if (!IsSplit(tp))
+		        /* Merge the outside tile to its right if possible */
+			if (TiGetTypeExact(TR(tp)) == TiGetTypeExact(tp))
+			    dbPaintMerge(tp, TiGetTypeExact(tp), plane,
+					MRG_RIGHT, undo, mark);
+		    tp = newtile;
+		    if (!IsSplit(tp))
+		        /* Merge the outside tile to its right if possible */
+			if (TiGetTypeExact(BL(tp)) == TiGetTypeExact(tp))
+			    newtile = dbPaintMerge(tp, TiGetTypeExact(tp), plane,
+					MRG_LEFT, undo, mark);
 		}
 		else
 		{
@@ -355,6 +391,20 @@ enumerate:
 			oldType = TiGetTypeExact(tile);
 			newType = resultTbl[oldType];
 		    }
+
+		    /* Ensure maximum horizontal strips for non-split-tile areas */
+		    tp = LB(newtile);
+		    if (!IsSplit(tp))
+		        /* Merge the outside tile to its right if possible */
+			if (TiGetTypeExact(TR(tp)) == TiGetTypeExact(tp))
+			    dbPaintMerge(tp, TiGetTypeExact(tp),
+					plane, MRG_RIGHT, undo, mark);
+		    tp = newtile;
+		    if (!IsSplit(tp))
+		        /* Merge the outside tile to its right if possible */
+			if (TiGetTypeExact(TR(tp)) == TiGetTypeExact(tp))
+			    newtile = dbPaintMerge(tp, TiGetTypeExact(tp),
+					plane, MRG_RIGHT, undo, mark);
 		}
 		else
 		{
@@ -394,6 +444,20 @@ enumerate:
 			oldType = TiGetTypeExact(tile);
 			newType = resultTbl[oldType];
 		    }
+
+		    /* Ensure maximum horizontal strips for non-split-tile areas */
+		    tp = LB(newtile);
+		    if (!IsSplit(tp))
+		        /* Merge the outside tile to its right if possible */
+			if (TiGetTypeExact(BL(tp)) == TiGetTypeExact(tp))
+			    dbPaintMerge(tp, TiGetTypeExact(tp),
+					plane, MRG_LEFT, undo, mark);
+		    tp = newtile;
+		    if (!IsSplit(tp))
+		        /* Merge the outside tile to its right if possible */
+			if (TiGetTypeExact(BL(tp)) == TiGetTypeExact(tp))
+			    newtile = dbPaintMerge(tp, TiGetTypeExact(tp),
+					plane, MRG_LEFT, undo, mark);
 		}
 		else
 		{
@@ -440,6 +504,9 @@ enumerate:
 		if (undo && UndoIsEnabled())
 		    DBPAINTUNDO(tile, newType, undo);
 		TiSetBody(tile, newType);
+	 	if (mark) tile->ti_client = (ClientData)1;
+		/* Reinstate the left merge requirement */
+		mergeFlags |= MRG_LEFT;
 	    }
 	    else
 		mergeFlags = 0;
@@ -458,7 +525,7 @@ enumerate:
 	    for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
 		if (TiGetTypeExact(tp) == newType)
 		{
-		    tile = dbPaintMerge(tile, newType, plane, mergeFlags, undo);
+		    tile = dbPaintMerge(tile, newType, plane, mergeFlags, undo, mark);
 		    goto paintdone;
 		}
 	    mergeFlags &= ~MRG_LEFT;
@@ -468,7 +535,7 @@ enumerate:
 	    for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
 		if (TiGetTypeExact(tp) == newType)
 		{
-		    tile = dbPaintMerge(tile, newType, plane, mergeFlags, undo);
+		    tile = dbPaintMerge(tile, newType, plane, mergeFlags, undo, mark);
 		    goto paintdone;
 		}
 	    mergeFlags &= ~MRG_RIGHT;
@@ -489,6 +556,7 @@ enumerate:
 #endif
 	    DBPAINTUNDO(tile, newType, undo);
 	TiSetBody(tile, newType);
+	if (mark) tile->ti_client = (ClientData)1;
 
 #ifdef	PAINTDEBUG
 	if (dbPaintDebug)
@@ -553,6 +621,53 @@ paintdone:
     }
 
 done:
+
+    if (mark)
+    {
+	/* Now unmark the processed tiles with the same search algorithm */
+	tile = plane->pl_hint;
+	GOTOPOINT(tile, &start);
+
+enum2:
+	while (TOP(tile) > area->r_ybot)
+	{
+	    tile->ti_client = (ClientData)CLIENTDEFAULT;
+
+	    /* Move right if possible */
+	    tpnew = TR(tile);
+	    if (LEFT(tpnew) < area->r_xtop)
+	    {
+		/* Move back down into clipping area if necessary */
+		while (BOTTOM(tpnew) >= clipTop) tpnew = LB(tpnew);
+		if (BOTTOM(tpnew) >= BOTTOM(tile) || BOTTOM(tile) <= area->r_ybot)
+		{
+		    tile = tpnew;
+		    goto enum2;
+		}
+	    }
+
+	    /* Each iteration returns one tile further to the left */
+	    while (LEFT(tile) > area->r_xbot)
+	    {
+		/* Move left if necessary */
+		if (BOTTOM(tile) <= area->r_ybot)
+		    goto done2;
+
+		/* Move down if possible; left otherwise */
+		tpnew = LB(tile); tile = BL(tile);
+		if (BOTTOM(tpnew) >= BOTTOM(tile) || BOTTOM(tile) <= area->r_ybot)
+		{
+		    tile = tpnew;
+		    goto enum2;
+		}
+	    }
+	    /* At left edge -- walk down to next tile along the left edge */
+	    for (tile = LB(tile); RIGHT(tile) <= area->r_xbot; tile = TR(tile))
+		/* Nothing */;
+	}
+    }
+
+done2:
     plane->pl_hint = tile;
 }
 
@@ -680,6 +795,22 @@ enumerate:
 	    {
 		TiNMSplitY(&tile, &newtile, area->r_ytop, 1, undo);
 		if (!IsSplit(tile)) oldType = TiGetTypeExact(tile);
+
+		/* Ensure maximum horizontal strips for non-split-tile	*/
+		/* areas outside of the fracture zone.			*/
+
+		tp = TR(newtile);
+		if (!IsSplit(tp))
+		    /* Merge the outside tile to its right if possible */
+		    if (TiGetTypeExact(TR(tp)) == TiGetTypeExact(tp))
+			dbPaintMerge(tp, TiGetTypeExact(tp), plane,
+					MRG_RIGHT, undo, FALSE);
+		tp = newtile;
+		if (!IsSplit(tp))
+		    /* Merge the outside tile to its right if possible */
+		    if (TiGetTypeExact(BL(tp)) == TiGetTypeExact(tp))
+			newtile = dbPaintMerge(tp, TiGetTypeExact(tp), plane,
+					MRG_LEFT, undo, FALSE);
 	    }
 	    else
 	    {
@@ -700,6 +831,22 @@ enumerate:
 	    {
 		TiNMSplitY(&tile, &newtile, area->r_ybot, 0, undo);
 		if (!IsSplit(tile)) oldType = TiGetTypeExact(tile);
+
+		/* Ensure maximum horizontal strips for non-split-tile areas	*/
+		/* outside of the fracture zone					*/
+
+		tp = TR(newtile);
+		if (!IsSplit(tp))
+		    /* Merge the outside tile to its right if possible */
+		    if (TiGetTypeExact(TR(tp)) == TiGetTypeExact(tp))
+			dbPaintMerge(tp, TiGetTypeExact(tp), plane,
+				MRG_RIGHT, undo, FALSE);
+		tp = newtile;
+		if (!IsSplit(tp))
+		    /* Merge the outside tile to its right if possible */
+		    if (TiGetTypeExact(BL(tp)) == TiGetTypeExact(tp))
+			newtile = dbPaintMerge(tp, TiGetTypeExact(tp), plane,
+					MRG_LEFT, undo, FALSE);
 	    }
 	    else
 	    {
@@ -720,6 +867,20 @@ enumerate:
 	    {
 		TiNMSplitX(&tile, &newtile, area->r_xtop, 1, undo);
 		if (!IsSplit(tile)) oldType = TiGetTypeExact(tile);
+
+		/* Ensure maximum horizontal strips for non-split-tile areas */
+		tp = LB(newtile);
+		if (!IsSplit(tp))
+		    /* Merge the outside tile to its right if possible */
+		    if (TiGetTypeExact(TR(tp)) == TiGetTypeExact(tp))
+			dbPaintMerge(tp, TiGetTypeExact(tp),
+				plane, MRG_RIGHT, undo, FALSE);
+		tp = newtile;
+		if (!IsSplit(tp))
+		    /* Merge the outside tile to its right if possible */
+		    if (TiGetTypeExact(TR(tp)) == TiGetTypeExact(tp))
+			newtile = dbPaintMerge(tp, TiGetTypeExact(tp),
+				plane, MRG_RIGHT, undo, FALSE);
 	    }
 	    else
 	    {
@@ -748,6 +909,20 @@ enumerate:
 	    {
 		TiNMSplitX(&tile, &newtile, area->r_xbot, 0, undo);
 		if (!IsSplit(tile)) oldType = TiGetTypeExact(tile);
+
+		/* Ensure maximum horizontal strips for non-split-tile areas */
+		tp = LB(newtile);
+		if (!IsSplit(tp))
+		    /* Merge the outside tile to its right if possible */
+		    if (TiGetTypeExact(BL(tp)) == TiGetTypeExact(tp))
+			dbPaintMerge(tp, TiGetTypeExact(tp),
+				plane, MRG_LEFT, undo, FALSE);
+		tp = newtile;
+		if (!IsSplit(tp))
+		    /* Merge the outside tile to its right if possible */
+		    if (TiGetTypeExact(BL(tp)) == TiGetTypeExact(tp))
+			newtile = dbPaintMerge(tp, TiGetTypeExact(tp),
+				plane, MRG_LEFT, undo, FALSE);
 	    }
 	    else
 	    {
@@ -789,7 +964,7 @@ paintmerge:
 	    for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
 		if (TiGetTypeExact(tp) == oldType)
 		{
-		    tile = dbPaintMerge(tile, oldType, plane, mergeFlags, undo);
+		    tile = dbPaintMerge(tile, oldType, plane, mergeFlags, undo, FALSE);
 		    goto paintdone;
 		}
 	    mergeFlags &= ~MRG_LEFT;
@@ -800,7 +975,7 @@ paintmerge:
 	    for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
 		if (TiGetTypeExact(tp) == oldType)
 		{
-		    tile = dbPaintMerge(tile, oldType, plane, mergeFlags, undo);
+		    tile = dbPaintMerge(tile, oldType, plane, mergeFlags, undo, FALSE);
 		    goto paintdone;
 		}
 	    mergeFlags &= ~MRG_RIGHT;
@@ -1126,6 +1301,12 @@ enumerate:
 	clipTop = TOP(tile);
 	if (clipTop > areaex.r_ytop) clipTop = areaex.r_ytop;
 
+	/* At this point, we should incorporate the routine	*/
+	/* dbMergeNMTilesFunc(), or at least its intended	*/
+	/* effect.						*/
+
+	if (IsSplit(tile)) goto paintdone;
+
 	/* These merges are opposite to what is done in the	*/
 	/* DBPaintPlane routine.				*/
 
@@ -1144,7 +1325,7 @@ enumerate:
 	    for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
 		if (TiGetTypeExact(tp) == ttype)
 		{
-		    tile = dbPaintMerge(tile, ttype, plane, mergeFlags, undo);
+		    tile = dbPaintMerge(tile, ttype, plane, mergeFlags, undo, FALSE);
 		    goto paintdone;
 		}
 	    mergeFlags &= ~MRG_LEFT;
@@ -1155,7 +1336,7 @@ enumerate:
 	    for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
 		if (TiGetTypeExact(tp) == ttype)
 		{
-		    tile = dbPaintMerge(tile, ttype, plane, mergeFlags, undo);
+		    tile = dbPaintMerge(tile, ttype, plane, mergeFlags, undo, FALSE);
 		    goto paintdone;
 		}
 	    mergeFlags &= ~MRG_RIGHT;
@@ -1355,7 +1536,7 @@ DBDiagonalProc(oldtype, cdata)
  */
 
 void
-DBNMPaintPlane(plane, exacttype, area, resultTbl, undo)
+DBNMPaintPlane0(plane, exacttype, area, resultTbl, undo, mark)
     Plane *plane;		/* Plane whose paint is to be modified */
     TileType exacttype;		/* diagonal info for tile to be changed */
     Rect *area;	/* Area to be changed */
@@ -1368,6 +1549,7 @@ DBNMPaintPlane(plane, exacttype, area, resultTbl, undo)
 				 * save undo entries for this operation.
 				 * If NULL, the undo package is not used.
 				 */
+    bool mark;			/* If true, track tiles as they are processed */
 
 #define RES_LEFT	0	/* Result is rect to left of diagonal */
 #define RES_DIAG	1	/* Resulting rectangle is on diagonal */
@@ -1376,7 +1558,8 @@ DBNMPaintPlane(plane, exacttype, area, resultTbl, undo)
 {
     DiagInfo dinfo;
     LinkedRect *lhead, *lr, *newlr;
-    int xc, yc, width, height, xref, yref;
+    int xc, yc, width, height;
+    dlong xref, yref;		/* xref, yref can easily exceed 32 bits */
     int resstate;
     mergeNMStruct nmstruct;
 
@@ -1441,6 +1624,12 @@ DBNMPaintPlane(plane, exacttype, area, resultTbl, undo)
 
 		else if (newType == -1)
 		{
+		    if ((width == 1) || (height == 1))
+		    {
+			freeMagic((char *) lr);
+			return;
+		    }
+
 		    /* lr->r_r is drawn & quartered */
 
 		    lr->r_r.r_xtop -= (width >> 1);
@@ -1511,11 +1700,12 @@ DBNMPaintPlane(plane, exacttype, area, resultTbl, undo)
 	    GeoClip(&lr->r_r, area);
 
 	    /* Split off left */
-	    yref = width * ((dinfo.dir) ? lr->r_r.r_ytop - area->r_ytop :
+	    yref = (dlong)width * (dlong)((dinfo.dir) ?
+			lr->r_r.r_ytop - area->r_ytop :
 			lr->r_r.r_ybot - area->r_ybot);
 	    xc = (((yref % height) << 1) >= height) ? 1 : 0;	/* round */
 	    if (dinfo.dir) yref = -yref;
-	    xc += area->r_xbot + yref / height;
+	    xc += area->r_xbot + (int)(yref / (dlong)height);
 	    if (xc > lr->r_r.r_xbot && xc < lr->r_r.r_xtop)
 	    {
 		newlr = (LinkedRect *) mallocMagic(sizeof(LinkedRect));
@@ -1536,7 +1726,8 @@ DBNMPaintPlane(plane, exacttype, area, resultTbl, undo)
 	    if (resstate != RES_DIAG) goto paintrect;
 
 	    /* Split off right */
-	    yref = width * ((dinfo.dir) ? lr->r_r.r_ybot - area->r_ytop :
+	    yref = (dlong)width * (dlong)((dinfo.dir) ?
+			lr->r_r.r_ybot - area->r_ytop :
 			lr->r_r.r_ytop - area->r_ybot);
 	    xc = (((yref % height) << 1) >= height) ? 1 : 0;	/* round */
 	    if (dinfo.dir) yref = -yref;
@@ -1561,11 +1752,12 @@ DBNMPaintPlane(plane, exacttype, area, resultTbl, undo)
 	    if (resstate != RES_DIAG) goto paintrect;
 
 	    /* Split off top */
-	    xref = height * ((dinfo.dir) ? lr->r_r.r_xbot - area->r_xtop :
+	    xref = (dlong)height * (dlong)((dinfo.dir) ?
+			lr->r_r.r_xbot - area->r_xtop :
 			lr->r_r.r_xtop - area->r_xbot);
 	    yc = (((xref % width) << 1) >= width) ? 1 : 0;	/* round */
 	    if (dinfo.dir) xref = -xref;
-	    yc += area->r_ybot + xref / width;
+	    yc += area->r_ybot + (int)(xref / (dlong)width);
 	    if (yc > lr->r_r.r_ybot && yc < lr->r_r.r_ytop)
 	    {
 		newlr = (LinkedRect *) mallocMagic(sizeof(LinkedRect));
@@ -1587,7 +1779,8 @@ DBNMPaintPlane(plane, exacttype, area, resultTbl, undo)
 	    if (resstate != RES_DIAG) goto paintrect;
 
 	    /* Split off bottom */
-	    xref = height * ((dinfo.dir) ? lr->r_r.r_xtop - area->r_xtop :
+	    xref = (dlong)height * (dlong)((dinfo.dir) ?
+			lr->r_r.r_xtop - area->r_xtop :
 			lr->r_r.r_xbot - area->r_xbot);
 	    yc = (((xref % width) << 1) >= width) ? 1 : 0;
 	    if (dinfo.dir) xref = -xref;
@@ -1615,7 +1808,7 @@ paintrect:
 	    if (resstate == RES_DIAG)
 	    {
 		/* Recursive call to self on sub-area */
-		DBNMPaintPlane(plane, exacttype, &(lr->r_r), resultTbl, undo);
+		DBNMPaintPlane0(plane, exacttype, &(lr->r_r), resultTbl, undo, mark);
 	    }
 	    else if ((resstate == RES_LEFT && !dinfo.side) ||
 		     (resstate == RES_RIGHT && dinfo.side)) {
@@ -1639,7 +1832,7 @@ nextrect:
 	DBUnFracturePlane(plane, area, undo);
     }
     else
-	DBPaintPlane(plane, area, resultTbl, undo);
+	DBPaintPlane0(plane, area, resultTbl, undo, mark);
 
 // nmmerge:
 //
@@ -1687,360 +1880,6 @@ dbNMEnumFunc(tile, arg)
 
 #endif
 
-
-/*
- * ----------------------------------------------------------------------------
- *
- * DBPaintPlaneMergeOnce --
- *
- * Paint a rectangular area ('area') on a single tile plane ('plane').
- * This is identical to DBPaintPlane(), except that we work in two
- * passes:
- *
- *	Pass 1: clip all tiles to lie inside the area to be painted,
- *		merging all outside tiles as required.  Change the
- *		types of each of these internal tiles.
- *
- *	Pass 2:	re split and merge to insure that the database is
- *		once again in maximal horizontal strips.
- *
- * See DBPaintPlane for other comments.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	Modifies the database plane that contains the given tile.
- *
- * ----------------------------------------------------------------------------
- */
-
-/* Currently, the non-manhattan version is non-functional, other than	*/
-/* to declare the proper parameters to match DBNMPaintPlane().  The	*/
-/* non-manhattan functionality will need to be added for proper DRC	*/
-/* checking of non-manhattan geometry.					*/
-
-#ifdef NONMANHATTAN
-
-void
-DBPaintPlaneMergeOnce(plane, exacttype, area, resultTbl, undo)
-    Plane *plane;		/* Plane whose paint is to be modified */
-    TileType exacttype;		/* Source tile type diagonal info (only) */
-    Rect *area;	/* Area to be changed */
-    PaintResultType *resultTbl;	/* Table, indexed by the type of tile already
-				 * present in the plane, giving the type to
-				 * which the existing tile must change as a
-				 * result of this paint operation.
-				 */
-    PaintUndoInfo *undo;	/* Record containing everything needed to
-				 * save undo entries for this operation.
-				 * If NULL, the undo package is not used.
-				 */
-
-#else
-
-void
-DBPaintPlaneMergeOnce(plane, area, resultTbl, undo)
-    Plane *plane;		/* Plane whose paint is to be modified */
-    Rect *area;	/* Area to be changed */
-    PaintResultType *resultTbl;	/* Table, indexed by the type of tile already
-				 * present in the plane, giving the type to
-				 * which the existing tile must change as a
-				 * result of this paint operation.
-				 */
-    PaintUndoInfo *undo;	/* Record containing everything needed to
-				 * save undo entries for this operation.
-				 * If NULL, the undo package is not used.
-				 */
-#endif
-
-{
-    Point start;
-    int clipTop, mergeFlags;
-    TileType oldType, newType;
-    Tile *tile, *newtile;
-    Tile *tpnew;	/* Used for area search */
-    Tile *tp;		/* Used for paint */
-
-    if (area->r_xtop <= area->r_xbot || area->r_ytop <= area->r_ybot)
-	return;
-
-    /*
-     * The following is a modified version of the area enumeration
-     * algorithm.  It expects the in-line paint code below to leave
-     * 'tile' pointing to the tile from which we should continue the
-     * search.
-     */
-
-    start.p_x = area->r_xbot;
-    start.p_y = area->r_ytop - 1;
-    tile = plane->pl_hint;
-    GOTOPOINT(tile, &start);
-
-    /* Each iteration visits another tile on the LHS of the search area */
-    while (TOP(tile) > area->r_ybot)
-    {
-	/***
-	 *** AREA SEARCH.
-	 *** Each iteration enumerates another tile.
-	 ***/
-enumerate:
-	if (SigInterruptPending)
-	    break;
-
-	clipTop = TOP(tile);
-	if (clipTop > area->r_ytop) clipTop = area->r_ytop;
-	oldType = TiGetTypeExact(tile);
-
-#ifdef	PAINTDEBUG
-	if (dbPaintDebug)
-	    dbPaintShowTile(tile, undo, "first area enum");
-#endif	/* PAINTDEBUG */
-
-	/***
-	 *** ---------- THE FOLLOWING IS IN-LINE PAINT CODE ----------
-	 ***/
-
-	/*
-	 * Determine new type of this tile.
-	 * Change the type if necessary.
-	 */
-
-	newType = resultTbl[oldType];
-	if (oldType != newType)
-	{
-	    /*
-	     * Clip the tile against the clipping rectangle.
-	     * Merging of the outside tiles is only necessary if we clip
-	     * to the left or to the right, and then only to the top or
-	     * the bottom.  We do the merge in-line for efficiency.
-	     */
-
-	    /* Clip up */
-	    if (TOP(tile) > area->r_ytop)
-	    {
-		newtile = TiSplitY(tile, area->r_ytop);
-		TiSetBody(newtile, TiGetBody(tile));
-	    }
-
-	    /* Clip down */
-	    if (BOTTOM(tile) < area->r_ybot)
-	    {
-		newtile = tile, tile = TiSplitY(tile, area->r_ybot);
-		TiSetBody(tile, TiGetBody(newtile));
-	    }
-
-	    /* Clip right */
-	    if (RIGHT(tile) > area->r_xtop)
-	    {
-		TISPLITX(newtile, tile, area->r_xtop);
-		TiSetBody(newtile, TiGetBody(tile));
-
-		/* Merge the outside tile to its top */
-		tp = RT(newtile);
-		if (CANMERGE_Y(newtile, tp)) TiJoinY(newtile, tp, plane);
-
-		/* Merge the outside tile to its bottom */
-		tp = LB(newtile);
-		if (CANMERGE_Y(newtile, tp)) TiJoinY(newtile, tp, plane);
-	    }
-
-	    /* Clip left */
-	    if (LEFT(tile) < area->r_xbot)
-	    {
-		newtile = tile;
-		TISPLITX(tile, tile, area->r_xbot);
-		TiSetBody(tile, TiGetBody(newtile));
-
-		/* Merge the outside tile to its top */
-		tp = RT(newtile);
-		if (CANMERGE_Y(newtile, tp)) TiJoinY(newtile, tp, plane);
-
-		/* Merge the outside tile to its bottom */
-		tp = LB(newtile);
-		if (CANMERGE_Y(newtile, tp)) TiJoinY(newtile, tp, plane);
-	    }
-
-
-#ifdef	PAINTDEBUG
-	    if (dbPaintDebug)
-		dbPaintShowTile(tile, undo, "after clip");
-#endif	/* PAINTDEBUG */
-
-	    /* Record the type of the new tile */
-	    if (undo && UndoIsEnabled())
-		DBPAINTUNDO(tile, newType, undo);
-	    TiSetBody(tile, newType);
-	}
-
-	/***
-	 ***		END OF PAINT CODE
-	 *** ---------- BACK TO AREA SEARCH ----------
-	 ***/
-	/* Move right if possible */
-	tpnew = TR(tile);
-	if (LEFT(tpnew) < area->r_xtop)
-	{
-	    /* Move back down into clipping area if necessary */
-	    while (BOTTOM(tpnew) >= clipTop) tpnew = LB(tpnew);
-	    if (BOTTOM(tpnew) >= BOTTOM(tile) || BOTTOM(tile) <= area->r_ybot)
-	    {
-		tile = tpnew;
-		goto enumerate;
-	    }
-	}
-
-	/* Each iteration returns one tile further to the left */
-	while (LEFT(tile) > area->r_xbot)
-	{
-	    /* Move left if necessary */
-	    if (BOTTOM(tile) <= area->r_ybot)
-		goto changedone;
-
-	    /* Move down if possible; left otherwise */
-	    tpnew = LB(tile); tile = BL(tile);
-	    if (BOTTOM(tpnew) >= BOTTOM(tile) || BOTTOM(tile) <= area->r_ybot)
-	    {
-		tile = tpnew;
-		goto enumerate;
-	    }
-	}
-	/* At left edge -- walk down to next tile along the left edge */
-	for (tile = LB(tile); RIGHT(tile) <= area->r_xbot; tile = TR(tile))
-	    /* Nothing */;
-    }
-
-changedone:
-    /*
-     * Done with the area enumeration to change the types of all tiles
-     * in this area.  Now go back and re-merge everything to form
-     * maximal horizontal strips.  The following is another in-line
-     * version of area enumeration, but is non-interruptible.
-     */
-    GOTOPOINT(tile, &start);
-
-    /* Each iteration visits another tile on the LHS of the search area */
-    while (TOP(tile) > area->r_ybot)
-    {
-	/***
-	 *** AREA SEARCH.
-	 *** Each iteration enumerates another tile.
-	 ***/
-mergenum:
-	clipTop = TOP(tile);
-	if (clipTop > area->r_ytop) clipTop = area->r_ytop;
-	oldType = TiGetTypeExact(tile);
-
-#ifdef	PAINTDEBUG
-	if (dbPaintDebug)
-	    dbPaintShowTile(tile, undo, "merge area enum");
-#endif	/* PAINTDEBUG */
-
-	/***
-	 *** ---------- THE FOLLOWING IS IN-LINE MERGE CODE ----------
-	 ***/
-
-	/* Set up initial merge directions */
-	mergeFlags = MRG_TOP | MRG_LEFT;
-	if (RIGHT(tile) >= area->r_xtop) mergeFlags |= MRG_RIGHT;
-	if (BOTTOM(tile) <= area->r_ybot) mergeFlags |= MRG_BOTTOM;
-
-	/*
-	 * Merge the tile back into the parts of the plane that have
-	 * already been visited.  Note that if we clipped in a particular
-	 * direction we avoid merging in that direction.
-	 * We avoid calling dbPaintMerge if at all possible.
-	 */
-	newType = TiGetTypeExact(tile);
-	if (mergeFlags & MRG_LEFT)
-	{
-	    for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
-		if (TiGetTypeExact(tp) == newType)
-		{
-		    tile = dbPaintMerge(tile, newType, plane, mergeFlags,
-			    (PaintUndoInfo *) NULL);
-		    goto mergedone;
-		}
-	    mergeFlags &= ~MRG_LEFT;
-	}
-	if (mergeFlags & MRG_RIGHT)
-	{
-	    for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
-		if (TiGetTypeExact(tp) == newType)
-		{
-		    tile = dbPaintMerge(tile, newType, plane, mergeFlags,
-			    (PaintUndoInfo *) NULL);
-		    goto mergedone;
-		}
-	    mergeFlags &= ~MRG_RIGHT;
-	}
-
-	/*
-	 * Cheap and dirty merge -- we don't have to merge to the
-	 * left or right, so the top/bottom merge is very fast.
-	 */
-
-	if (mergeFlags & MRG_TOP)
-	{
-	    tp = RT(tile);
-	    if (CANMERGE_Y(tile, tp)) TiJoinY(tile, tp, plane);
-#ifdef	PAINTDEBUG
-	    if (dbPaintDebug)
-		dbPaintShowTile(tile, undo, "merged up (CHEAP)");
-#endif	/* PAINTDEBUG */
-	}
-	if (mergeFlags & MRG_BOTTOM)
-	{
-	    tp = LB(tile);
-	    if (CANMERGE_Y(tile, tp)) TiJoinY(tile, tp, plane);
-#ifdef	PAINTDEBUG
-	    if (dbPaintDebug)
-		dbPaintShowTile(tile, undo, "merged down (CHEAP)");
-#endif	/* PAINTDEBUG */
-	}
-
-
-	/***
-	 ***		END OF MERGE CODE
-	 *** ---------- BACK TO AREA SEARCH ----------
-	 ***/
-mergedone:
-	/* Move right if possible */
-	tpnew = TR(tile);
-	if (LEFT(tpnew) < area->r_xtop)
-	{
-	    /* Move back down into clipping area if necessary */
-	    while (BOTTOM(tpnew) >= clipTop) tpnew = LB(tpnew);
-	    if (BOTTOM(tpnew) >= BOTTOM(tile) || BOTTOM(tile) <= area->r_ybot)
-	    {
-		tile = tpnew;
-		goto mergenum;
-	    }
-	}
-
-	/* Each iteration returns one tile further to the left */
-	while (LEFT(tile) > area->r_xbot)
-	{
-	    /* Move left if necessary */
-	    if (BOTTOM(tile) <= area->r_ybot)
-		goto done;
-
-	    /* Move down if possible; left otherwise */
-	    tpnew = LB(tile); tile = BL(tile);
-	    if (BOTTOM(tpnew) >= BOTTOM(tile) || BOTTOM(tile) <= area->r_ybot)
-	    {
-		tile = tpnew;
-		goto mergenum;
-	    }
-	}
-	/* At left edge -- walk down to next tile along the left edge */
-	for (tile = LB(tile); RIGHT(tile) <= area->r_xbot; tile = TR(tile))
-	    /* Nothing */;
-    }
-
-done:
-    plane->pl_hint = tile;
-}
 
 /*
  * ----------------------------------------------------------------------------
@@ -2078,12 +1917,13 @@ done:
  */
 
 Tile *
-dbPaintMerge(tile, newType, plane, mergeFlags, undo)
+dbPaintMerge(tile, newType, plane, mergeFlags, undo, mark)
     Tile *tile;	/* Tile to be merged with its neighbors */
     TileType newType;	/* Type to which we will change 'tile' */
     Plane *plane;		/* Plane on which this resides */
     int mergeFlags;		/* Specify which directions to merge */
     PaintUndoInfo *undo;	/* See DBPaintPlane() above */
+    bool mark;			/* Mark tiles that were processed */
 {
     Tile *tp, *tpLast;
     int ysplit;
@@ -2098,6 +1938,8 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo)
 	 * If the topmost tile is NOT of type 'newType', then the split
 	 * point will be no lower than the top of the first tile along
 	 * the LHS that is of type 'newType'.
+	 *
+	 * NOTE:  This code depends on the maximum horizontal stripes rule!
 	 */
 	for (tpLast = NULL, tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
 	    if (TiGetTypeExact(tp) == newType)
@@ -2121,6 +1963,8 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo)
 	 * If the topmost tile is NOT of type 'newType', then the split
 	 * point will be no lower than the top of the first tile along
 	 * the RHS that is of type 'newType'.
+	 *
+	 * NOTE:  This code depends on the maximum horizontal stripes rule!
 	 */
 	tp = TR(tile);
 	if (TiGetTypeExact(tp) == newType)
@@ -2164,6 +2008,7 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo)
 	DBPAINTUNDO(tile, newType, undo);
 
     TiSetBody(tile, newType);
+    if (mark) tile->ti_client = (ClientData)1;
 
 #ifdef	PAINTDEBUG
     if (dbPaintDebug)
@@ -2180,7 +2025,11 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo)
     {
 	tp = BL(tile);
 	if (TOP(tp) > TOP(tile))
-	    tpLast = TiSplitY(tp, TOP(tile)), TiSetBody(tpLast, newType);
+	{
+	    tpLast = TiSplitY(tp, TOP(tile));
+	    TiSetBody(tpLast, newType);
+	    if (mark) tile->ti_client = (ClientData)1;
+	}
 	if (BOTTOM(tp) < BOTTOM(tile)) tp = TiSplitY(tp, BOTTOM(tile));
 	TiJoinX(tile, tp, plane);
 #ifdef	PAINTDEBUG
@@ -2192,7 +2041,11 @@ dbPaintMerge(tile, newType, plane, mergeFlags, undo)
     {
 	tp = TR(tile);
 	if (TOP(tp) > TOP(tile))
-	    tpLast = TiSplitY(tp, TOP(tile)), TiSetBody(tpLast, newType);
+	{
+	    tpLast = TiSplitY(tp, TOP(tile));
+	    TiSetBody(tpLast, newType);
+	    if (mark) tile->ti_client = (ClientData)1;
+	}
 	if (BOTTOM(tp) < BOTTOM(tile)) tp = TiSplitY(tp, BOTTOM(tile));
 	TiJoinX(tile, tp, plane);
 #ifdef	PAINTDEBUG
@@ -3212,7 +3065,7 @@ dbPaintShowTile(tile, undo, str)
 
     TxPrintf("%s --more--", str); fflush(stdout);
     (void) TxGetLine(answer, sizeof answer);
-    DBWFeedbackClear();
+    DBWFeedbackClear(NULL);
 }
 #endif	/* PAINTDEBUG */
 
@@ -3272,11 +3125,9 @@ TiNMSplitY(oldtile, newtile, y, dir, undo)
     /* fprintf(stderr, "x = %d, left = %d, right = %d\n",	*/
     /*		x, LEFT(*oldtile), RIGHT(*oldtile));		*/
 
-    if (haschanged)
-    {
-	TiToRect(*oldtile, &r);
-	DBPAINTUNDO(*oldtile, TiGetTypeExact(*oldtile), undo);
-    }
+    if (haschanged) TiToRect(*oldtile, &r);
+
+    if (undo) DBPAINTUNDO(*oldtile, TiGetTypeExact(*oldtile), undo);
 
     *newtile = TiSplitY(*oldtile, y);
 
@@ -3389,10 +3240,9 @@ TiNMSplitX(oldtile, newtile, x, dir, undo)
 	y = BOTTOM(*oldtile) + dely;
 
     if (haschanged)
-    {
 	TiToRect(*oldtile, &r);
-	DBPAINTUNDO(*oldtile, TiGetTypeExact(*oldtile), undo);
-    }
+
+    if (undo) DBPAINTUNDO(*oldtile, TiGetTypeExact(*oldtile), undo);
 
     *newtile = TiSplitX(*oldtile, x);
 
