@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/dbwind/DBWcommands.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/dbwind/DBWcommands.c,v 1.2 2008/02/15 04:33:59 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -50,7 +50,7 @@ extern void CmdLabel(), CmdLoad();
 extern void CmdMove(), CmdNetlist(), CmdPaint(), CmdPath(), CmdPlow();
 extern void CmdPolygon(), CmdPort(), CmdProperty();
 extern void CmdSave(), CmdScaleGrid(), CmdSee();
-extern void CmdSelect(), CmdSideways();
+extern void CmdSelect(), CmdSetLabel(), CmdSideways();
 extern void CmdShell(), CmdSnap();
 extern void CmdStretch(), CmdStraighten();
 extern void CmdTech(), CmdTool(), CmdUnexpand();
@@ -408,6 +408,9 @@ DBWInitCommands()
 	"select [option]		change selection; type \"select help\"\n\
 			for information on options",
 	CmdSelect, FALSE);
+    WindAddCommand(DBWclientID,
+	"setlabel [option [value]]	place a label",
+	CmdSetLabel, FALSE);
     WindAddCommand(DBWclientID,
 	"shell [command]		execute a command in a subshell",
 	CmdShell, FALSE);
