@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/dbwind/DBWdisplay.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/dbwind/DBWdisplay.c,v 1.5 2008/12/11 04:20:05 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -197,6 +197,9 @@ DBWredisplay(w, rootArea, clipArea)
     crec = ((DBWclientRec *) w->w_clientData);
     cellDef = ((CellUse *)w->w_surfaceID)->cu_def;
 
+    pixelsPerLambda = w->w_scale / SUBPIXEL;
+    lambdasPerPixel = (SUBPIXEL / w->w_scale) + 1; 
+
     if ((crec->dbw_origin.p_x != w->w_origin.p_x)
 	|| (crec->dbw_origin.p_y != w->w_origin.p_y)
 	|| (crec->dbw_scale != w->w_scale)
@@ -218,12 +221,12 @@ DBWredisplay(w, rootArea, clipArea)
 	halfWireWidth = (halfWireWidth * w->w_scale) >> (SUBPIXELBITS + 1);
 	for (i = GR_TEXT_XLARGE; i >= GR_TEXT_SMALL; i--)
 	{
-	    GrLabelSize("B", -1, GEO_CENTER, i, &text);
+	    GrLabelSize("B", GEO_CENTER, i, &text);
 	    if (halfWireWidth > (text.r_ytop - text.r_ybot)) break;
 	}
 	if (i < GR_TEXT_SMALL) i = GR_TEXT_SMALL;
 	if ((3 * halfWireWidth) <= (text.r_ytop - text.r_ybot))
-	    crec->dbw_labelSize = -1;
+	    crec->dbw_labelSize = -lambdasPerPixel;
 	else crec->dbw_labelSize = i;
 
 	crec->dbw_expandAmounts.r_xbot = 0;
@@ -246,8 +249,6 @@ DBWredisplay(w, rootArea, clipArea)
      * in the area to be redisplayed.
      */
 
-    pixelsPerLambda = w->w_scale / SUBPIXEL;
-    lambdasPerPixel = (SUBPIXEL / w->w_scale) + 1; 
     labelArea = largerArea;
     (void) GeoInclude(&GrCrossRect, &crec->dbw_expandAmounts);
     if (pixelsPerLambda > 0)
@@ -331,6 +332,7 @@ DBWredisplay(w, rootArea, clipArea)
 	if (!TTMaskIsZero(&layers))
 	{
 	    TileType t, s;
+	    TileTypeBitMask *rMask;
 
 	    /* For each contact type, if the contact is not visible,	*/  
 	    /* display any of its residue layers that are visible.	*/  
@@ -338,8 +340,11 @@ DBWredisplay(w, rootArea, clipArea)
 	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
 		if (DBIsContact(t))
 		    if (!TTMaskHasType(&layers, t))
-			if (TTMaskIntersect(DBResidueMask(t), &layers))
+		    {
+			rMask = DBResidueMask(t);
+			if (TTMaskIntersect(rMask, &layers))
 			    TTMaskSetType(&layers, t);
+		    }
 
 	    /* Treat any stacking type in the mask like the	*/
 	    /* residue type whose home plane is the same as	*/
@@ -348,10 +353,13 @@ DBWredisplay(w, rootArea, clipArea)
 	    for (t = TT_TECHDEPBASE; t < DBNumUserLayers; t++)
 		if (TTMaskHasType(&layers, t))
 		    for (s = DBNumUserLayers; s < DBNumTypes; s++)
+		    {
+			rMask = DBResidueMask(s);
 			if (TTMaskHasType(&layers, s) &&
-				TTMaskHasType(DBResidueMask(s), t))
+				TTMaskHasType(rMask, t))
 			    if (DBPlane(s) != DBPlane(t))
 				TTMaskClearType(&layers, s);
+		    }
 
 	    /*
 	     * To avoid holding a lock for too long, we don't set it
@@ -410,8 +418,8 @@ DBWredisplay(w, rootArea, clipArea)
         /* Set style information beforehand */
         GrSetStuff(STYLE_LABEL);
 	(void) DBTreeSrLabels(&scontext, &DBAllTypeBits, bitMask,
-		(TerminalPath *) NULL, dbwLabelFunc, 
-		(ClientData) NULL);
+		(TerminalPath *) NULL, TF_LABEL_DISPLAY,
+		dbwLabelFunc, (ClientData) NULL);
 	GrClipTo(&rootClip);
     }
     
@@ -423,7 +431,7 @@ DBWredisplay(w, rootArea, clipArea)
 
     if (crec->dbw_flags & DBW_SEECELLS)
     {
-	GrLabelSize("BBB", -1, GEO_CENTER, GR_TEXT_SMALL, &dbwMinBBox);
+	GrLabelSize("BBB", GEO_CENTER, GR_TEXT_SMALL, &dbwMinBBox);
 	dbwMinBBox.r_xtop -= dbwMinBBox.r_xbot;
 	dbwMinBBox.r_ytop -= dbwMinBBox.r_ybot;
 
@@ -637,6 +645,7 @@ dbwPaintFunc(tile, cxp)
     return 0;
 }
 
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -661,11 +670,8 @@ dbwPaintFunc(tile, cxp)
  */
 
 void
-DBWDrawLabel(label, labrect, rect, pos, style, labelSize, sizeBox)
+DBWDrawLabel(label, rect, pos, style, labelSize, sizeBox)
     Label *label;		/* Text to be displayed. */
-    Rect *labrect;		/* Area of label's box/line/cross, in screen
-				 * coordinates.
-				 */
     Rect *rect;			/* labrect, clipped to the visible window
 				 */
     int pos;			/* Position of text relative to rect (e.g.
@@ -684,74 +690,53 @@ DBWDrawLabel(label, labrect, rect, pos, style, labelSize, sizeBox)
 				 */
 {
     Point p;
-    Rect location, *rectptr;
+    Rect location;
     char *text = label->lab_text; 
     int result;
 
-    if (style < 0) GrFastBox(rect);
-    else GrClipBox(rect, style);
+    if (style >= 0) GrSetStuff(style);
+    GrDrawFastBox(rect, labelSize);
     if (labelSize < 0) return;
 
-    rectptr = (label->lab_font < 0) ? rect : labrect;
-
     switch (pos)
     {
 	case GEO_CENTER:
-	    p.p_x = (rectptr->r_xbot + rectptr->r_xtop)/2;
-	    p.p_y = (rectptr->r_ybot + rectptr->r_ytop)/2;
+	    p.p_x = (rect->r_xbot + rect->r_xtop)/2;
+	    p.p_y = (rect->r_ybot + rect->r_ytop)/2;
 	    break;
 	case GEO_NORTH:
-	    p.p_x = (rectptr->r_xbot + rectptr->r_xtop)/2;
-	    p.p_y = rectptr->r_ytop;
+	    p.p_x = (rect->r_xbot + rect->r_xtop)/2;
+	    p.p_y = rect->r_ytop;
 	    break;
 	case GEO_NORTHEAST:
-	    p = rectptr->r_ur;
+	    p = rect->r_ur;
 	    break;
 	case GEO_EAST:
-	    p.p_x = rectptr->r_xtop;
-	    p.p_y = (rectptr->r_ybot + rectptr->r_ytop)/2;
+	    p.p_x = rect->r_xtop;
+	    p.p_y = (rect->r_ybot + rect->r_ytop)/2;
 	    break;
 	case GEO_SOUTHEAST:
-	    p.p_x = rectptr->r_xtop;
-	    p.p_y = rectptr->r_ybot;
+	    p.p_x = rect->r_xtop;
+	    p.p_y = rect->r_ybot;
 	    break;
 	case GEO_SOUTH:
-	    p.p_x = (rectptr->r_xbot + rectptr->r_xtop)/2;
-	    p.p_y = rectptr->r_ybot;
+	    p.p_x = (rect->r_xbot + rect->r_xtop)/2;
+	    p.p_y = rect->r_ybot;
 	    break;
 	case GEO_SOUTHWEST:
-	    p = rectptr->r_ll;
+	    p = rect->r_ll;
 	    break;
 	case GEO_WEST:
-	    p.p_x = rectptr->r_xbot;
-	    p.p_y = (rectptr->r_ybot + rectptr->r_ytop)/2;
+	    p.p_x = rect->r_xbot;
+	    p.p_y = (rect->r_ybot + rect->r_ytop)/2;
 	    break;
 	case GEO_NORTHWEST:
-	    p.p_x = rectptr->r_xbot;
-	    p.p_y = rectptr->r_ytop;
+	    p.p_x = rect->r_xbot;
+	    p.p_y = rect->r_ytop;
 	    break;
     }
-
-    if (label->lab_font >= 0)
-    {
-	Point offset;
-	Rect surfloc;
-	int scale = dbwWindow->w_scale >> SUBPIXELBITS;
-
-	/* Offsets are in quarter-lambda units (downshift by 2) */
-	offset.p_x = (label->lab_offset.p_x * scale) >> 2;
-	offset.p_y = (label->lab_offset.p_y * scale) >> 2;
-
-	result = GrFontText(text, style, &p, label->lab_font,
-		label->lab_size * scale, label->lab_rotate,
-		&offset, pos, &GrScreenRect, &surfloc);
-	WindSurfaceToScreen(dbwWindow, &surfloc, &location);
-    }
-    else
-	result = GrPutText(text, style, &p, pos, labelSize, FALSE,
-			&GrScreenRect, &location);
-
-    if (result)
+    if (GrPutText(text, style, &p, pos, labelSize, FALSE,
+			&GrScreenRect, &location))
     {
 	sizeBox->r_xbot = MIN(sizeBox->r_xbot, location.r_xbot - p.p_x);
 	sizeBox->r_ybot = MIN(sizeBox->r_ybot, location.r_ybot - p.p_y);
@@ -760,6 +745,152 @@ DBWDrawLabel(label, labrect, rect, pos, style, labelSize, sizeBox)
     }
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBWDrawFontLabel --
+ *
+ *	This procedure is like DBWDrawLabel but handles rendered font
+ *	labels.  It is invoked by the label redisplay code in dbwind
+ *	and select.  Unlike DBWDrawLabel, it does all the layout-to-screen
+ *	transformations itself.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The label given by text, rect, and pos is drawn on the screen
+ *	in style "style" and size "labelSize".  The caller must have set
+ *	up all of the clipping information for the window in which the
+ *	label is to be drawn.  This procedure updates sizeBox to be
+ *	large enough to hold the total pixel area occupied by the label
+ *	(assuming it were drawn at (0,0)).
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBWDrawFontLabel(label, window, trans, style)
+    Label *label;
+    MagWindow *window;
+    Transform *trans;
+    int style;		/* If -1, style is already set */
+{
+    Point *p, newcorner, scrncorners[4], labOrigin;
+    Rect rootArea, labrect;
+    int i, rotate, minv, scaledLambdasPerPixel;
+    dlong tmp, dval, scale;
+    double rads;
+
+    GeoTransRect(trans, &label->lab_rect, &rootArea);
+    WindSurfaceToScreen(window, &rootArea, &labrect);
+    labOrigin.p_x = (rootArea.r_xbot + rootArea.r_xtop) << 2;
+    labOrigin.p_y = (rootArea.r_ybot + rootArea.r_ytop) << 2;
+
+    /* This bit of code sets the scale for the "cross" associated with	*/
+    /* a label as a logarithmic function of the window scale.  This is	*/
+    /* completely subjective, but seems to produce a decent result.	*/
+
+    scaledLambdasPerPixel = 0;
+    i = (SUBPIXEL / window->w_scale);
+    while (i != 0)
+    {
+	i >>= 1;
+	scaledLambdasPerPixel++;
+    }
+
+    if (style >= 0) GrSetStuff(style);
+    GrDrawFastBox(&labrect, -scaledLambdasPerPixel);
+
+    for (i = 0; i < 4; i++)
+    {
+	GeoTransPointDelta(trans, &label->lab_corners[i], &newcorner);
+
+	/* Effectively, WindPointToScreen(), but with an	*/
+	/* extra scalefactor of 8, including computing the 	*/
+	/* (unclipped) origin from the center of rootArea.	*/
+
+	tmp = labOrigin.p_x + newcorner.p_x;
+	tmp -= (dlong)window->w_surfaceArea.r_xbot << 3;
+	dval = ((dlong)window->w_origin.p_x << 3) + (dlong)(tmp * window->w_scale);
+	scrncorners[i].p_x = (int)(dval >> (SUBPIXELBITS + 3));
+
+	tmp = labOrigin.p_y + newcorner.p_y;
+	tmp -= (dlong)window->w_surfaceArea.r_ybot << 3;
+	dval = ((dlong)window->w_origin.p_y << 3) + (dlong)(tmp * window->w_scale);
+	scrncorners[i].p_y = (int)(dval >> (SUBPIXELBITS + 3));
+    }
+
+    /* Ensure that the label is always drawn with text upright.	*/
+    /* Compute rotation from the transformed baseline angle.	*/
+    /* (The "2" is a slop factor to account for round-off error	*/
+    /* in the computation of scrncorners.)			*/
+
+    rotate = GeoTransAngle(trans, label->lab_rotate);
+    if ((rotate >= 0 && rotate < 90) || (rotate >= 180 && rotate < 270))
+    {
+	/* Startpoint is the bottommost point.   Due to roundoff error,	*/
+	/* we need to watch closely when the angle is close to a	*/
+	/* multiple of 90 degrees.					*/
+
+	minv = scrncorners[0].p_y;
+	p = &scrncorners[0];
+	for (i = 1; i < 4; i++)
+	{
+	    if ((scrncorners[i].p_y - 2) < minv)
+	    {
+		if ((scrncorners[i].p_y + 2) > minv)
+		{
+		    if (((rotate < 5) || (rotate >= 180 && rotate < 185)) &&
+				(scrncorners[i].p_x > p->p_x))
+			continue;
+		    if (((rotate > 85 && rotate < 90) || (rotate > 265)) &&
+				(scrncorners[i].p_x < p->p_x))
+			continue;
+		}
+		minv = scrncorners[i].p_y;
+		p = &scrncorners[i];
+	    }
+	}
+    }
+    else
+    {
+	/* startpoint is the leftmost point */
+	minv = scrncorners[0].p_x;
+	p = &scrncorners[0];
+	for (i = 1; i < 4; i++)
+	{
+	    if ((scrncorners[i].p_x - 2) < minv)
+	    {
+		if ((scrncorners[i].p_x + 2) > minv)
+		{
+		    if (((rotate < 95) || (rotate >= 270 && rotate < 275)) &&
+				(scrncorners[i].p_y < p->p_y))
+			continue;
+		    if (((rotate > 175 && rotate < 180) || (rotate > 355)) &&
+				(scrncorners[i].p_y > p->p_y))
+			continue;
+		}
+		minv = scrncorners[i].p_x;
+		p = &scrncorners[i];
+	    }
+	}
+    }
+    if (rotate >= 90 && rotate < 270)
+    {
+	rotate += 180;
+	if (rotate >= 360) rotate -= 360;
+    }
+
+    scale = ((dlong)window->w_scale * (dlong)label->lab_size) >> (SUBPIXELBITS + 3);
+
+    if (scale > 0)
+    {
+	GrFontText(label->lab_text, style, p, label->lab_font,
+			(int)scale, rotate, &GrScreenRect);
+    }
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -788,21 +919,42 @@ dbwLabelFunc(scx, label, tpath)
     TerminalPath *tpath;	/* Contains pointer to full pathname of label */
 {
     Rect labRect, labRectNoClip, tmp;
-    int screenPos;
+    int screenPos, screenRot, newStyle;
 
-    screenPos = GeoTransPos(&scx->scx_trans, label->lab_just);
-    GeoTransRect(&scx->scx_trans, &label->lab_rect, &tmp);
-    WindSurfaceToScreen(dbwWindow, &tmp, &labRect);
-    WindSurfaceToScreenNoClip(dbwWindow, &tmp, &labRectNoClip);
-
-    /* Let the label drawing routine determine if the label is in the window */
-    /* if (!GEO_TOUCH(&labRect, &dbwWindow->w_screenArea)) return 0; */
+    if (!dbwAllSame && ((editDef != scx->scx_use->cu_def)
+	|| (scx->scx_trans.t_a != editTrans.t_a)
+	|| (scx->scx_trans.t_b != editTrans.t_b)
+	|| (scx->scx_trans.t_c != editTrans.t_c)
+	|| (scx->scx_trans.t_d != editTrans.t_d)
+	|| (scx->scx_trans.t_e != editTrans.t_e)
+	|| (scx->scx_trans.t_f != editTrans.t_f)))
+	disWasPale = TRUE;
+    else
+	disWasPale = FALSE;
 
     if (label->lab_flags & PORT_DIR_MASK)
-        GrSetStuff(STYLE_PORT);
+	newStyle = (disWasPale) ? STYLE_PORT_PALE : STYLE_PORT;
+    else
+	newStyle = (disWasPale) ? STYLE_LABEL_PALE : STYLE_LABEL;
+
+    if (newStyle != disStyle)
+    {
+	disStyle = newStyle;
+        GrSetStuff(newStyle);
+    }
 	
-    DBWDrawLabel(label, &labRectNoClip, &labRect, screenPos, -1,
-	dbwLabelSize, dbwExpandAmounts);
+    if (label->lab_font < 0)
+    {
+	screenPos = GeoTransPos(&scx->scx_trans, label->lab_just);
+	GeoTransRect(&scx->scx_trans, &label->lab_rect, &tmp);
+	WindSurfaceToScreen(dbwWindow, &tmp, &labRect);
+	if (!GEO_TOUCH(&labRect, &dbwWindow->w_screenArea)) return 0;
+	DBWDrawLabel(label, &labRect, screenPos, -1, dbwLabelSize, dbwExpandAmounts);
+    }
+    else
+    {
+	DBWDrawFontLabel(label, dbwWindow, &scx->scx_trans, -1);
+    }
 
     if (label->lab_flags & PORT_DIR_MASK)
     {
@@ -820,9 +972,7 @@ dbwLabelFunc(scx, label, tpath)
 	if (label->lab_flags & PORT_DIR_WEST)
 	    GrClipLine(labRectNoClip.r_xbot, labRectNoClip.r_ybot,
 			labRectNoClip.r_xbot, labRectNoClip.r_ytop);
-
-	/* Revert back to label style */
-        GrSetStuff(STYLE_LABEL);
+        GrSetStuff(disStyle);
     }
     return 0;
 }
@@ -1338,10 +1488,6 @@ dbwChangedFunc(w, area)
 
 /* The following stuff is shared by the procedures below. */
 
-static Rect boxArea;		/* Area of label box (not including text)
-				 * in root cell coords.
-				 */
-static int rootPos;		/* Label position in root cell coords. */
 extern int dbwLabelChangedFunc();  /* Function to call for each label. */
 
 void
@@ -1377,10 +1523,8 @@ DBWLabelChanged(cellDef, lab, mask)
 	     * and do the rest of the processing on a per-window basis.
 	     */
 
-	    boxArea = lab->lab_rect;
-	    rootPos = lab->lab_just;
 	    (void) WindSearch((ClientData) DBWclientID, (ClientData) use,
-		(Rect *) NULL, dbwLabelChangedFunc, (ClientData) lab->lab_text);
+		(Rect *) NULL, dbwLabelChangedFunc, (ClientData) lab);
 	    continue;
 	}
 
@@ -1420,28 +1564,33 @@ DBWLabelChanged(cellDef, lab, mask)
 }
 
 int
-dbwLabelChangedFunc(w, text)
-    MagWindow *w;			/* Window in which label is displayed. */
-    char *text;			/* Text of label.  BoxArea gives positioning
-				 * point and pos gives relative location.
-				 */
+dbwLabelChangedFunc(w, lab)
+    MagWindow *w;		/* Window in which label is displayed. */
+    Label *lab;			/* Label being changed.	*/
 {
     Rect screenArea, textArea;
     int size;
 
-    WindSurfaceToScreen(w, &boxArea, &screenArea);
-    size = ((DBWclientRec *) w->w_clientData)->dbw_labelSize;
-    if (size < GR_TEXT_SMALL)
-	textArea = GrCrossRect;
+    if (lab->lab_font < 0)
+    {
+	WindSurfaceToScreen(w, &lab->lab_rect, &screenArea);
+	size = ((DBWclientRec *) w->w_clientData)->dbw_labelSize;
+	if (size < GR_TEXT_SMALL)
+	    textArea = GrCrossRect;
+	else
+	{
+	    GrLabelSize(lab->lab_text, lab->lab_just, size, &textArea);
+	    (void) GeoInclude(&GrCrossRect, &textArea);
+	}
+	screenArea.r_xbot += textArea.r_xbot;
+	screenArea.r_ybot += textArea.r_ybot;
+	screenArea.r_xtop += textArea.r_xtop;
+	screenArea.r_ytop += textArea.r_ytop;
+    }
     else
     {
-	GrLabelSize(text, -1, rootPos, size, &textArea);
-	(void) GeoInclude(&GrCrossRect, &textArea);
+	WindSurfaceToScreen(w, &lab->lab_bbox, &screenArea);
     }
-    screenArea.r_xbot += textArea.r_xbot;
-    screenArea.r_ybot += textArea.r_ybot;
-    screenArea.r_xtop += textArea.r_xtop;
-    screenArea.r_ytop += textArea.r_ytop;
     WindAreaChanged(w, &screenArea);
     return 0;
 }
@@ -1549,6 +1698,7 @@ DBWTechAddStyle(sectionName, argc, argv)
     char *argv[];
 {
     TileType t, r;
+    TileTypeBitMask *rMask;
     int i, sidx;
     static char styleType[50];
     char *pathptr;
@@ -1612,9 +1762,11 @@ DBWTechAddStyle(sectionName, argc, argv)
 
 		if (DBIsContact(t) && (t < DBNumUserLayers))
 		    for (r = DBNumUserLayers; r < DBNumTypes; r++)
-			if (TTMaskHasType(DBResidueMask(r), t)
-				&& (DBPlane(r) == DBPlane(t)))
+		    {
+			rMask = DBResidueMask(r);
+			if (TTMaskHasType(rMask, t) && (DBPlane(r) == DBPlane(t)))
 			    TTMaskSetType(&DBWStyleToTypesTbl[sidx], r);
+		    }
 	    }
 	}
     }
