@@ -20,7 +20,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCbasic.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCbasic.c,v 1.3 2008/12/04 16:21:44 tim Exp $";
 #endif	/* not lint */
 
 #include <sys/types.h>
@@ -32,6 +32,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "database/database.h"
 #include "drc/drc.h"
 #include "utils/signals.h"
+#include "utils/maxrect.h"
 
 int dbDRCDebug = 0;
 
@@ -52,6 +53,69 @@ static DRCCookie drcOverlapCookie = {
 
 extern int areaCheck();
 extern int drcTile();
+extern MaxRectsData *drcCanonicalMaxwidth();
+
+#ifdef NONMANHATTAN
+
+/*
+ *-----------------------------------------------------------------------
+ *
+ * point_to_segment
+ *
+ *	Euclidean-distance point-to-segment distance (squared)
+ *	calculation (borrowed from XCircuit)
+ *
+ * Results:
+ *	Squared Euclidean distance of the closest approach of the
+ *	line segment to the point (long result).
+ *
+ * Side Effects:
+ *	None.
+ *
+ *-----------------------------------------------------------------------
+ */
+
+long
+point_to_segment(px, py, s1x, s1y, s2x, s2y)
+    int px, py;		/* The position of the point */
+    int s1x, s1y;	/* One endpoint of the line segment */
+    int s2x, s2y;	/* The other endpoint of the line segment */
+{
+    long x, y;
+    long a, b, c, frac;
+    float protod;
+
+    x = (long)s2x - (long)s1x;
+    y = (long)s2y - (long)s1y;
+    c = (x * x + y * y);
+
+    x = (long)px - (long)s1x;
+    y = (long)py - (long)s1y;
+    a = (x * x + y * y);
+
+    x = (long)px - (long)s2x;
+    y = (long)py - (long)s2y;
+    b = (x * x + y * y);
+
+    frac = a - b;
+    if (frac >= c) return b;
+    else if (-frac >= c) return a;
+    else
+    {
+	protod = (float)(c + a - b);
+	return (a - (long)((protod * protod) / (float)(c << 2)));
+    }
+}
+
+#endif	/* NONMANHATTAN */
+
+/* Define Euclidean distance checks */
+
+#define RADIAL_NW	0x1000
+#define RADIAL_NE	0x8000
+#define RADIAL_SW	0x2000
+#define RADIAL_SE	0x4000
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -84,23 +148,6 @@ areaCheck(tile, arg)
 {
     Rect rect;		/* Area where error is to be recorded. */
 
-#ifdef NONMANHATTAN
-
-    /* If the tile has a legal type, then return.  This is where we	*/
-    /* would want to add extra processing to check whether the half of	*/
-    /* the tile that is of an illegal type is inside or outside the	*/
-    /* area of interest.						*/
-
-    if (IsSplit(tile))
-    {
-	if (TTMaskHasType(&arg->dCD_cptr->drcc_mask, SplitLeftType(tile)) &&
-		TTMaskHasType(&arg->dCD_cptr->drcc_mask, SplitRightType(tile)))
-	    return 0;
-    }
-    else 
-#endif
-    if (TTMaskHasType(&arg->dCD_cptr->drcc_mask, TiGetType(tile))) return 0;
-
     TiToRect(tile, &rect);
 
     /* Only consider the portion of the suspicious tile that overlaps
@@ -114,42 +161,100 @@ areaCheck(tile, arg)
     if ((rect.r_xbot >= rect.r_xtop) || (rect.r_ybot >= rect.r_ytop))
 	return 0;
 
-    /* Apply extra clipping for radial distance check on corner
-     * extensions.
+    /* 
+     * When Euclidean distance checks are enabled, check for error tiles
+     * outside of the perimeter of the circle in the corner extension area
+     * that extends "sdist" from the corner of the edge.
+     *
+     * Also check the relatively rare case where the tile is inside the
+     * circle perimeter, but only the corner of the triangle projects into
+     * the error check rectangle, and is outside of the circle.
      */
 
     if (arg->dCD_radial != 0)
-    {   
+    {
 	unsigned int i;
 	int sqx, sqy;
 	int sdist = arg->dCD_radial & 0xfff;
-	long ssdist = sdist * sdist;
-
-	for (i = 0x1000; i <= 0x8000; i <<= 1)
+	long sstest, ssdist = sdist * sdist;
+	
+	if ((arg->dCD_radial & RADIAL_NW) != 0)
 	{
-	    if ((arg->dCD_radial & i) != 0)
+	    if (((sqx = arg->dCD_constraint->r_xbot + sdist
+			 - rect.r_xtop) >= 0) && ((sqy = rect.r_ybot
+			- arg->dCD_constraint->r_ytop + sdist) >= 0)
+			&& ((sqx * sqx + sqy * sqy) >= ssdist))
+		return 0;
+#ifdef NONMANHATTAN
+	    else if (IsSplit(tile) && !SplitDirection(tile) && !SplitSide(tile))
 	    {
-	 	/* check for correct orientation */
-		if ((sqx = (i == 0x4000 || i == 0x8000) ?
-	  		rect.r_xbot - arg->dCD_constraint->r_xtop + sdist:
-	  		arg->dCD_constraint->r_xbot + sdist - rect.r_xtop) < 0)
-	  	    continue;
-		if ((sqy = (i == 0x2000 || i == 0x8000) ?
-			rect.r_ybot - arg->dCD_constraint->r_ytop + sdist:
-			arg->dCD_constraint->r_ybot + sdist - rect.r_ytop) < 0)
-		    continue;
-
-		if ((sqx * sqx + sqy * sqy) >= ssdist)
-		    return 0;
+		sstest = point_to_segment(arg->dCD_constraint->r_xbot + sdist,
+			arg->dCD_constraint->r_ytop - sdist,
+			LEFT(tile), BOTTOM(tile), RIGHT(tile), TOP(tile));
+		if (sstest > ssdist) return 0;
 	    }
+#endif
+	}
+	if ((arg->dCD_radial & RADIAL_NE) != 0)
+	{
+	    if (((sqx = rect.r_xbot - arg->dCD_constraint->r_xtop
+			+ sdist) >= 0) && ((sqy = rect.r_ybot
+			- arg->dCD_constraint->r_ytop + sdist) >= 0)
+			&& ((sqx * sqx + sqy * sqy) >= ssdist))
+		return 0;
+#ifdef NONMANHATTAN
+	    else if (IsSplit(tile) && SplitDirection(tile) && SplitSide(tile))
+	    {
+		sstest = point_to_segment(arg->dCD_constraint->r_xtop - sdist,
+			arg->dCD_constraint->r_ytop - sdist,
+			LEFT(tile), TOP(tile), RIGHT(tile), BOTTOM(tile));
+		if (sstest > ssdist) return 0;
+	    }
+#endif
+	}
+	if ((arg->dCD_radial & RADIAL_SW) != 0)
+	{
+	    if (((sqx = arg->dCD_constraint->r_xbot + sdist
+			- rect.r_xtop) >= 0) &&
+			((sqy = arg->dCD_constraint->r_ybot
+			+ sdist - rect.r_ytop) >= 0)
+			&& ((sqx * sqx + sqy * sqy) >= ssdist))
+		return 0;
+#ifdef NONMANHATTAN
+	    else if (IsSplit(tile) && SplitDirection(tile) && !SplitSide(tile))
+	    {
+		sstest = point_to_segment(arg->dCD_constraint->r_xbot + sdist,
+			arg->dCD_constraint->r_ybot + sdist,
+			LEFT(tile), TOP(tile), RIGHT(tile), BOTTOM(tile));
+		if (sstest > ssdist) return 0;
+	    }
+#endif
+	}
+	if ((arg->dCD_radial & RADIAL_SE) != 0)
+	{
+	    if (((sqx = rect.r_xbot - arg->dCD_constraint->r_xtop
+			+ sdist) >= 0) &&
+			((sqy = arg->dCD_constraint->r_ybot
+			+ sdist - rect.r_ytop) >= 0)
+			&& ((sqx * sqx + sqy * sqy) >= ssdist))
+		return 0;
+#ifdef NONMANHATTAN
+	    else if (IsSplit(tile) && !SplitDirection(tile) && SplitSide(tile))
+	    {
+		sstest = point_to_segment(arg->dCD_constraint->r_xtop - sdist,
+			arg->dCD_constraint->r_ybot + sdist,
+			LEFT(tile), BOTTOM(tile), RIGHT(tile), TOP(tile));
+		if (sstest > ssdist) return 0;
+	    }
+#endif
 	}
     }
 
     if (arg->dCD_cptr->drcc_flags & DRC_TRIGGER)
 	return 1;
 
-    (*(arg->dCD_function)) (arg->dCD_celldef, &rect,
-	arg->dCD_cptr, arg->dCD_clientData);
+    (*(arg->dCD_function))(arg->dCD_celldef, &rect, arg->dCD_cptr,
+			arg->dCD_clientData);
     (*(arg->dCD_errors))++;
     return 0;
 }
@@ -263,8 +368,10 @@ drcTile (tile, arg)
     DRCCookie *cptr;	/* Current design rule on list */
     Rect *rect = arg->dCD_rect;	/* Area being checked */
     Rect errRect;		/* Area checked for an individual rule */
+    MaxRectsData *mrd;		/* Used by widespacing rule */
     TileTypeBitMask tmpMask, *rMask;
-    bool triggered, trigpending;	/* Hack for widespacing rule */
+    bool trigpending;	/* Hack for widespacing rule */
+    int triggered;
     int cdist, dist, ccdist, result;
 
     arg->dCD_constraint = &errRect;
@@ -392,27 +499,90 @@ drcTile (tile, arg)
 	    if (edgeTop <= edgeBot)
 		continue;
 
-	    triggered = FALSE;
+	    triggered = 0;
 	    for (cptr = DRCCurStyle->DRCRulesTbl[to][tt]; cptr != (DRCCookie *) NULL;
 			cptr = cptr->drcc_next)
 	    {
-		    if (cptr->drcc_flags & DRC_ANGLES) continue;
+		if (cptr->drcc_flags & DRC_ANGLES) continue;
 
-		    /* Find the rule distances according to the scale factor */
-		    dist = cptr->drcc_dist;
-		    cdist = cptr->drcc_cdist;
-		    trigpending = (cptr->drcc_flags & DRC_TRIGGER) ? TRUE : FALSE;
+		/* Find the rule distances according to the scale factor */
+		dist = cptr->drcc_dist;
+		cdist = cptr->drcc_cdist;
+		trigpending = (cptr->drcc_flags & DRC_TRIGGER) ? TRUE : FALSE;
 
-		    /* drcc_edgeplane is used to avoid checks on edges	*/
-		    /* in more than one plane				*/
+		/* drcc_edgeplane is used to avoid checks on edges	*/
+		/* in more than one plane				*/
 
-		    if (arg->dCD_plane != cptr->drcc_edgeplane)
+		if (arg->dCD_plane != cptr->drcc_edgeplane)
+		{
+		    if (trigpending) cptr = cptr->drcc_next;
+		    continue;
+		}
+
+		DRCstatRules++;
+
+		if (cptr->drcc_flags & DRC_AREA)
+		{
+		    drcCheckArea(tile, arg, cptr);
+		    continue;
+		}
+
+		if ((cptr->drcc_flags & (DRC_MAXWIDTH | DRC_BENDS)) ==
+				(DRC_MAXWIDTH | DRC_BENDS))
+		{
+		    /* New algorithm --- Tim 3/6/05 */
+		    Rect *lr;
+		    int i;
+
+		    if (!trigpending) cptr->drcc_dist++;
+
+		    if (cptr->drcc_flags & DRC_REVERSE)
+			mrd = drcCanonicalMaxwidth(tpleft, GEO_WEST, arg, cptr);
+		    else
+			mrd = drcCanonicalMaxwidth(tile, GEO_EAST, arg, cptr);
+		    if (!trigpending) cptr->drcc_dist--;
+		    if (trigpending)
 		    {
-			if (trigpending) cptr = cptr->drcc_next;
-			continue;
+			if (mrd)
+			    triggered = mrd->entries;
+			else
+			    cptr = cptr->drcc_next;
 		    }
+		    else if (mrd)
+		    {
+			for (i = 0; i < mrd->entries; i++)
+			{
+			    lr = &mrd->rlist[i];
+			    GeoClip(lr, arg->dCD_clip);
+			    if (!GEO_RECTNULL(lr))  
+			    {
+				(*(arg->dCD_function)) (arg->dCD_celldef,
+					lr, cptr, arg->dCD_clientData);
+				(*(arg->dCD_errors))++;
+			    }
+			}
+		    }
+		    continue;
+		}
+		else if (cptr->drcc_flags & DRC_MAXWIDTH)
+		{
+		    /* bends_illegal option only */
+		    drcCheckMaxwidth(tile, arg, cptr);
+		    continue;
+		}
+		if (cptr->drcc_flags & DRC_RECTSIZE)
+		{
+		    /* only checked for bottom-left tile in a rect area */
+		    if (!TTMaskHasType(&cptr->drcc_mask,
+				TiGetRightType(BL(tile))) &&
+				!TTMaskHasType(&cptr->drcc_mask,
+				TiGetTopType(LB(tile))))
+			drcCheckRectSize(tile, arg, cptr);
+		    continue;
+		}
 
-		    DRCstatRules++;
+		result = 0;
+		do {
 		    if (triggered)
 		    {
 			/* Widespacing hack---rule is triggered by MAXWIDTH	*/
@@ -421,11 +591,11 @@ drcTile (tile, arg)
 			/* is incorrectly flagged on thin wires that happen to	*/
 			/* share a common edge with the maxwidth region.	*/
 
+			errRect = mrd->rlist[--triggered];
 			errRect.r_ytop += cdist;
 			errRect.r_ybot -= cdist;
 			if (errRect.r_ytop > edgeTop) errRect.r_ytop = edgeTop;
 			if (errRect.r_ybot < edgeBot) errRect.r_ybot = edgeBot;
-			triggered = FALSE;
 		    }
 		    else
 		    {
@@ -433,64 +603,6 @@ drcTile (tile, arg)
 			errRect.r_ybot = edgeBot;
 		    }
 
-		    if (cptr->drcc_flags & DRC_AREA)
-		    {
-			drcCheckArea(tile, arg, cptr);
-			continue;
-		    }
-
-		    if ((cptr->drcc_flags & (DRC_MAXWIDTH | DRC_BENDS)) ==
-				(DRC_MAXWIDTH | DRC_BENDS))
-		    {
-			/* New algorithm --- Tim 3/6/05 */
-			Rect *lr;
-
-		        if (!trigpending) cptr->drcc_dist++;
-
-			if (cptr->drcc_flags & DRC_REVERSE)
-			    lr = drcCanonicalMaxwidth(tpleft, GEO_WEST, arg, cptr);
-			else
-			    lr = drcCanonicalMaxwidth(tile, GEO_EAST, arg, cptr);
-		        if (!trigpending) cptr->drcc_dist--;
-		        if (trigpending)
-			{
-			    if (lr)
-			    {
-			        errRect = *lr;
-				triggered = TRUE;
-			    }
-			    else
-				cptr = cptr->drcc_next;
-			}
-			else if (lr)
-			{
-		            GeoClip(lr, arg->dCD_clip);
-			    if (!GEO_RECTNULL(lr))  
-			    {
-				(*(arg->dCD_function)) (arg->dCD_celldef,
-					lr, cptr, arg->dCD_clientData);
-				(*(arg->dCD_errors))++;
-			    }
-			}
-			continue;
-		    }
-		    else if (cptr->drcc_flags & DRC_MAXWIDTH)
-		    {
-			/* bends_illegal option only */
-			drcCheckMaxwidth(tile, arg, cptr);
-			continue;
-		    }
-		    if (cptr->drcc_flags & DRC_RECTSIZE)
-		    {
-		        /* only checked for bottom-left tile in a rect area */
-		        if (!TTMaskHasType(&cptr->drcc_mask,
-					TiGetRightType(BL(tile))) &&
-					!TTMaskHasType(&cptr->drcc_mask,
-					TiGetTopType(LB(tile))))
-			    drcCheckRectSize(tile, arg, cptr);
-			continue;
-		    }
-
 		    if (cptr->drcc_flags & DRC_REVERSE)
 		    {
 			/*
@@ -514,7 +626,7 @@ drcTile (tile, arg)
 		 	    {
 				errRect.r_ybot -= cdist;
 				if (DRCEuclidean)
-				    arg->dCD_radial |= 0x1000;
+				    arg->dCD_radial |= RADIAL_SW;
 			    }
 			}
 
@@ -542,7 +654,7 @@ drcTile (tile, arg)
 				{
 				    errRect.r_ytop += cdist;
 				    if (DRCEuclidean)
-					arg->dCD_radial |= 0x2000;
+					arg->dCD_radial |= RADIAL_NW;
 				}
 			    }
 			}
@@ -558,11 +670,8 @@ drcTile (tile, arg)
 				&& arg->dCD_plane == cptr->drcc_plane
 				&& TTMaskHasType(&cptr->drcc_mask,
 				TiGetRightType(tpleft)))
-			{
-			    if (trigpending)
-				cptr = cptr->drcc_next;
 			    continue;
-			}
+
 			errRect.r_xtop = edgeX;
 			arg->dCD_initial = tile;
 		    }
@@ -589,7 +698,7 @@ drcTile (tile, arg)
 			    {
 				errRect.r_ytop += cdist;
 				if (DRCEuclidean)
-				    arg->dCD_radial |= 0x8000;
+				    arg->dCD_radial |= RADIAL_NE;
 			    }
 			}
 
@@ -616,7 +725,7 @@ drcTile (tile, arg)
 				{
 				    errRect.r_ybot -= cdist;
 				    if (DRCEuclidean)
-					arg->dCD_radial |= 0x4000;
+					arg->dCD_radial |= RADIAL_SE;
 				}
 			    }
 			}
@@ -632,11 +741,8 @@ drcTile (tile, arg)
 				&& arg->dCD_plane == cptr->drcc_plane
 				&& TTMaskHasType(&cptr->drcc_mask,
 				TiGetLeftType(tile)))
-			{
-			    if (trigpending)
-				cptr = cptr->drcc_next;
 			    continue;
-			}
+
 			errRect.r_xbot = edgeX;
 			arg->dCD_initial= tpleft;
 		    }
@@ -650,20 +756,21 @@ drcTile (tile, arg)
 		    arg->dCD_cptr = cptr;
 		    TTMaskCom2(&tmpMask, &cptr->drcc_mask);
 
-		    result = DBSrPaintArea((Tile *) NULL,
+		    result += DBSrPaintArea((Tile *) NULL,
 				arg->dCD_celldef->cd_planes[cptr->drcc_plane],
 				&errRect, &tmpMask, areaCheck, (ClientData) arg);
+		} while (triggered);
 
-		    if (result == 0)
-		    {
-		        /* Trigger rule:  If rule check found errors,	*/
-		        /* do the next rule.  Otherwise, skip it.	*/
+		if (result == 0)
+		{
+		    /* Trigger rule:  If rule check found errors,	*/
+		    /* do the next rule.  Otherwise, skip it.		*/
 
-			if (trigpending)
-			    cptr = cptr->drcc_next;
-		    }
+		    if (trigpending)
+			cptr = cptr->drcc_next;
 		}
-		DRCstatEdges++;
+	    }
+	    DRCstatEdges++;
         }
     }
 
@@ -741,85 +848,92 @@ checkbottom:
 	    if (edgeLeft >= edgeRight)
 		continue;
 
-	    triggered = FALSE;
+	    triggered = 0;
 	    for (cptr = DRCCurStyle->DRCRulesTbl[to][tt]; cptr != (DRCCookie *) NULL;
 				cptr = cptr->drcc_next)
 	    {
-		    if (cptr->drcc_flags & DRC_ANGLES) continue;
+		if (cptr->drcc_flags & DRC_ANGLES) continue;
 
-		    /* Find the rule distances according to the scale factor */
-		    dist = cptr->drcc_dist;
-		    cdist = cptr->drcc_cdist;
-		    trigpending = (cptr->drcc_flags & DRC_TRIGGER) ? TRUE : FALSE;
+		/* Find the rule distances according to the scale factor */
+		dist = cptr->drcc_dist;
+		cdist = cptr->drcc_cdist;
+		trigpending = (cptr->drcc_flags & DRC_TRIGGER) ? TRUE : FALSE;
 
-		    /* drcc_edgeplane is used to avoid checks on edges	*/
-		    /* in more than one plane				*/
+		/* drcc_edgeplane is used to avoid checks on edges	*/
+		/* in more than one plane				*/
 
-		    if (arg->dCD_plane != cptr->drcc_edgeplane)
-		    {
-			if (trigpending) cptr = cptr->drcc_next;
-			continue;
-		    }
+		if (arg->dCD_plane != cptr->drcc_edgeplane)
+		{
+		    if (trigpending) cptr = cptr->drcc_next;
+		    continue;
+		}
 
-		    DRCstatRules++;
-		    if (triggered)
-		    {
-			errRect.r_xtop += cdist;
-			errRect.r_xbot -= cdist;
-			if (errRect.r_xtop > edgeRight) errRect.r_xtop = edgeRight;
-			if (errRect.r_xbot < edgeLeft) errRect.r_xbot = edgeLeft;
-			triggered = FALSE;
-		    }
-		    else
-		    {
-			errRect.r_xbot = edgeLeft;
-			errRect.r_xtop = edgeRight;
-		    }
+		DRCstatRules++;
 
-		    /* top to bottom */
+		/* top to bottom */
 
-		    if ((cptr->drcc_flags & (DRC_MAXWIDTH | DRC_BENDS)) ==
+		if ((cptr->drcc_flags & (DRC_MAXWIDTH | DRC_BENDS)) ==
 				(DRC_MAXWIDTH | DRC_BENDS))
+		{
+		    /* New algorithm --- Tim 3/6/05 */
+		    Rect *lr;
+		    int i;
+
+		    if (!trigpending) cptr->drcc_dist++;
+
+		    if (cptr->drcc_flags & DRC_REVERSE)
+			mrd = drcCanonicalMaxwidth(tpbot, GEO_SOUTH, arg, cptr);
+		    else
+			mrd = drcCanonicalMaxwidth(tile, GEO_NORTH, arg, cptr);
+		    if (!trigpending) cptr->drcc_dist--;
+		    if (trigpending)
 		    {
-			/* New algorithm --- Tim 3/6/05 */
-			Rect *lr;
-
-		        if (!trigpending) cptr->drcc_dist++;
-
-			if (cptr->drcc_flags & DRC_REVERSE)
-			    lr = drcCanonicalMaxwidth(tpbot, GEO_SOUTH, arg, cptr);
+			if (mrd)
+			    triggered = mrd->entries;
 			else
-			    lr = drcCanonicalMaxwidth(tile, GEO_NORTH, arg, cptr);
-		        if (!trigpending) cptr->drcc_dist--;
-		        if (trigpending)
+			    cptr = cptr->drcc_next;
+		    }
+		    else if (mrd)
+		    {
+			for (i = 0; i < mrd->entries; i++)
 			{
-			    if (lr)
-			    {
-				errRect = *lr;
-				triggered = TRUE;
-			    }
-			    else
-				cptr = cptr->drcc_next;
-			}
-			else if (lr)
-			{
-		            GeoClip(lr, arg->dCD_clip);
+			    lr = &mrd->rlist[i];
+			    GeoClip(lr, arg->dCD_clip);
 			    if (!GEO_RECTNULL(lr))  
-			    {
+			     {
 				(*(arg->dCD_function)) (arg->dCD_celldef,
 					lr, cptr, arg->dCD_clientData);
 				(*(arg->dCD_errors))++;
 			    }           
 			}
-
-			continue;
 		    }
-		    else if (cptr->drcc_flags & (DRC_AREA | DRC_RECTSIZE
+		    continue;
+		}
+		else if (cptr->drcc_flags & (DRC_AREA | DRC_RECTSIZE
 				| DRC_MAXWIDTH))
+		{
+		    /* only have to do these checks in one direction */
+		    if (trigpending) cptr = cptr->drcc_next;
+		    continue;
+		}
+
+		result = 0;
+		do {
+		    if (triggered)
 		    {
-			/* only have to do these checks in one direction */
-			if (trigpending) cptr = cptr->drcc_next;
-			continue;
+			errRect = mrd->rlist[--triggered];
+			errRect.r_xtop += cdist;
+			errRect.r_xbot -= cdist;
+			if (errRect.r_xtop > edgeRight) errRect.r_xtop = edgeRight;
+			if (errRect.r_xbot < edgeLeft) errRect.r_xbot = edgeLeft;
+
+			/* Temporary. . . we need to loop through all rects */
+			triggered = 0;
+		    }
+		    else
+		    {
+			errRect.r_xbot = edgeLeft;
+			errRect.r_xtop = edgeRight;
 		    }
 
 		    if (cptr->drcc_flags & DRC_REVERSE)
@@ -836,7 +950,7 @@ checkbottom:
 			{
 			    errRect.r_xtop += cdist;
 			    if (DRCEuclidean)
-				arg->dCD_radial |= 0x4000;
+				arg->dCD_radial |= RADIAL_SE;
 			}
 
 			if (cptr->drcc_flags & DRC_BOTHCORNERS)
@@ -853,7 +967,7 @@ checkbottom:
 			    {
 				errRect.r_xbot -= cdist;
 				if (DRCEuclidean)
-				    arg->dCD_radial |= 0x1000;
+				    arg->dCD_radial |= RADIAL_SW;
 			    }
 			}
 
@@ -868,11 +982,8 @@ checkbottom:
 				&& arg->dCD_plane == cptr->drcc_plane
 				&& TTMaskHasType(&cptr->drcc_mask,
 				TiGetTopType(tpbot)))
-			{
-			    if (trigpending)
-				cptr = cptr->drcc_next;
 			    continue;
-			}
+
 			errRect.r_ytop = edgeY;
 			arg->dCD_initial = tile;
 		    }
@@ -891,7 +1002,7 @@ checkbottom:
 			{
 			    errRect.r_xbot -= cdist;
 			    if (DRCEuclidean)
-				arg->dCD_radial |= 0x2000;
+				arg->dCD_radial |= RADIAL_NW;
 			}
 
 			if (cptr->drcc_flags & DRC_BOTHCORNERS)
@@ -907,7 +1018,7 @@ checkbottom:
 			    {
 				errRect.r_xtop += cdist;
 				if (DRCEuclidean)
-				    arg->dCD_radial |= 0x8000;
+				    arg->dCD_radial |= RADIAL_NE;
 			    }
 			}
 
@@ -922,11 +1033,8 @@ checkbottom:
 				&& arg->dCD_plane == cptr->drcc_plane
 				&& TTMaskHasType(&cptr->drcc_mask,
 				TiGetBottomType(tile)))
-			{
-			    if (trigpending)
-				cptr = cptr->drcc_next;
 			    continue;
-			}
+
 			errRect.r_ybot = edgeY;
 			arg->dCD_initial = tpbot;
 		    }
@@ -939,21 +1047,21 @@ checkbottom:
 		    DRCstatSlow++;
 		    arg->dCD_cptr = cptr;
 		    TTMaskCom2(&tmpMask, &cptr->drcc_mask);
-		    result = DBSrPaintArea((Tile *) NULL,
+		    result += DBSrPaintArea((Tile *) NULL,
 				arg->dCD_celldef->cd_planes[cptr->drcc_plane],
 				&errRect, &tmpMask, areaCheck, (ClientData) arg);
+		} while (triggered);
 
-		    if (result == 0)
-		    {
-		        /* Trigger rule:  If rule check found errors,	*/
-		        /* do the next rule.  Otherwise, skip it.	*/
+		if (result == 0)
+		{
+		    /* Trigger rule:  If rule check found errors,	*/
+		    /* do the next rule.  Otherwise, skip it.	*/
 
-			if (trigpending)
-			    cptr = cptr->drcc_next;
-		    }
+		    if (trigpending)
+			cptr = cptr->drcc_next;
 		}
-		DRCstatEdges++;
-
+	    }
+	    DRCstatEdges++;
         }
     }
     return (0);
