@@ -20,7 +20,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCcontin.c,v 1.2 2008/06/01 18:37:40 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCcontin.c,v 1.6 2009/09/11 18:03:47 tim Exp $";
 #endif	/* not lint */
 
 #include <stdio.h>
@@ -59,7 +59,7 @@ global unsigned char DRCStatus = DRC_NOT_RUNNING;
  * tiles pile up but nothing is checked.
  */
 
-global bool DRCBackGround = TRUE;
+global u_char DRCBackGround = DRC_NOT_SET;
 
 /* Global variable, settable by outside world, that enables
  * and disables Euclidean DRC checking.  If enabled, magic
@@ -447,7 +447,7 @@ DRCBreak()
  */
 
 void
-DRCContinuous ()
+DRCContinuous()
 {
 #ifndef MAGIC_WRAPPER
     Rect drc_orig_bbox;			/* Area of DRC def that changed. */
@@ -486,7 +486,9 @@ DRCContinuous ()
 			     /* check for new user command (without blocking) */
 
 #ifdef MAGIC_WRAPPER
-	    /* Execute pending Tcl events, so the DRC process doesn't block. */
+	    /* Execute pending Tcl events, so the DRC process doesn't block.	*/
+	    /* NOTE:  Exclude file events, or else "drc catchup" will not work	*/
+	    /* in batch mode.							*/
 	    UndoEnable();
 	    while (Tcl_DoOneEvent(TCL_DONT_WAIT))
 	    {
