@@ -19,7 +19,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCmain.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCmain.c,v 1.2 2009/09/11 15:47:09 tim Exp $";
 #endif	/* not lint */
 
 #include <sys/types.h>
@@ -616,7 +616,7 @@ DRCCatchUp()
     int background;
 
     background = DRCBackGround;
-    DRCBackGround = TRUE;
+    DRCBackGround = DRC_SET_ON;
     DRCContinuous();
     DRCBackGround = background;
 }
