@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCtech.c,v 1.5 2009/01/13 22:02:09 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCtech.c,v 1.6 2009/04/10 12:31:12 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -3234,7 +3234,7 @@ drcTechFinalStyle(style)
     {
 	for (j = 0; j < DBNumTypes; j++)
 	{
-	    DRCCookie *keep = NULL;
+	    DRCCookie *keep = NULL, *dptest, *dptemp, *dpnew;
 	    
 	    for (dp = style->DRCRulesTbl[i][j]; dp != NULL; dp = dp->drcc_next)
 	    {
@@ -3268,26 +3268,40 @@ drcTechFinalStyle(style)
 		// if (DBIsContact(i) || DBIsContact(j) || i == TT_SPACE ||
 		//	j == TT_SPACE) continue;
 
+		dpnew = NULL;
 		keep = dp;
-		goto illegalEdge;
-	    }
-	    continue;
 
-	    /* This edge is illegal.  Throw away all rules except the one
-	     * needed that is always violated.
-	     */
+		/* This edge is illegal.  Throw away all rules except the one
+		 * needed that is always violated.
+		 */
 	    
-	    illegalEdge:
-	    for (dp = style->DRCRulesTbl[i][j]; dp != NULL; dp = dp->drcc_next)
-	    {
-		if (dp == keep) continue;
-		/* Don't free the shared drcc_why string here! */
-		freeMagic((char *) dp);
-		drcRulesOptimized += 1;
+		dptest = style->DRCRulesTbl[i][j];
+		while (dptest != NULL)
+		{
+		    dptemp = dptest->drcc_next;
+		    if ((dptest == keep) || (dptest->drcc_edgeplane !=
+				keep->drcc_edgeplane))
+		    {
+			dptest->drcc_next = NULL;
+			if (dpnew == NULL)
+			    style->DRCRulesTbl[i][j] = dptest;
+			else
+			    dpnew->drcc_next = dptest;
+			dpnew = dptest;
+
+			/* "keep" can't be a trigger rule! */
+			if (dptest == keep)
+			    keep->drcc_flags &= ~DRC_TRIGGER;
+		    }
+		    else
+		    {
+			/* Don't free the shared drcc_why string here! */
+			freeMagic((char *)dptest);
+			drcRulesOptimized++;
+		    }
+		    dptest = dptemp;
+		}
 	    }
-	    style->DRCRulesTbl[i][j] = keep;
-	    keep->drcc_flags &= ~DRC_TRIGGER;	/* Can't be a trigger rule! */
-	    keep->drcc_next = NULL;
 
 	    /* TxPrintf("Edge %s-%s is illegal.\n", DBTypeShortName(i),
 		DBTypeShortName(j));
