@@ -13,7 +13,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/ext2sim/ext2sim.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/ext2sim/ext2sim.c,v 1.2 2008/12/03 14:12:09 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1082,10 +1082,10 @@ simdevVisit(dev, hierName, trans)
 		r.r_xbot, r.r_ybot);
     }
     else if (dev->dev_class == DEV_RES) {	/* generate a resistor */
-       fprintf(esSimF, " %f", (double)(dev->dev_values.dev_res));
+       fprintf(esSimF, " %f", (double)(dev->dev_res));
     }
     else if (dev->dev_class == DEV_CAP) {	/* generate a capacitor */
-       fprintf(esSimF, " %f", (double)(dev->dev_values.dev_cap));
+       fprintf(esSimF, " %f", (double)(dev->dev_cap));
     }
     else if (dev->dev_class != DEV_DIODE) {
 
