@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFvisit.c,v 1.2 2008/12/03 14:12:09 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFvisit.c,v 1.4 2009/07/01 13:39:16 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -178,11 +178,13 @@ EFGetLengthAndWidth(dev, lptr, wptr)
     switch (dev->dev_class)
     {
 	case DEV_MOSFET:
+	case DEV_ASYMMETRIC:
 	case DEV_BJT:
 	case DEV_SUBCKT:
 	case DEV_RSUBCKT:
 	case DEV_DIODE:
 	case DEV_CAP:
+	case DEV_RES:
 	    l = dev->dev_length;
 	    w = dev->dev_width;
 	    break;
