@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtArray.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtArray.c,v 1.2 2009/05/30 03:14:00 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -654,11 +654,11 @@ extArrayNodeName(np, ha, et1, et2)
 {
     Tile *tp;
 
-    NODETOTILE(np, et1, tp);	/* tp = extHierLookTile(np, et1); */
+    tp = extNodeToTile(np, et1);
     if (tp && TiGetType(tp) != TT_SPACE && extHasRegion(tp, extUnInit))
 	return (extArrayTileToNode(tp, np->nreg_pnum, et1, ha, TRUE));
 
-    NODETOTILE(np, et2, tp);	/* tp = extHierLookTile(np, et2); */
+    tp = extNodeToTile(np, et2);
     if (tp && TiGetType(tp) != TT_SPACE && extHasRegion(tp, extUnInit))
 	return (extArrayTileToNode(tp, np->nreg_pnum, et2, ha, TRUE));
 
