@@ -53,8 +53,8 @@ static char sccsid[] = "@(#)ExtBasic.c	4.13 MAGIC (Berkeley) 12/5/85";
 /* ext2spice/ext2sim are compiled as separate programs (i.e., non-Tcl)	*/
 
 #ifdef MAGIC_WRAPPER
-char *extDevTable[] = {"fet", "mosfet", "bjt", "devres", "devcap",
-	"diode", "subckt", "rsubckt", NULL};
+char *extDevTable[] = {"fet", "mosfet", "asymmetric", "bjt", "devres",
+	"devcap", "diode", "subckt", "rsubckt", NULL};
 #endif
 
 /* --------------------- Data local to this file ---------------------- */
@@ -625,7 +625,7 @@ ExtSortTerminals(tran, ll)
 	    tran->tr_termpos[nsd+1] = tmp_pos;
 	    tran->tr_termlen[nsd+1] = tmp_len;
 	   /* Need to SWAP the indices in the labRegion too. 
-            * These for loops within the  bubllesort in here are kinda slow 
+            * These for loops within the  bubblesort in here are kinda slow 
             *  but S,D attributes are not that common so it should not matter 
             * that much -- Stefanos 5/96 */
             for ( lp = ll ; lp ; lp = lp->ll_next ) 
@@ -1175,8 +1175,11 @@ extOutputTrans(def, transList, outFile)
 
 	/* Set all terminals to NULL to guard against	*/
 	/* asymmetric devices missing a terminal.	*/
+	/* 5/30/09---but, reinitialize the array out to MAXSD,	*/
+	/* or devices declaring minterms < maxterms screw up!	*/
+
 	nsd = ExtCurStyle->exts_transSDCount[t];
-	for (i = 0; i < nsd; i++) extTransRec.tr_termnode[i] = NULL;
+	for (i = 0; i < MAXSD; i++) extTransRec.tr_termnode[i] = NULL;
 
 	/* Mark with reg and process each perimeter segment */
 	arg.fra_uninit = (ClientData) extTransRec.tr_gatenode;
@@ -1282,6 +1285,7 @@ extOutputTrans(def, transList, outFile)
 	    case DEV_MOSFET:
 	    case DEV_BJT:
 	    case DEV_SUBCKT:
+	    case DEV_ASYMMETRIC:
 		length = extTransRec.tr_gatelen / 2;	/* (default) */
 		width = 0;
 		isAnnular = FALSE;
@@ -1319,7 +1323,12 @@ extOutputTrans(def, transList, outFile)
 
 		if (n == 0)
 		{
-		    if (ExtDoWarn)
+		    /* Don't issue a warning on devices such as a 	*/
+		    /* vertical diode that may declare zero terminals	*/
+		    /* because the substrate node (i.e., well) is the	*/
+		    /* other terminal.					*/
+
+		    if (ExtDoWarn && (ExtCurStyle->exts_transSDCount[t] > 0))
 			extTransBad(def, reg->treg_tile,
 				"Could not determine device boundary");
 		    length = width = 0;
@@ -1403,7 +1412,7 @@ extOutputTrans(def, transList, outFile)
 		    }
 
 		}
-		else  /* DEV_MOSFET, DEV_BJT */
+		else  /* DEV_MOSFET, DEV_ASYMMETRIC, DEV_BJT */
 		{
 		    fprintf(outFile, " %d %d", length, width);
 		}
@@ -1775,7 +1784,7 @@ extTransTileFunc(tile, pNum)
     if (IsSplit(tile))
     {
         loctype = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
-	return (0); 	/* Hack alert!  We must properly handle diagonals! */
+	// return (0); 	/* Hack alert!  We must properly handle diagonals! */
     }
     else
 #endif
@@ -1791,8 +1800,8 @@ int
 extTransPerimFunc(bp)
     Boundary *bp;
 {
-    TileType tinside = TiGetType(bp->b_inside);
-    TileType toutside = TiGetType(bp->b_outside);
+    TileType tinside, toutside;
+    Tile *tile;
     NodeRegion *diffNode = (NodeRegion *) extGetRegion(bp->b_outside);
     int i, len = BoundaryLength(bp);
     int thisterm;
@@ -1800,7 +1809,18 @@ extTransPerimFunc(bp)
     Label *lab;
     Rect r;
     bool SDterm = FALSE;
- 
+
+    tile = bp->b_inside;
+    if (IsSplit(tile))
+        tinside = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
+    else
+        tinside = TiGetTypeExact(bp->b_inside);
+    tile = bp->b_outside;
+    if (IsSplit(tile))
+        toutside = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
+    else
+        toutside = TiGetTypeExact(bp->b_outside);
+
     for (i = 0; !TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside][i], TT_SPACE);
 		i++)
     {
@@ -1912,7 +1932,8 @@ extTransPerimFunc(bp)
      */
 
     /* Don't double-count contact perimeters (added by Tim 1/9/07) */
-    if (!DBIsContact(toutside) || (bp->b_plane == extTransRec.tr_gatenode->nreg_pnum))
+    if ((!DBIsContact(toutside) && !DBIsContact(tinside)) ||
+		(bp->b_plane == extTransRec.tr_gatenode->nreg_pnum))
 	extTransRec.tr_perim += len;
 
     return (0);
@@ -2339,6 +2360,50 @@ extLabType(text, typeMask)
 }
 
 
+/*
+ * ----------------------------------------------------------------------------
+ * extNodeToTile --
+ *
+ *	Sets tp to be the tile containing the lower-leftmost point of the
+ *	NodeRegion *np, but in the tile planes of the ExtTree *et instead
+ *	of the tile planes originally containing *np.  This routine used
+ *	to be defined as the macro NODETOTILE().
+ *
+ * Results:
+ *	Returns a pointer to the tile
+ *
+ * Side effects:
+ *	None.
+ * ----------------------------------------------------------------------------
+ */
+
+Tile *extNodeToTile(np, et)
+    NodeRegion *np;
+    ExtTree *et;
+{
+    Tile *tp;
+    Plane *myplane;
+
+    myplane = et->et_use->cu_def->cd_planes[np->nreg_pnum];
+
+    tp = myplane->pl_hint;
+    GOTOPOINT(tp, &np->nreg_ll);
+    myplane->pl_hint = tp;
+
+#ifdef NONMANHATTAN
+    if (IsSplit(tp))
+    {
+	TileType tpt = TiGetTypeExact(tp);
+	if ((tpt & TT_LEFTMASK) == (np->nreg_type & TT_LEFTMASK))
+	    TiSetBody(tp, tpt & ~TT_SIDE);
+	else
+	    TiSetBody(tp, tpt | TT_SIDE);
+    }
+#endif
+
+    return tp;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2369,12 +2434,23 @@ extSetNodeNum(reg, plane, tile)
     TileType type;
 
 #ifdef NONMANHATTAN
-    type = SplitSide(tile) ? SplitRightType(tile) : SplitLeftType(tile);
-#else
-    type = TiGetType(tile);
-#endif
+    if (IsSplit(tile))
+    {
+	/* Only consider split tiles if the lower-left-hand corner      */
+	/* is only the type under consideration.                        */
 
-    if (plane < reg->lreg_pnum)
+	if (!SplitSide(tile) && SplitDirection(tile))
+	    type = SplitSide(tile) ? SplitRightType(tile) : SplitLeftType(tile);
+	else if (reg->lreg_pnum == DBNumPlanes)
+	    type = TiGetTypeExact(tile);
+	else
+	    return;
+    }
+    else
+#endif
+	type = TiGetType(tile);
+
+    if ((plane < reg->lreg_pnum) || (reg->lreg_type & TT_DIAGONAL))
     {
 	reg->lreg_type = type;
 	reg->lreg_pnum = plane;
@@ -2652,6 +2728,31 @@ extNodeAreaFunc(tile, arg)
 	    extResistArea[resistClass] += area;
 	reg->nreg_cap += area * ExtCurStyle->exts_areaCap[residue];
 
+#ifdef NONMANHATTAN
+	/* Compute perimeter of nonManhattan edges */
+	if (IsSplit(tile))
+	{
+	    len = ((RIGHT(tile) - LEFT(tile)) * (RIGHT(tile) - LEFT(tile))) +
+		 	((TOP(tile) - BOTTOM(tile)) * (TOP(tile) - BOTTOM(tile)));
+	    len = (int)sqrt((double)len);
+
+	    if (extNodeClipArea)
+	    {
+		/* To-do:  Find perimeter length of clipped edge */
+	    }
+
+	    /* Find the type on the other side of the tile */
+            t = (SplitSide(tile)) ? SplitLeftType(tile):
+                        SplitRightType(tile);
+	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, tilePlaneNum) : t;
+	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
+		reg->nreg_cap += capval * len;
+	    if (TTMaskHasType(resMask, tres) && resistClass != -1)
+		extResistPerim[resistClass] += len;
+	}
+
+#endif
+
 	/*
 	 * Walk along all four sides of tile.
 	 * Sum perimeter capacitance as we go.
