@@ -19,10 +19,11 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtPerim.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtPerim.c,v 1.2 2009/06/01 17:18:10 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <math.h>
 
 #include "utils/magic.h"
 #include "utils/geometry.h"
@@ -87,6 +88,21 @@ extEnumTilePerim(tpIn, mask, pNum, func, cdata)
     b.b_plane = pNum;
     perimLength = 0;
 
+#ifdef NONMANHATTAN
+	/* Diagonal */
+    if (IsSplit(tpIn))
+    {
+	TileType otype = (SplitSide(tpIn)) ? SplitLeftType(tpIn): SplitRightType(tpIn);
+	if (TTMaskHasType(&mask, otype))
+	{
+	    int width = RIGHT(tpIn) - LEFT(tpIn);
+	    int height = TOP(tpIn) - BOTTOM(tpIn);
+	    perimLength = width * width + height * height;
+	    perimLength = (int)sqrt((double)perimLength);
+	}
+    }
+#endif
+
 	/* Top */
     b.b_segment.r_ybot = b.b_segment.r_ytop = TOP(tpIn);
     b.b_direction = BD_TOP;
