@@ -23,7 +23,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtSubtree.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtSubtree.c,v 1.2 2009/05/30 03:14:00 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -532,11 +532,11 @@ extSubtreeOutputCoupling(ha)
 
 	ck = (CoupleKey *) he->h_key.h_words;
 
-	NODETOTILE(ck->ck_1, &ha->ha_cumFlat, tp);
+	tp = extNodeToTile(ck->ck_1, &ha->ha_cumFlat);
 	name = extSubtreeTileToNode(tp, ck->ck_1->nreg_pnum, &ha->ha_cumFlat, ha, TRUE);
 	fprintf(ha->ha_outf, "cap \"%s\" ", name);
 
-	NODETOTILE(ck->ck_2, &ha->ha_cumFlat, tp);
+	tp = extNodeToTile(ck->ck_2, &ha->ha_cumFlat);
 	name = extSubtreeTileToNode(tp, ck->ck_2->nreg_pnum, &ha->ha_cumFlat, ha, TRUE);
 	fprintf(ha->ha_outf, "\"%s\" %lg\n", name, cap);
     }
