@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/graphics/grMain.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/graphics/grMain.c,v 1.2 2009/05/13 15:03:17 tim Exp $";
 #endif  /* not lint */
 
 /*
@@ -111,11 +111,11 @@ static char *grDisplayTypes[] = {
     "NULL",
     NULL};
 
-extern x11SetDisplay();
-extern oglSetDisplay();
-extern nullSetDisplay();
+extern bool x11SetDisplay();
+extern bool oglSetDisplay();
+extern bool nullSetDisplay();
 
-static (*(grInitProcs[]))() = {
+static bool (*(grInitProcs[]))() = {
 #ifdef	X11
     x11SetDisplay,  
     x11SetDisplay,  
