@@ -804,9 +804,17 @@ keys_and_buttons:
 		grCurrent.windowid = Tk_WindowId(wind);
 		grCurrent.mw = mw;
 
-		keymod = (LockMask | ControlMask | ShiftMask | Mod1Mask)
+		keymod = (LockMask | ControlMask | ShiftMask)
 				& KeyPressedEvent->state;
 
+#ifdef macosx
+		if (KeyPressedEvent->state & (Mod1Mask | Mod2Mask |
+				Mod3Mask | Mod4Mask | Mod5Mask))
+		    keymod |= Mod1Mask;
+#else
+		keymod |= (Mod1Mask & KeyPressedEvent->state);
+#endif
+
 		if (nbytes == 0)
 		{
 		    /* No ASCII equivalent */
@@ -883,9 +891,10 @@ keys_and_buttons:
 			        TxSetPoint(KeyPressedEvent->x,
 					grXToMagic(KeyPressedEvent->y),
 					grCurrent.mw->w_wid);
-				TxInputRedirect = TX_INPUT_NORMAL;
+				TxInputRedirect = TX_INPUT_PROCESSING;
 				Tcl_EvalEx(consoleinterp, "::tkcon::Eval .text",
 						19, 0);
+				TxInputRedirect = TX_INPUT_NORMAL;
 				TxSetPrompt('%');
 
 				Tcl_SaveResult(magicinterp, &state);
@@ -1213,9 +1222,7 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
     /* local indirections */
     grSetSPatternPtr = grtkSetSPattern;
     grPutTextPtr = grtkPutText;
-#ifdef VECTOR_FONTS
     grFontTextPtr = grtkFontText;
-#endif
     grDefineCursorPtr = grTkDefineCursor;
     grFreeCursorPtr = grTkFreeCursors;
     GrBitBltPtr = GrTkBitBlt;
