@@ -9,7 +9,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/lef/defRead.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/lef/defRead.c,v 1.2 2008/06/01 18:37:43 tim Exp $";            
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -554,23 +554,32 @@ DefReadNets(f, rootDef, sname, oscale, special, total)
  *	Read location and orientation of a cell use
  *	Syntax: ( X Y ) O
  *
+ * Results:
+ *	0 on success, -1 on failure
+ *
+ * Side Effects:
+ *	Transform is placed in the location pointed to by "tptr".
+ *	This routine can be used for placement of geometry (as
+ *	opposed to cell uses) by setting "use" to NULL, and using
+ *	the resulting transform to modify the geometry.
  *------------------------------------------------------------
  */
 enum def_orient {DEF_NORTH, DEF_SOUTH, DEF_EAST, DEF_WEST,
 	DEF_FLIPPED_NORTH, DEF_FLIPPED_SOUTH, DEF_FLIPPED_EAST,
 	DEF_FLIPPED_WEST};
 
-void
-DefReadLocation(use, f, oscale)
+int
+DefReadLocation(use, f, oscale, tptr)
     CellUse *use;
     FILE *f;
     float oscale;
+    Transform *tptr;
 {
     Rect *r, tr;
     int keyword;
     char *token;
     float x, y;
-    Transform t, t2;
+    Transform t2;
 
     static char *orientations[] = {
 	"N", "S", "E", "W", "FN", "FS", "FE", "FW"
@@ -599,45 +608,215 @@ DefReadLocation(use, f, oscale)
     /* unchanged after the transformation.  Case conditions	*/
     /* restore the lower-left corner position.			*/
 
-    r = &use->cu_def->cd_bbox;
+    if (use)
+	r = &use->cu_def->cd_bbox;
+    else
+	r = &GeoNullRect;
+
     switch (keyword)
     {
 	case DEF_NORTH:
-	    t = GeoIdentityTransform;
+	    *tptr = GeoIdentityTransform;
 	    break;
 	case DEF_SOUTH:
-	    t = Geo180Transform;
+	    *tptr = Geo180Transform;
 	    break;
 	case DEF_EAST:
-	    t = Geo90Transform;
+	    *tptr = Geo90Transform;
 	    break;
 	case DEF_WEST:
-	    t = Geo270Transform;
+	    *tptr = Geo270Transform;
 	    break;
 	case DEF_FLIPPED_NORTH:
-	    t = GeoSidewaysTransform;
+	    *tptr = GeoSidewaysTransform;
 	    break;
 	case DEF_FLIPPED_SOUTH:
-	    t = GeoUpsideDownTransform;
+	    *tptr = GeoUpsideDownTransform;
 	    break;
 	case DEF_FLIPPED_EAST:
-	    t = GeoRef135Transform;
+	    *tptr = GeoRef135Transform;
 	    break;
 	case DEF_FLIPPED_WEST:
-	    t = GeoRef45Transform;
+	    *tptr = GeoRef45Transform;
 	    break;
     }
-    GeoTransRect(&t, r, &tr);
-    GeoTranslateTrans(&t, -tr.r_xbot, -tr.r_ybot, &t2);
-    GeoTranslateTrans(&t2, (int)roundf(x / oscale), (int)roundf(y / oscale), &t);
-    DBSetTrans(use, &t);
-    return;
+    GeoTransRect(tptr, r, &tr);
+    GeoTranslateTrans(tptr, -tr.r_xbot, -tr.r_ybot, &t2);
+    GeoTranslateTrans(&t2, (int)roundf(x / oscale), (int)roundf(y / oscale), tptr);
+    if (use)
+	DBSetTrans(use, tptr);
+    return 0;
 
 parse_error:
     LefError("Cannot parse location: must be ( X Y ) orient\n");
-    return;
+    return -1;
 }
 
+/*
+ *------------------------------------------------------------
+ *
+ * DefReadPins --
+ *
+ *	Read a PINS section from a DEF file.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Generates paint and labels in the layout.
+ *
+ *------------------------------------------------------------
+ */
+
+enum def_pins_keys {DEF_PINS_START = 0, DEF_PINS_END};
+enum def_pins_prop_keys {
+	DEF_PINS_PROP_NET = 0, DEF_PINS_PROP_DIR,
+	DEF_PINS_PROP_LAYER, DEF_PINS_PROP_PLACED};
+
+void
+DefReadPins(f, rootDef, sname, oscale, total)
+    FILE *f;
+    CellDef *rootDef;
+    char *sname;
+    float oscale;
+    int total;
+{
+    char *token;
+    char pinname[LEF_LINE_MAX];
+    int keyword, subkey, values;
+    int processed = 0;
+    int pinDir = PORT_CLASS_DEFAULT;
+    TileType curlayer;
+    Rect *currect, topRect;
+    Transform t;
+    lefLayer *lefl;
+
+    static char *pin_keys[] = {
+	"-",
+	"END",
+	NULL
+    };
+
+    static char *pin_property_keys[] = {
+	"NET",
+	"DIRECTION",
+	"LAYER",
+	"PLACED",
+	NULL
+    };
+
+    static char *pin_classes[] = {
+	"DEFAULT",
+	"INPUT",
+	"OUTPUT TRISTATE",
+	"OUTPUT",
+	"INOUT",
+	"FEEDTHRU",
+	NULL
+    };
+
+    static int lef_class_to_bitmask[] = {
+	PORT_CLASS_DEFAULT,
+	PORT_CLASS_INPUT,
+	PORT_CLASS_TRISTATE,
+	PORT_CLASS_OUTPUT,
+	PORT_CLASS_BIDIRECTIONAL,
+	PORT_CLASS_FEEDTHROUGH
+    };
+
+    while ((token = LefNextToken(f, TRUE)) != NULL)
+    {
+	keyword = Lookup(token, pin_keys);
+
+	if (keyword < 0)
+	{
+	    LefError("Unknown keyword \"%s\" in PINS "
+			"definition; ignoring.\n", token);
+	    LefEndStatement(f);
+	    continue;
+	}
+	switch (keyword)
+	{
+	    case DEF_PINS_START:		/* "-" keyword */
+
+		/* Update the record of the number of pins		*/
+		/* processed and spit out a message for every 5% done.	*/
+ 
+		LefEstimate(processed++, total, "pins");
+
+		/* Get pin name */
+		token = LefNextToken(f, TRUE);
+		if (sscanf(token, "%2047s", pinname) != 1)
+		{
+		    LefError("Bad pin statement:  Need pin name\n");
+		    LefEndStatement(f);
+		    break;
+		}
+
+		/* Now do a search through the line for "+" entries	*/
+		/* And process each.					*/
+
+		while ((token = LefNextToken(f, TRUE)) != NULL)
+		{
+		    if (*token == ';') break;
+		    if (*token != '+') continue;
+
+		    token = LefNextToken(f, TRUE);
+		    subkey = Lookup(token, pin_property_keys);
+		    if (subkey < 0)
+		    {
+			LefError("Unknown pin property \"%s\" in "
+				"PINS definition; ignoring.\n", token);
+			continue;
+		    }
+		    switch (subkey)
+		    {
+			case DEF_PINS_PROP_NET:
+			    /* Get the net name, but ignore it */
+			    token = LefNextToken(f, TRUE);
+			    break;
+			case DEF_PINS_PROP_DIR:
+			    token = LefNextToken(f, TRUE);
+			    subkey = Lookup(token, pin_classes);
+			    if (subkey < 0)
+				LefError("Unknown pin class\n");
+			    else
+				pinDir = lef_class_to_bitmask[subkey];
+			    break;
+			case DEF_PINS_PROP_LAYER:
+			    curlayer = LefReadLayer(f, FALSE);
+			    currect = LefReadRect(f, curlayer, oscale);
+			    break;
+			case DEF_PINS_PROP_PLACED:
+			    DefReadLocation(NULL, f, oscale, &t);
+			    GeoTransRect(&t, currect, &topRect);
+			    DBPaint(rootDef, &topRect, curlayer);
+			    DBPutLabel(rootDef, &topRect, -1, pinname, curlayer,
+					pinDir);
+			    break;
+		    }
+		}
+
+		break;
+
+	    case DEF_PINS_END:
+		if (!LefParseEndStatement(f, sname))
+		{
+		    LefError("Pins END statement missing.\n");
+		    keyword = -1;
+		}
+		break;
+	}
+	if (keyword == DEF_PINS_END) break;
+    }
+
+    if (processed == total)
+	TxPrintf("  Processed %d pins total.\n", processed);
+    else
+	LefError("Warning:  Number of pins read (%d) does not match "
+		"the number declared (%d).\n", processed, total);
+}
+ 
 /*
  *------------------------------------------------------------
  *
@@ -814,6 +993,7 @@ DefReadComponents(f, rootDef, sname, oscale, total)
 {
     CellDef *defMacro;
     CellUse *defUse;
+    Transform t;
     char *token;
     char usename[512];
     int keyword, subkey, values;
@@ -925,7 +1105,7 @@ DefReadComponents(f, rootDef, sname, oscale, total)
 			case DEF_PROP_UNPLACED:
 			case DEF_PROP_FIXED:
 			case DEF_PROP_COVER:
-			    DefReadLocation(defUse, f, oscale);
+			    DefReadLocation(defUse, f, oscale, &t);
 			    break;
 			case DEF_PROP_SOURCE:
 			case DEF_PROP_WEIGHT:
@@ -941,7 +1121,10 @@ DefReadComponents(f, rootDef, sname, oscale, total)
 
 		/* Place the cell */
 		if (defUse != NULL)
+		{
 		    DBPlaceCell(defUse, rootDef);
+		    defUse = NULL;
+		}
 		break;
 
 	    case DEF_COMP_END:
@@ -953,8 +1136,10 @@ DefReadComponents(f, rootDef, sname, oscale, total)
 
 		/* Finish final call by placing the cell use */
 		if ((total > 0) && (defUse != NULL))
+		{
 		    DBPlaceCell(defUse, rootDef);
-
+		    defUse = NULL;
+		}
 		break;
 	}
 	if (keyword == DEF_COMP_END) break;
@@ -1165,7 +1350,7 @@ DefRead(inName)
 		token = LefNextToken(f, TRUE);
 		if (sscanf(token, "%d", &total) != 1) total = 0;
 		LefEndStatement(f);
-		LefSkipSection(f, sections[DEF_PINS]);
+		DefReadPins(f, rootDef, sections[DEF_PINS], oscale, total);
 		break;
 	    case DEF_PINPROPERTIES:
 		LefSkipSection(f, sections[DEF_PINPROPERTIES]);
