@@ -123,7 +123,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/mzrouter/mzSearch.c,v 1.10 2006/12/02 00:36:58 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/mzrouter/mzSearch.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -148,7 +148,8 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 
 /* Forward declarations */
 
-extern void mzExtendViaContacts();
+extern void mzExtendViaLRContacts();
+extern void mzExtendViaUDContacts();
 extern void mzBloomInit();
 extern void mzMakeStatReport();
 extern void mzExtendPath(RoutePath *);
@@ -751,9 +752,13 @@ mzExtendPath(path)
 	mzExtendDown(path);
     }
 
-    if (extendCode & EC_CONTACTS)
+    if (extendCode & EC_UDCONTACTS)
     {
-	mzExtendViaContacts(path);
+	mzExtendViaUDContacts(path);
+    }
+    if (extendCode & EC_LRCONTACTS)
+    {
+	mzExtendViaLRContacts(path);
     }
 
     if (extendCode >= EC_WALKRIGHT)
@@ -776,9 +781,13 @@ mzExtendPath(path)
 	{
 	    mzWalkDown(path);
 	}
-	else if (extendCode & EC_WALKCONTACT)
+	else if (extendCode & EC_WALKUDCONTACT)
 	{
-	    mzWalkContact(path);
+	    mzWalkUDContact(path);
+	}
+	else if (extendCode & EC_WALKLRCONTACT)
+	{
+	    mzWalkLRContact(path);
 	}
     }
 
@@ -789,10 +798,11 @@ mzExtendPath(path)
 /*
  * ----------------------------------------------------------------------------
  *
- * mzExtendViaContacts --
+ * mzExtendViaLRContacts --
  *
  * Spread from a point to other planes reachable from it,
- * by using contacts.  Stacked contacts are allowed.
+ * by using contacts.  Stacked contacts are allowed.  Search the horizontal
+ * block plane.  Limit to areas that fit the contact minimum length.
  *
  * Results	None.
  *
@@ -802,7 +812,7 @@ mzExtendPath(path)
  * ----------------------------------------------------------------------------
  */
 void
-mzExtendViaContacts(path)
+mzExtendViaLRContacts(path)
     RoutePath *path;
 {
     Point p = path->rp_entry, *lastCpos = NULL;
@@ -817,7 +827,7 @@ mzExtendViaContacts(path)
 
     /* DEBUG - trace calls to this routine */
     if (DebugIsSet(mzDebugID, mzDebMaze))
-	TxPrintf("EXTENDING WITH CONTACTS\n");
+	TxPrintf("EXTENDING WITH CONTACTS (HORIZONTAL)\n");
 
     /* Check what the last contact was and remember its	*/
     /* position.  Do not allow two contacts of the same	*/
@@ -874,7 +884,7 @@ mzExtendViaContacts(path)
 	else
 	{
 	    ASSERT(rC->rc_rLayer2 == rLayer,
-	    "mzExtendViaContacts");
+	    "mzExtendViaLRContacts");
 	    newRLayer = rC->rc_rLayer1;
 	}
 
@@ -887,9 +897,15 @@ mzExtendViaContacts(path)
 	type = TiGetType(tp);
 
 	/* 1. If blocked, don't place a contact */
-	if ((type != TT_SPACE) && (type != TT_SAMENODE)) continue;
+	if ((type != TT_SPACE) && (type != TT_SAMENODE))
+	    continue;
 
-	/* 2. Check distance from last contact, if the same type */
+	/* 2. If tile space is not long enough for the contact, don't place */
+	if (RIGHT(tp) - p.p_x <= rC->rc_routeType.rt_length
+			- rC->rc_routeType.rt_width)
+	    continue;
+
+	/* 3. Check distance from last contact, if the same type */
 	if (rC->rc_routeType.rt_tileType == lastCtype)
 	{
 	    int cdist = rC->rc_routeType.rt_spacing[lastCtype] +
@@ -899,7 +915,7 @@ mzExtendViaContacts(path)
 		continue;
 	}
 
-	/* 3. Check distance to last route bend */
+	/* 4. Check distance to last route bend */
 	if (bendDist != 0)
 	{
 	    int cwidth = rC->rc_routeType.rt_width;
@@ -945,10 +961,16 @@ mzExtendViaContacts(path)
 		    extendCode = EC_WALKUP;
 		    break;
 
-		case TT_ABOVE_WALK:
-		case TT_BELOW_WALK:
+		case TT_ABOVE_LR_WALK:
+		case TT_BELOW_LR_WALK:
 		    /* TO DO:  Check if stacked contacts are allowed! */
-		    extendCode = EC_WALKCONTACT;
+		    extendCode = EC_WALKLRCONTACT;
+		    break;
+
+		case TT_ABOVE_UD_WALK:
+		case TT_BELOW_UD_WALK:
+		    /* TO DO:  Check if stacked contacts are allowed! */
+		    extendCode = EC_WALKUDCONTACT;
 		    break;
 
 		case TT_DEST_AREA:
@@ -958,7 +980,7 @@ mzExtendViaContacts(path)
 
 		default:
 		    /* this shouldn't happen */
-		    ASSERT(FALSE,"mzExtendViaContacts");
+		    ASSERT(FALSE,"mzExtendViaLRContacts");
 		    continue;
 	    }
 
@@ -970,6 +992,202 @@ mzExtendViaContacts(path)
     return;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * mzExtendViaUDContacts --
+ *
+ * Spread from a point to other planes reachable from it,
+ * by using contacts.  Stacked contacts are allowed.  Search the vertical
+ * block plane.  Limit to areas that fit the contact minimum length.
+ *
+ * Results	None.
+ *
+ * Side effects:
+ *	Adds RoutePaths to the heap (mzReservePathCostHeap).
+ *
+ * ----------------------------------------------------------------------------
+ */
+void
+mzExtendViaUDContacts(path)
+    RoutePath *path;
+{
+    Point p = path->rp_entry, *lastCpos = NULL;
+    RouteLayer *rLayer = path->rp_rLayer; 
+    RouteContact *rC;
+    List *cL;
+    RouteLayer *newRLayer;
+    Tile *tp;
+    TileType type, lastCtype = TT_SPACE;
+    RoutePath *spath;
+    int bendDist = 0;
+
+    /* DEBUG - trace calls to this routine */
+    if (DebugIsSet(mzDebugID, mzDebMaze))
+	TxPrintf("EXTENDING WITH CONTACTS (VERTICAL)\n");
+
+    /* Check what the last contact was and remember its	*/
+    /* position.  Do not allow two contacts of the same	*/
+    /* type within a DRC error distance of each other.	*/
+
+    for (spath = path; spath && spath->rp_back && (spath->rp_orient != 'X');
+		spath = spath->rp_back);
+    if (spath->rp_back)
+    {
+	lastCpos = &spath->rp_entry;
+	rC = MZGetContact(spath, spath->rp_back);
+	lastCtype = rC->rc_routeType.rt_tileType;
+    }
+
+    /* Check where the last route bend is.  Don't allow a	*/
+    /* contact within a DRC error distance of a	bend.		*/
+
+    if (path)
+    {
+	if (path->rp_orient == 'V')
+	{
+	    for (spath = path->rp_back; spath && spath->rp_orient == 'V';
+			spath = spath->rp_back);
+	    if (spath && spath->rp_orient == 'H')
+		bendDist = spath->rp_entry.p_y - p.p_y;
+	    if (bendDist < 0)
+		bendDist += rLayer->rl_routeType.rt_width;
+	}
+	else if (path->rp_orient == 'H')
+	{
+	    for (spath = path->rp_back; spath && spath->rp_orient == 'H';
+			spath = spath->rp_back);
+	    if (spath && spath->rp_orient == 'V')
+		bendDist = spath->rp_entry.p_x - p.p_x;
+	    if (bendDist < 0)
+		bendDist += rLayer->rl_routeType.rt_width;
+	}
+    }
+
+    /* Loop through contacts that connect to current rLayer */
+    for (cL=rLayer->rl_contactL; cL!=NULL; cL=LIST_TAIL(cL))
+    {
+	rC = (RouteContact *) LIST_FIRST(cL);
+
+	/* Don't use inactive contacts */
+	if (!(rC->rc_routeType.rt_active))
+	    continue;
+
+	/* Get "other" route Layer contact connects to */
+	if (rC->rc_rLayer1 == rLayer)
+	{
+	    newRLayer = rC->rc_rLayer2;
+	}
+	else
+	{
+	    ASSERT(rC->rc_rLayer2 == rLayer,
+	    "mzExtendViaUDContacts");
+	    newRLayer = rC->rc_rLayer1;
+	}
+
+	/* Don't spread to inactive layers */
+	if (!(newRLayer->rl_routeType.rt_active)) continue;
+
+	/* Find tile on contact plane that contains point. */
+
+	tp = TiSrPointNoHint(rC->rc_routeType.rt_vBlock, &p);
+	type = TiGetType(tp);
+
+	/* 1. If blocked, don't place a contact */
+	if ((type != TT_SPACE) && (type != TT_SAMENODE))
+	    continue;
+
+	/* 2. If tile space is not long enough for the contact, don't place */
+	if (TOP(tp) - p.p_y <= rC->rc_routeType.rt_length
+			- rC->rc_routeType.rt_width)
+	    continue;
+
+	/* 3. Check distance from last contact, if the same type */
+	if (rC->rc_routeType.rt_tileType == lastCtype)
+	{
+	    int cdist = rC->rc_routeType.rt_spacing[lastCtype] +
+			rC->rc_routeType.rt_width;
+	    if ((abs(p.p_x - lastCpos->p_x) < cdist) &&
+			(abs(p.p_y - lastCpos->p_y) < cdist))
+		continue;
+	}
+
+	/* 4. Check distance to last route bend */
+	if (bendDist != 0)
+	{
+	    int cwidth = rC->rc_routeType.rt_width;
+	    int spacing = rC->rc_routeType.rt_spacing[rLayer->rl_routeType.rt_tileType];
+	    if (bendDist > cwidth && bendDist < cwidth + spacing)
+		continue;
+	    if (bendDist < 0 && bendDist > -spacing)
+		continue;
+	}
+
+	/* OK to drop a contact here */
+	{
+	    dlong conCost;
+	    int extendCode;
+
+	    /* set contact cost */
+	    conCost = (dlong) rC->rc_cost;
+
+	    /* determine appropriate extendcode */
+	    tp = TiSrPointNoHint(newRLayer->rl_routeType.rt_vBlock, &p);
+	    type = TiGetType(tp);
+
+	    switch (type)
+	    {
+		case TT_SPACE:
+		case TT_SAMENODE:
+		   extendCode = EC_RIGHT | EC_LEFT | EC_UP | EC_DOWN;
+		   break;
+
+		case TT_LEFT_WALK:
+		    extendCode = EC_WALKRIGHT;
+		    break;
+
+		case TT_RIGHT_WALK:
+		    extendCode = EC_WALKLEFT;
+		    break;
+
+		case TT_TOP_WALK:
+		    extendCode = EC_WALKDOWN;
+		    break;
+
+		case TT_BOTTOM_WALK:
+		    extendCode = EC_WALKUP;
+		    break;
+
+		case TT_ABOVE_LR_WALK:
+		case TT_BELOW_LR_WALK:
+		    /* TO DO:  Check if stacked contacts are allowed! */
+		    extendCode = EC_WALKLRCONTACT;
+		    break;
+
+		case TT_ABOVE_UD_WALK:
+		case TT_BELOW_UD_WALK:
+		    /* TO DO:  Check if stacked contacts are allowed! */
+		    extendCode = EC_WALKUDCONTACT;
+		    break;
+
+		case TT_DEST_AREA:
+
+		    extendCode = EC_COMPLETE;
+		    break;
+
+		default:
+		    /* this shouldn't happen */
+		    ASSERT(FALSE,"mzExtendViaUDContacts");
+		    continue;
+	    }
+
+	    /* Finally add point after contact */
+	    mzAddPoint(path, &p, newRLayer, 'X', extendCode, &conCost);
+	}
+    }
+
+    return;
+}
 
 /*
  * ----------------------------------------------------------------------------
@@ -994,7 +1212,8 @@ mzAddPoint(path, p, rLayer, orient, extendCode, costptr)
     Point *p;		/* new point */
     RouteLayer *rLayer;	/* Route Layer of new point */
     int orient;		/* 'H' = endpt of hor seg, 'V' = endpt of vert seg, 
-			 * 'O' = contact, 'B' = first point in path and blocked
+			 * 'O' = LR contact, 'X' = UD contact,
+			 * 'B' = first point in path and blocked
 			 */
     int extendCode;	/* interesting directions to extend in */
     dlong *costptr;	/* Incremental cost of new path segment */
@@ -1036,7 +1255,7 @@ mzAddPoint(path, p, rLayer, orient, extendCode, costptr)
 
 	if (path != NULL && 
 	   path->rp_rLayer == rLayer &&
-	   path->rp_orient != 'O' &&
+	   path->rp_orient != 'O' && path->rp_orient != 'X' &&
 	   path->rp_orient != orient)
 	{
 	    cost += rLayer->rl_jogCost;
