@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/plot/plotPS.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/plot/plotPS.c,v 1.2 2008/06/01 18:37:45 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1197,7 +1197,7 @@ PlotPS(fileName, scx, layers, xMask)
 	curMask = *layers;
 	TTMaskSetType(&curMask, TT_SPACE);
 	(void) DBTreeSrLabels(scx, &curMask, xMask, (TerminalPath *) NULL,
-		plotPSLabelBounds, (ClientData) NULL);
+		TF_LABEL_ATTACH, plotPSLabelBounds, (ClientData) NULL);
 	fscale = (float)(PlotPSWidth - 2 * PlotPSMargin - xnmargin - xpmargin)
 		/ (float)(xsize);
 	yscale = (float)(PlotPSHeight - 2 * PlotPSMargin - ynmargin - ypmargin)
@@ -1316,7 +1316,7 @@ PlotPS(fileName, scx, layers, xMask)
 	curMask = *layers;
 	TTMaskSetType(&curMask, TT_SPACE);
 	(void) DBTreeSrLabels(scx, &curMask, xMask, (TerminalPath *) NULL,
-		plotPSLabelBox, (ClientData) NULL);
+		TF_LABEL_ATTACH, plotPSLabelBox, (ClientData) NULL);
 	plotPSFlushRect(BORDER);
 	plotPSFlushLine();
 	PSReset();
@@ -1326,7 +1326,7 @@ PlotPS(fileName, scx, layers, xMask)
 	curMask = *layers;
 	TTMaskSetType(&curMask, TT_SPACE);
 	(void) DBTreeSrLabels(scx, &curMask, xMask, (TerminalPath *) NULL,
-		plotPSLabel, (ClientData) NULL);
+		TF_LABEL_ATTACH, plotPSLabel, (ClientData) NULL);
     }
     else
     {
