@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/plot/plotVers.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/plot/plotVers.c,v 1.2 2008/06/01 18:37:45 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1424,8 +1424,9 @@ PlotVersatec(scx, layers, xMask, user_scale)
 		curMask = *layers;
 		TTMaskSetType(&curMask, TT_SPACE);
 		GeoTransRect(&tinv, &labelArea, &scx2.scx_area);
-		(void) DBTreeSrLabels(&scx2, &curMask, xMask, (TerminalPath *) NULL,
-				      plotVersLabel, (ClientData) raster);
+		(void) DBTreeSrLabels(&scx2, &curMask, xMask,
+				(TerminalPath *) NULL, TF_LABEL_ATTACH,
+				plotVersLabel, (ClientData) raster);
 	    }
 
 	    /* Output subcell bounding boxes, if they are wanted. */
