@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResBasic.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResBasic.c,v 1.2 2008/06/01 18:37:45 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -253,7 +253,7 @@ ResEachTile(tile, startpoint)
     {
 	t2 = TiGetRightType(tp);
 	if(TTMaskHasType(&(ExtCurStyle->exts_transMask), t2) &&
-	      TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2]), t1))
+	      TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2][0]), t1))
         /* found transistor */
 	{
 	    xj = LEFT(tile);
@@ -275,7 +275,7 @@ ResEachTile(tile, startpoint)
     {
       	t2 = TiGetLeftType(tp);
 	if(TTMaskHasType(&(ExtCurStyle->exts_transMask), t2) &&
-	      TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2]), t1))
+	      TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2][0]), t1))
         /* found transistor */
 	{
 	    xj = RIGHT(tile);
@@ -297,7 +297,7 @@ ResEachTile(tile, startpoint)
     {
       	t2 = TiGetBottomType(tp);
 	if(TTMaskHasType(&(ExtCurStyle->exts_transMask),t2) &&
-	      TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2]),t1))
+	      TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2][0]),t1))
         /* found transistor */
 	{
 	    yj = TOP(tile);
@@ -318,7 +318,7 @@ ResEachTile(tile, startpoint)
     {
       	t2 = TiGetTopType(tp);
 	if(TTMaskHasType(&(ExtCurStyle->exts_transMask), t2) &&
-	      TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2]), t1))
+	      TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[t2][0]), t1))
         /* found transistor */
 	{
 	    yj = BOTTOM(tile);
