@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResConDCS.c,v 1.3 2008/06/01 18:37:45 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResConDCS.c,v 1.4 2009/01/30 03:11:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -446,6 +446,7 @@ DBTreeCopyConnectDCS(scx, mask, xMask, connect, area, destUse)
     static int 		first = 1;
     struct conSrArg2	csa2; 
     int			tran, pNum;
+    char		*tran_name;
     TileTypeBitMask	*newmask;
     ResTranTile		*CurrentT;
     CellDef		*def = destUse->cu_def;
@@ -464,16 +465,20 @@ DBTreeCopyConnectDCS(scx, mask, xMask, connect, area, destUse)
 
     if (first)
     {
-         TTMaskZero(&DiffTypeBitMask);
-	 TTMaskZero(&ResSubsTypeBitMask);
-	 for (tran = TT_TECHDEPBASE; tran < TT_MAXTYPES; tran++)
-	 {
-	      TTMaskSetMask(&DiffTypeBitMask,
+	TTMaskZero(&DiffTypeBitMask);
+	TTMaskZero(&ResSubsTypeBitMask);
+	for (tran = TT_TECHDEPBASE; tran < TT_MAXTYPES; tran++)
+	{
+	    tran_name = (ExtCurStyle->exts_transName)[tran];
+	    if ((tran_name != NULL) && (strcmp(tran_name, "None")))
+	    {
+		TTMaskSetMask(&DiffTypeBitMask,
 	      		&(ExtCurStyle->exts_transSDTypes[tran][0]));
-	      TTMaskSetMask(&ResSubsTypeBitMask,
+		TTMaskSetMask(&ResSubsTypeBitMask,
 	      		&(ExtCurStyle->exts_transSubstrateTypes[tran]));
-	 }
-	 first = 0;
+	    }
+	}
+	first = 0;
     }
 
     TransList = NULL;
