@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResMain.c,v 1.2 2008/06/01 18:37:45 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResMain.c,v 1.3 2009/01/30 03:11:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -63,26 +63,27 @@ extern HashTable	ResNodeTable;
 void
 ResInitializeConn()
 {
-     TileType tran, diff;
+    TileType tran, diff;
+    char *tran_name;
  	
-     for (tran = TT_TECHDEPBASE; tran < TT_MAXTYPES; tran++)
-     {
-     	 for (diff = TT_TECHDEPBASE; diff < TT_MAXTYPES; diff++)
-	 {
-	     if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[tran][0]),diff)
-	     {
-		  TTMaskSetType(&ResConnectWithSD[diff],tran);
-	     }
-	     if TTMaskHasType(&(ExtCurStyle->exts_transSubstrateTypes[tran]),diff)
-	     {
-		  TTMaskSetType(&ResConnectWithSD[diff],tran);
-	     }
-	          
-	 }
-	 TTMaskSetMask(&ResConnectWithSD[tran],&DBConnectTbl[tran]);
-     }
+    for (tran = TT_TECHDEPBASE; tran < TT_MAXTYPES; tran++)
+    {
+	tran_name = (ExtCurStyle->exts_transName)[tran];
+	if ((tran_name != NULL) && (strcmp(tran_name, "None")))
+	{
+	    for (diff = TT_TECHDEPBASE; diff < TT_MAXTYPES; diff++)
+	    {
+		if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[tran][0]), diff)
+		    TTMaskSetType(&ResConnectWithSD[diff],tran);
+
+		if TTMaskHasType(&(ExtCurStyle->exts_transSubstrateTypes[tran]),diff)
+		    TTMaskSetType(&ResConnectWithSD[diff],tran);
+	    }
+	}
+	TTMaskSetMask(&ResConnectWithSD[tran],&DBConnectTbl[tran]);
+    }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
