@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResRex.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResRex.c,v 1.2 2009/08/03 13:41:27 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -259,7 +259,7 @@ typedef enum {
 	case RES_SILENT:
 	    if (cmd->tx_argc > 2)
 	    {
-		Lookup(cmd->tx_argv[2], onOff);
+		value = Lookup(cmd->tx_argv[2], onOff);
 		if (value < 0)
 		{
 		    TxError("Value must be either \"on\" or \"off\".\n");
