@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/select/selCreate.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/select/selCreate.c,v 1.7 2008/12/12 01:29:10 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -112,6 +112,7 @@ SelectInit()
 	ASSERT(SelectDef != (CellDef *) NULL, "SelectInit");
 	DBCellSetAvail(SelectDef);
 	SelectDef->cd_flags |= CDINTERNAL;
+	TTMaskZero(&SelectDef->cd_types);
     }
     SelectUse = DBCellNewUse(SelectDef, (char *) NULL);
     DBSetTrans(SelectUse, &GeoIdentityTransform);
@@ -164,25 +165,27 @@ void
 SelectClear()
 {
     SearchContext scx;
-    Rect r;
+    Rect r, expand;
     extern int selClearFunc();		/* Forward declaration. */
 
     if (SelectRootDef == NULL) return;
 
     scx.scx_area = SelectDef->cd_bbox;
+    expand = scx.scx_area;
 
-    /* Special handling when the cell is named __SELECT_NET__ 	*/
+    /* Special handling when the cell is flagged as a net selection */
 
-    if (!strncmp(SelectDef->cd_name + 9, "NET", 3))
+    if (SelectUse->cu_flags & CU_SELECT_NET)
     {
 	SelNetRememberForUndo((CellDef *)NULL, (Point *)NULL,
 			TT_SPACE, FALSE, FALSE);
-	DBCellRenameDef(SelectDef, "__SELECT__");
+	SelectUse->cu_flags = 0;
 	DBCellClearDef(SelectDef);
     }
     else
     {
 	SelRememberForUndo(TRUE, (CellDef *) NULL, (Rect *) NULL);
+	SelectUse->cu_flags = 0;
 
 	/* Erase all the paint from the select cell. */
 	DBEraseMask(SelectDef, &TiPlaneRect, &DBAllButSpaceBits);
@@ -192,7 +195,7 @@ SelectClear()
 
 	/* Erase all of the labels from the select cell. */
 
-	(void) DBEraseLabel(SelectDef, &TiPlaneRect, &DBAllTypeBits);
+	(void) DBEraseLabel(SelectDef, &TiPlaneRect, &DBAllTypeBits, &expand);
 
 	/* Erase all of the subcells from the select cell.  This is a bit tricky,
 	 * because we can't erase the subcells while searching for them (it will
@@ -222,15 +225,17 @@ SelectClear()
 	SelRememberForUndo(FALSE, SelectRootDef, &scx.scx_area);
     }
 
+    TTMaskZero(&SelectDef->cd_types);
+
     /* Reset the transform, if we have been moving the selection around */
-    GeoTransRect(&SelectUse->cu_transform, &scx.scx_area, &r);
+    GeoTransRect(&SelectUse->cu_transform, &expand, &r);
     SelectUse->cu_transform = GeoIdentityTransform;
 
     /* Erase the selection from the screen. */
     DBWHLRedraw(SelectRootDef, &r, TRUE);
 
     DBReComputeBbox(SelectDef);
-    DBWAreaChanged(SelectDef, &scx.scx_area, DBW_ALLWINDOWS,
+    DBWAreaChanged(SelectDef, &expand, DBW_ALLWINDOWS,
 	(TileTypeBitMask *) NULL);
 }
 
@@ -309,6 +314,8 @@ SelectArea(scx, types, xMask)
 
     (void) DBCellCopyAllPaint(scx, types, xMask, SelectUse);
 
+    SelectDef->cd_types = *types;	/* Remember what types were requested */
+
     /* Select labels. */
 
     if (TTMaskHasType(types, L_LABEL))
@@ -333,7 +340,7 @@ SelectArea(scx, types, xMask)
     SelRememberForUndo(FALSE, SelectRootDef, &labelArea);
     DBReComputeBbox(SelectDef);
     DBWHLRedraw(SelectRootDef, &labelArea, TRUE);
-    DBWAreaChanged(SelectDef, &SelectDef->cd_bbox, DBW_ALLWINDOWS,
+    DBWAreaChanged(SelectDef, &SelectDef->cd_extended, DBW_ALLWINDOWS,
 	&DBAllButSpaceBits);
 }
 
@@ -360,9 +367,11 @@ SelectArea(scx, types, xMask)
  * ----------------------------------------------------------------------------
  */
 
+#define MAX_RECURSION_LEVEL 22
+
 void
 selFindChunk(plane, wrongTypes, searchArea, containedArea, bestMin,
-	bestMax, bestChunk)
+	bestMax, bestChunk, level)
     Plane *plane;			/* Plane on which to hunt for chunk. */
     TileTypeBitMask *wrongTypes;	/* Types that are not allowed to be
 					 * part of the chunk.
@@ -388,12 +397,17 @@ selFindChunk(plane, wrongTypes, searchArea, containedArea, bestMin,
 					 * so far, if we find one better than
 					 * bestMin and bestMax.
 					 */
+    int level;				/* Recursion level (to avoid worst-
+					 * case scenarios)
+					 */
 {
     Rect smaller;
     int min, max;
     extern int selChunkFunc();
     Rect wrong;
 
+    if (level == MAX_RECURSION_LEVEL) return;
+
     /* If the search area is already smaller than the chunk to beat,
      * there's no point in even examining this chunk.
      */
@@ -445,7 +459,7 @@ selFindChunk(plane, wrongTypes, searchArea, containedArea, bestMin,
 	smaller.r_xbot = wrong.r_xtop;
     else goto tryY;  /* Bad material overlaps containedArea in x. */
     selFindChunk(plane, wrongTypes, &smaller, containedArea,
-	    bestMin, bestMax, bestChunk);
+	    bestMin, bestMax, bestChunk, level + 1);
     
 
     /* Also try reducing the y-range to see if that works better. */
@@ -457,7 +471,7 @@ selFindChunk(plane, wrongTypes, searchArea, containedArea, bestMin,
 	smaller.r_ybot = wrong.r_ytop;
     else return;  /* Bad material overlaps containedArea in y. */
     selFindChunk(plane, wrongTypes, &smaller, containedArea,
-	    bestMin, bestMax, bestChunk);
+	    bestMin, bestMax, bestChunk, level + 1);
 }
 
 #ifdef NONMANHATTAN
@@ -596,6 +610,9 @@ SelectChunk(scx, type, xMask, pArea, less)
 
 #endif
 
+    bestMin = bestMax = 0;
+    bestChunk = GeoNullRect;
+
     GEO_EXPAND(&newscx.scx_area, INITIALSIZE, &newscx.scx_area);
     while (TRUE)
     {
@@ -610,12 +627,9 @@ SelectChunk(scx, type, xMask, pArea, less)
 
 	/* Now find the best chunk in the area. */
 
-	bestMin = bestMax = 0;
-	bestChunk = GeoNullRect;
-
 	selFindChunk(Select2Def->cd_planes[DBPlane(type)],
 	    &wrongTypes, &newscx.scx_area, &scx->scx_area,
-	    &bestMin, &bestMax, &bestChunk);
+	    &bestMin, &bestMax, &bestChunk, 0);
 	if (GEO_RECTNULL(&bestChunk))
 	{
 	    /* No chunk was found, so return. */
@@ -631,9 +645,9 @@ SelectChunk(scx, type, xMask, pArea, less)
 	
 	if (GEO_SURROUND_STRONG(&newscx.scx_area, &bestChunk)) break;
 
-	/* The chunk extends to the edge of the area.  Anyplace that the
+	/* The chunk extends to the edge of the area.  Any place that the
 	 * chunk touches an edge, move that edge out by a factor of two.
-	 * Anyplace it doesn't touch, move the edge in to be just one
+	 * Any place it doesn't touch, move the edge in to be just one
 	 * unit out from the chunk.
 	 */
 	
@@ -656,7 +670,12 @@ SelectChunk(scx, type, xMask, pArea, less)
 
 chunkdone:
 
+    /* Flag the cell as a chunk selection so we can treat	*/
+    /* the contents differently when doing a move op.		*/
+
+    SelectUse->cu_flags |= CU_SELECT_CHUNK;
     UndoEnable();
+
     if (less)
       {
 	SelRemoveArea(&bestChunk, &typeMask);
@@ -770,17 +789,14 @@ SelectRegion(scx, type, xMask, pArea, less)
 
     SelRememberForUndo(FALSE, SelectRootDef, &Select2Def->cd_bbox);
 
-    /* SelectDef should not need to recompute the bounding box;	*/
-    /* only to copy it (transformed) from Select2Def.		*/
-    /* DBReComputeBbox(SelectDef); */
-    GeoTransRect(&scx->scx_trans, &Select2Def->cd_bbox, &SelectDef->cd_bbox);
+    DBReComputeBbox(SelectDef);
     DBComputeUseBbox(SelectUse);
 
-    DBWHLRedraw(SelectRootDef, &Select2Def->cd_bbox, TRUE);
-    DBWAreaChanged(SelectDef, &Select2Def->cd_bbox, DBW_ALLWINDOWS,
+    DBWHLRedraw(SelectRootDef, &Select2Def->cd_extended, TRUE);
+    DBWAreaChanged(SelectDef, &Select2Def->cd_extended, DBW_ALLWINDOWS,
 	 &DBAllButSpaceBits);
 
-    if (pArea != NULL) *pArea = Select2Def->cd_bbox;
+    if (pArea != NULL) *pArea = Select2Def->cd_extended;
 }
 
 /*
@@ -872,27 +888,21 @@ SelectNet(scx, type, xMask, pArea, less)
 			(Rect *) NULL);
     }
 
-    /* Rename the cell to "__SELECT_NET__" so that we can treat */
-    /* it differently when it comes time to unselect it.	*/
+    /* Set the cell use flags to mark this as a net selection,	*/
+    /* so we can treat it differently when it comes time to	*/
+    /* unselect it.						*/
 
-    DBCellRenameDef(SelectDef, "__SELECT_NET__");
+    SelectUse->cu_flags |= CU_SELECT_NET;
     UndoEnable(); 
 
-    /* Display the newly-selected material. */
-    /* SelectDef should not need to recompute the bounding box;	*/
-    /* only to copy it (transformed) from Select2Def, and then	*/
-    /* transfer this bounding box to the use.			*/
-    /* ---Nishit, 7/6/04					*/
-    /* DBReComputeBbox(SelectDef); */
-
-    GeoTransRect(&scx->scx_trans, &Select2Def->cd_bbox, &SelectDef->cd_bbox);
+    DBReComputeBbox(SelectDef);
     DBComputeUseBbox(SelectUse);
 
-    DBWHLRedraw(SelectRootDef, &Select2Def->cd_bbox, TRUE);
-    DBWAreaChanged(SelectDef, &Select2Def->cd_bbox, DBW_ALLWINDOWS,
+    DBWHLRedraw(SelectRootDef, &Select2Def->cd_extended, TRUE);
+    DBWAreaChanged(SelectDef, &Select2Def->cd_extended, DBW_ALLWINDOWS,
 	&DBAllButSpaceBits);
 
-    if (pArea != NULL) *pArea = Select2Def->cd_bbox;
+    if (pArea != NULL) *pArea = Select2Def->cd_extended;
 }
 
 /*
@@ -1072,7 +1082,7 @@ SelectAndCopy2(newSourceDef)
 					 */
 {
     SearchContext scx;
-    Rect editArea, labelArea;
+    Rect editArea, labelArea, expanded;
     int plane;
     void (*savedPaintPlane)();
     extern int selACPaintFunc();	/* Forward reference. */
@@ -1121,8 +1131,8 @@ SelectAndCopy2(newSourceDef)
 		(ClientData) plane);
     }
 
-    (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_SPECIAL, SelectUse,
-	    &labelArea);
+    (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_SPECIAL,
+		SelectUse, NULL);
 
     /* We also have to be careful about copying subcells into the
      * main selection cell.  It might not have been possible to copy
@@ -1136,6 +1146,7 @@ SelectAndCopy2(newSourceDef)
 	    (ClientData) NULL);
 
     DBReComputeBbox(SelectDef);
+    DBComputeUseBbox(SelectUse);
     
     /* A little hack here:  don't do explicit redisplay of the selection,
      * or record a very large redisplay area for undo-ing.  It's not
@@ -1146,7 +1157,7 @@ SelectAndCopy2(newSourceDef)
      */
 
     SelRememberForUndo(FALSE, SelectRootDef, (Rect *) NULL);
-    DBWAreaChanged(SelectDef, &SelectDef->cd_bbox, DBW_ALLWINDOWS,
+    DBWAreaChanged(SelectDef, &SelectDef->cd_extended, DBW_ALLWINDOWS,
 	(TileTypeBitMask *) NULL);
 }
 
@@ -1215,7 +1226,7 @@ selACPaintFunc2(tile, selACarg)
     Rect *editClip = selACarg->editClip;
     Rect area, selArea;
     TileType type = TiGetTypeExact(tile);
-    TileTypeBitMask tmask;
+    TileTypeBitMask tmask, *rmask;
     TileType ttype, rtype;
 
 #ifdef NONMANHATTAN
@@ -1245,8 +1256,13 @@ selACPaintFunc2(tile, selACarg)
 	else
 	    rtype = type;
 
-	if (rtype >= DBNumUserLayers && TTMaskHasType(DBResidueMask(rtype), ttype))
-	    rtype = ttype;
+	if (rtype >= DBNumUserLayers)
+	{
+	    rmask = DBResidueMask(rtype);
+	    if (TTMaskHasType(rmask, ttype))
+		rtype = ttype;
+	}
+
 	TTMaskSetOnlyType(&tmask, rtype);
 
 	type = (dinfo & TT_SIDE) ? (rtype << 14) : rtype;
@@ -1339,15 +1355,23 @@ topbottom:
 	else
 	    rtype = type;
 
-	if (rtype >= DBNumUserLayers && TTMaskHasType(DBResidueMask(rtype), ttype))
-	    rtype = ttype;
+	if (rtype >= DBNumUserLayers)
+	{
+	    rmask = DBResidueMask(rtype);
+	    if (TTMaskHasType(rmask, ttype))
+		rtype = ttype;
+	}
 	TTMaskSetOnlyType(&tmask, rtype);
     }
 #else	/* !NONMANHATTAN */
 
     ttype = selACarg->ttype;
-    if (type >= DBNumUserLayers && TTMaskHasType(DBResidueMask(type), ttype))
+    if (type >= DBNumUserLayers)
+    {
+	rmask = DBResidueMask(type);
+	if (TTMaskHasType(rmask, ttype))
 	    type = ttype;
+    }
     TTMaskSetOnlyType(&tmask, type);
 
 #endif
