@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/select/selDisplay.c,v 1.2 2008/02/07 05:44:21 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/select/selDisplay.c,v 1.4 2008/12/09 15:08:02 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -91,7 +91,7 @@ SelRedisplay(window, plane)
     CellDef *displayDef;
     Label *label;
     Transform tinv;
-    Rect planeArea, screenArea, fullArea, selArea;
+    Rect planeArea, screenArea, selArea;
     SearchContext scx;
     DBWclientRec *crec = (DBWclientRec *) window->w_clientData;
 
@@ -108,7 +108,16 @@ SelRedisplay(window, plane)
     if (!DBBoundPlane(plane, &planeArea)) return;
     GeoInvertTrans(&selDisUse->cu_transform, &tinv);
     GeoTransRect(&tinv, &planeArea, &selArea);
-    if (!GEO_OVERLAP(&displayDef->cd_bbox, &selArea)) return;
+    if (!GEO_OVERLAP(&displayDef->cd_bbox, &selArea))
+    {
+	/* Check if any labels in the selection overlap */
+	for (label = displayDef->cd_labels; label != NULL; label = label->lab_next)
+	    if (label->lab_font >= 0)
+		if (GEO_OVERLAP(&label->lab_bbox, &selArea))
+		    break;
+
+	if (label == NULL) return;
+    }
 
     /* Redisplay the information on the paint planes. */
 
@@ -125,6 +134,7 @@ SelRedisplay(window, plane)
 
     labelSize = crec->dbw_labelSize;
     if (labelSize < GR_TEXT_SMALL) labelSize = GR_TEXT_SMALL;
+
     for (label = displayDef->cd_labels; label != NULL; label = label->lab_next)
     {
 	Rect larger, tmpr;
@@ -133,19 +143,34 @@ SelRedisplay(window, plane)
 	 * search with a non-null area, or it will never return "yes").
 	 */
 
-	larger = label->lab_rect;
-	if (larger.r_xbot == larger.r_xtop)
-	    larger.r_xtop += 1;
-	if (larger.r_ybot == larger.r_ytop)
-	    larger.r_ytop += 1;
-	if (!DBSrPaintArea((Tile *) NULL, plane, &larger, &DBAllButSpaceBits,
-		selAlways1, (ClientData) NULL))
-	    continue;
-	GeoTransRect(&selDisUse->cu_transform, &label->lab_rect, &tmpr);
-	WindSurfaceToScreen(window, &tmpr, &screenArea);
-	WindSurfaceToScreenNoClip(window, &tmpr, &fullArea);
-	DBWDrawLabel(label, &fullArea, &screenArea, label->lab_just,
-		STYLE_OUTLINEHIGHLIGHTS, labelSize, &crec->dbw_expandAmounts);
+	if (label->lab_font < 0)
+	{
+	    larger = label->lab_rect;
+	    if (larger.r_xbot == larger.r_xtop)
+		larger.r_xtop += 1;
+	    if (larger.r_ybot == larger.r_ytop)
+		larger.r_ytop += 1;
+	    if (!DBSrPaintArea((Tile *) NULL, plane, &larger, &DBAllButSpaceBits,
+			selAlways1, (ClientData) NULL))
+		continue;
+
+	    GeoTransRect(&selDisUse->cu_transform, &label->lab_rect, &tmpr);
+	    WindSurfaceToScreen(window, &tmpr, &screenArea);
+
+	    DBWDrawLabel(label, &screenArea, label->lab_just,
+			STYLE_OUTLINEHIGHLIGHTS, labelSize,
+			&crec->dbw_expandAmounts);
+	}
+	else
+	{
+	    DBWDrawFontLabel(label, window, &selDisUse->cu_transform,
+			STYLE_OUTLINEHIGHLIGHTS);
+
+	    /* Disabled for now because there is no routine to compute bbox  */
+	    // GeoTransRect(&selDisUse->cu_transform, &label->lab_bbox, &tmpr);
+	    // WindSurfaceToScreen(window, &tmpr, &screenArea);
+	    // GeoInclude(&screenArea, &crec->dbw_expandAmounts);
+	}
 	if (SigInterruptPending) break;
     }
 
@@ -309,7 +334,7 @@ selRedisplayCellFunc(scx, window)
 
     /* Don't futz around with text if the bbox is tiny. */
 
-    GrLabelSize("BBB", -1, GEO_CENTER, GR_TEXT_SMALL, &tmp);
+    GrLabelSize("BBB", GEO_CENTER, GR_TEXT_SMALL, &tmp);
     if (((screen.r_xtop-screen.r_xbot) < tmp.r_xtop)
 	|| ((screen.r_ytop-screen.r_ybot) < tmp.r_ytop)) return 0;
 
